/**
 * \file <gre/lrcad.h>
 * \brief GRE::LAYER_CAD definitions
 *
 * \if NODOC
 * $Id: lrcad.h_v 1.45 2012/05/22 20:39:56 mju Exp $
 *
 * $Log: lrcad.h_v $
 * Revision 1.45  2012/05/22 20:39:56  mju
 * Add get/setAutoLabelParms.
 *
 * Revision 1.44  2012/05/07 20:49:31  mju
 * Make dispparm members private.
 *
 * Revision 1.43  2012/05/03 20:12:21  mju
 * Use layer_cad::dispparm::setSelect/StyleMode.
 *
 * Revision 1.42  2012/05/03 20:07:13  mju
 * Use gremode hdr.
 *
 * Revision 1.41  2012/05/03 20:00:17  mju
 * Add dispparm get/setStyleMode.
 *
 * Revision 1.40  2012/05/03 19:53:56  mju
 * Rename some dispparm members.
 *
 * Revision 1.39  2012/05/03 16:51:13  mju
 * Rename dlg member.
 *
 * Revision 1.38  2011/11/04 17:47:59  mju
 * Remove unneeded method.
 *
 * Revision 1.37  2011/08/17 20:49:17  mju
 * Add v_closeObject.
 *
 * Revision 1.36  2010/11/15 18:25:58  mju
 * Add v_snapPoint...
 *
 * Revision 1.35  2010/08/19 17:40:28  mju
 * Use elemnum type.
 *
 * Revision 1.34  2010/04/23 16:54:29  mju
 * Use unsignedset.
 *
 * Revision 1.33  2008/05/15 17:54:51  mju
 * Chg v_constructLayerSection.
 *
 * Revision 1.32  2008/05/12 16:59:35  mju
 * Add parm to constructLayerSection.
 *
 * Revision 1.31  2008/03/17 21:33:16  mju
 * Change v_getLayerView to v_getLayerSection.
 *
 * Revision 1.28  2007/04/13 17:51:46  mju
 * Add method to get menu for region generation.
 *
 * Revision 1.27  2007/03/12 17:42:41  mju
 * Add v_setObjectDescriptor.
 *
 * Revision 1.26  2006/10/10 13:58:54  mju
 * Use miconstruct* instead of micon.
 *
 * Revision 1.25  2006/10/09 17:50:57  mju
 * Rmv v_setDftName.
 *
 * Revision 1.24  2006/10/02 17:47:51  mju
 * DataTip return pointer.
 *
 * Revision 1.23  2006/09/29 14:45:52  scowan
 * Set objitem to its namespace.
 *
 * Revision 1.22  2006/09/28 19:56:20  mju
 * Use rvc objitem hdr.
 *
 * Revision 1.21  2006/09/28 13:21:34  mju
 * Use datatip, lablstyl hdrs.
 *
 * Revision 1.20  2006/09/18 14:17:14  mju
 * Move layer_sketch to own header.
 *
 * Revision 1.19  2006/09/15 15:30:16  mju
 * Serializer changes.
 *
 * Revision 1.18  2006/09/14 19:11:56  mju
 * Use ptr for dbedit.
 *
 * Revision 1.17  2006/04/19 20:36:12  mju
 * Add cadstdattr.
 *
 * Revision 1.16  2006/02/20 22:14:45  mju
 * Use mistring for querystr in dispparm.
 *
 * Revision 1.15  2005/11/28 16:38:14  mju
 * Treeview legend update.
 *
 * Revision 1.14  2005/11/04 18:52:32  mju
 * enums in gre namespace.
 *
 * Revision 1.13  2005/10/26 20:44:14  mju
 * Use gre namespace instead of legacy types.
 *
 * Revision 1.12  2005/10/18 13:23:00  mju
 * Fix file doc comments.
 *
 * Revision 1.11  2005/10/14 17:31:07  mju
 * GRE namespace and treeview.
 *
 * Revision 1.10  2005/03/31 16:57:06  fileserver!dwilliss
 * Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 * Revision 1.9  2005/02/28 18:00:59  scowan
 * Support layer copy virtual method.
 *
 * Revision 1.8  2005/02/25 23:20:51  scowan
 * Added copy virtual method implementation.
 *
 * Revision 1.7  2005/02/23 18:54:25  mju
 * Moved labelplacement to layerview_labeled.
 *
 * Revision 1.6  2005/01/20 16:56:12  mju
 * Redesign layer type registration.
 *
 * Revision 1.5  2004/10/01 17:19:48  mju
 * Add dlgparent to assignObject.
 *
 * Revision 1.4  2004/09/08 20:23:04  dwilliss
 * Implement LAYERPLACEMENT stuff
 *
 * Revision 1.3  2004/07/12 23:13:44  mju
 * Use region2d.
 *
 * Revision 1.2  2004/06/16 21:54:25  scowan
 * Significant changes to cad layer and dispparm definitions.
 *
 * Revision 1.1  2004/06/04 20:50:00  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_LRCAD_H
#define	INC_GRE_LRCAD_H

#ifndef	INC_GRE_LAYER_H
#include <gre/layer.h>
#endif

#ifndef	INC_GRE_DATATIP_H
#include <gre/datatip.h>
#endif

#ifndef	INC_GRE_ELEMPARMS_H
#include <gre/elemparms.h>
#endif

#ifndef	INC_RVC_OBJITEM_H
#include <rvc/objitem.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
namespace RVC {
	class CAD;
	class CADELEMENT;
	}
namespace SML {
	class CLASSMEMBER;
	}
#endif

namespace GRE {
//===================================================================================================================

//! GRE layer class for viewing CAD spatial object.
class LAYER_CAD : public LAYER {
	public:

		// LAYER_CAD::DISPPARM
		class DISPPARM : public LAYER::DISPPARM {
			public:

				class DLG;

				//! Default constructor.
				DISPPARM ();

				//! Copy constructor.
				DISPPARM (const DISPPARM& rhs);

				//! Destructor.
				virtual ~DISPPARM ();

				//! Assignment.
				DISPPARM& operator= (const DISPPARM& rhs);

				void ClearGeorefName ()
					{ m_GeorefName.Clear(); }

				const AUTOLABELPARMS& GetAutoLabelParms () const
					{ return (m_AutoLabel); }

				INT32 GetBlockNum () const
					{ return (m_BlockNum); }

				const DATATIPPARMS& GetDataTip () const
					{ return (m_DataTip); }

				const RVC::OBJECTNAME& GetGeorefName () const
					{ return (m_GeorefName); }

				const LINESTYLE& GetLabelBaseStyle () const
					{ return (m_LabelBaseStyle); }

				bool GetLabelClipUnder () const
					{ return (m_EnableLabelClipUnder); }

				const LABELFRAMEPARMS& GetLabelFrame () const
					{ return (m_LabelFrame); }

				bool GetLabelShowBase () const
					{ return (m_ShowLabelBase); }

				const RVC::OBJITEM& GetObjItem () const
					{ return (m_ObjItem); }

				const RVC::OBJITEM& GetObjItemStyle () const
					{ return (m_ObjItemStyle); }

				SELECTMODE GetSelectMode () const
					{ return (static_cast<SELECTMODE>(m_SelectMode)); }

				const MISTRING& GetSelectQueryStr () const
					{ return (m_SelectQueryStr); }

				INT16 GetSelectTableNum () const
					{ return (m_SelectTable); }

				const MULTISTYLE& GetStyleAllSame () const
					{ return (m_AllSame); }

				STYLEMODE GetStyleMode () const
					{ return (static_cast<STYLEMODE>(m_StyleMode)); }

				const MISTRING& GetStyleQueryStr () const
					{ return (m_StyleQueryStr); }

				INT16 GetStyleTableNum () const
					{ return (m_StyleTable); }

				bool IsElemTypeSelected (
					UINT32 cadelemtype
					) const;

				void SetDataTip (
					const DATATIPPARMS DataTip
					) { m_DataTip = DataTip; }

				void SetGeorefName (
					const RVC::OBJECTNAME& GeorefName
					) { m_GeorefName = GeorefName; }

				//! Set to use specified object.
				ERRVALUE SetObject (
					const RVC::OBJITEM& objitem,		//!< Object to set
					bool keepsettings = false,			//!< Retain current display settings
					RVC::DISPPARMUSAGE dpusage = RVC::DISPPARMUSAGE_Display
					);

				void SetObjectDescriptor (
					const RVC::DESCRIPTOR descriptor
					) { m_ObjItem.SetObjectDescriptor(descriptor); }

				void SetObjItemStyle (
					const RVC::OBJITEM& ObjItemStyle
					);

				void SetSaveObjItems (
					const RVC::OBJITEM& ObjItemCAD,
					const RVC::OBJITEM& ObjItemStyle
					) { m_SaveObjItem = ObjItemCAD; m_SaveObjItemStyle = ObjItemStyle; m_HasSaveInfo = true; }

				void SetSelectMode (
					SELECTMODE SelectMode
					) { m_SelectMode = static_cast<UINT16>(SelectMode); }

				void SetStyleAllSame (
					const MULTISTYLE& style
					) { m_AllSame = style; }

				void SetStyleAllSame (
					const POINTSTYLE& PointStyle
					) { m_AllSame.Point = PointStyle; }

				void SetStyleAllSame (
					const LINESTYLE& LineStyle
					) { m_AllSame.Line = LineStyle; }

				void SetStyleAllSame (
					const POLYSTYLE& PolyStyle
					) { m_AllSame.Poly = PolyStyle; }

				void SetStyleAllSame (
					const TEXTSTYLE& TextStyle
					) { m_AllSame.Text = TextStyle; }

				void SetStyleMode (
					STYLEMODE StyleMode
					) { m_StyleMode = static_cast<UINT16>(StyleMode); }

				void TransferTextStyleToElement (
					RVC::CADELEMENT cadelement
					) const;

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				typedef LAYER_CAD::DISPPARM THISCLASS;

				RVC::OBJITEM m_ObjItem;
				RVC::OBJECTNAME m_GeorefName;		// Name of georeference object
				RVC::OBJITEM m_ObjItemStyle;
				MULTISTYLE m_AllSame;				// Style to use if StyleMode == STYLEMODE_AllSame
				INT32 m_BlockNum;						// Number of block to display
				UINT16 m_SelectMode;					// Element selection mode - SELECTMODE_xxxxx...
				UINT16 m_StyleMode;					// Element style mode - STYLEMODE_xxxxx...
				UINT8 m_CADElemSelect[4];
				INT16 m_SelectTable;					// "Style assignment" table number for CAD element selection
				INT16 m_StyleTable;					// "Style assignment" table number for CAD element styling
				MISTRING m_SelectQueryStr;			// Query string for element selection
				MISTRING m_StyleQueryStr;			// Query string for element styling
				DATATIPPARMS m_DataTip;				// DataTip parameters
				AUTOLABELPARMS m_AutoLabel;
				bool m_HasSaveInfo;
				bool m_EnableLabelClipUnder;
				bool m_ShowLabelBase;
				LINESTYLE m_LabelBaseStyle;
				LABELFRAMEPARMS m_LabelFrame;
				mutable RVC::OBJITEM m_SaveObjItem;
				mutable RVC::OBJITEM m_SaveObjItemStyle;


				static ERRVALUE SerializerCB_StyleFileName (SERIALIZER&, const SERIALIZERITEM *, void *, SERIALIZERITEM::ACTION);

				void Copy (const DISPPARM&);
				void Free ();
				void UpdateSaveInfo () const;

				// Overrides from SERIALIZABLE.
				virtual ERRVALUE SerialRead (SERIALIZER& serializer);
				virtual ERRVALUE SerialWrite (SERIALIZER& serializer, const char *tagname = 0) const;
				
				// Overrides from LAYER::DISPPARM.
				virtual const SERIALIZERITEM* SerialGetItemDef (SERIALIZER& serializer) const;
				virtual ERRVALUE v_SetObjItem (const RVC::OBJITEM& objitem, bool replace);
				virtual ERRVALUE v_Validate ();

				#endif // GENERATING_DOXYGEN_OUTPUT
			};	// End of LAYER_CAD::DISPPARM

		//! Register this layer type
		static void RegisterType ();

		//! Install SML class, called by internal SML function.
		static void SMLClassInstall (
			SMLCONTEXT *context
			);

		//! Construct layer and insert into list
		explicit LAYER_CAD (
			GRE::GROUP *group,										//!< Group to contain layer
			CREATEFLAGS createflags = CREATEFLAG_None,	//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,		//!< Position in list
			LAYER *reflayer = 0								//!< Reference layer for insertion
			);

		virtual ~LAYER_CAD ();

		//! Retrieve layer-specific display parameters
		const DISPPARM& GetDispParm (
			) const { return (m_dispparm); }

		//! Set layer-specific display parameters.
		//!
		//! This is the recommended method for changing how an existing layer is to be
		//! rendered.  Validation will be performed on the specified parameters and
		//! adjustments made to the layer's copy if necessary.  Use of this method
		//! usually result in a ChangeBegin / ChangeEnd message pair, and will also
		//! generate a ChangeObject message if the spatial object used in the layer
		//! is changed.
		ERRVALUE SetDispParm (
			const DISPPARM& dispparm					//!< Parameters to set
			);

	protected:

		DISPPARM m_dispparm;								//!< Layer-specific display parameters
		COMPINFO m_ElemCompInfo;

		virtual ERRVALUE v_AssignObject (const RVC::OBJITEM& objitem, MDLGPARENT dlgparent);
		virtual void v_CloseObject ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		
		#ifdef USE_MGUI
			DISPPARM::DLG *m_pDlgControlPanel;
		#endif
		RVC::CAD *m_pObjCad;
		GRE::GEOMETRIC3D *m_pGeometric3D;
		GRE::WIREFRAME *m_pWireframe;
		
		class PRIVDRAWDATA;

		bool CanDraw3dDirect (const LAYERDC& layerdc) const;
		ERRVALUE Draw2D (LAYERDC& layerdc);
		ERRVALUE Draw3D (LAYERDC& layerdc);
		ERRVALUE DrawWireframe (LAYERDC& layerdc);
		ERRVALUE OpenPrivateObject () const;
		int SetupRegion ();
		ERRVALUE WireReadCAD (LAYERDC& layerdc);

		//! Overrides from LAYER.
		virtual bool v_CanShowDetails () const;
		virtual GRE::LAYERSECTION* v_ConstructLayerSection (GRE::VIEW* view);
	#ifdef USE_MGUI
		virtual int v_ControlPanel (MDLGPARENT dlgparent,UINT32 flags = 0);
	#endif
		virtual LAYER* v_Copy (GRE::GROUP *group) const;
		virtual const DATATIPPARMS* v_DataTipGetParms (int component) const;
		virtual ERRVALUE v_DataTipSetParms (int component, const DATATIPPARMS& DataTipParms);
		virtual INT32 v_DbListElemNumToIndex (int component, ELEMNUM ElemNum);
		virtual ERRVALUE v_Draw (LAYERDC& layerdc);
		virtual ERRVALUE v_DrawElementLow (LAYERDC& layerdc, ELEMTYPE ElemType, ELEMNUM ElemNum, const void *style);
		virtual ERRVALUE v_DrawElementsLow (LAYERDC& layerdc, ELEMTYPE ElemType, const UNSIGNEDSET *pElemSet, const void *style);
		virtual ELEMNUM v_FindClosestElement (const DPOINT2D& point, double searchdist, double *distret, int component, const UNSIGNEDSET *pSearchSet, GRE::FINDFLAGS findflags);
		virtual int v_GetComponent (ELEMTYPE ElemType) const;
		virtual MICONSTRUCT* v_GetComponentIcon (int component) const;
		virtual COMPINFO* v_GetComponentInfo (int component);
		virtual MISTRING v_GetComponentName (int component) const;
		virtual const void* v_GetElemDefaultStyle (ELEMTYPE ElemType) const;
		virtual UINT16 v_GetElemDrawSelectionMode (ELEMTYPE ElemType) const;
		virtual UINT16 v_GetElemDrawStyleMode (ELEMTYPE ElemType) const;
		virtual ERRVALUE v_GetElemExtents (ELEMTYPE ElemType, ELEMNUM ElemNum, DRECT3D& Extents);
		virtual ERRVALUE v_GetElemSetByExtent (ELEMTYPE ElemType, const DRECT2D& Extent, UNSIGNEDSET& ElemSet);
		virtual ELEMTYPE v_GetElemType (int component) const;
		virtual bool v_GetGeorefName (RVC::OBJECTNAME& name, int component = 0) const;
		virtual bool v_GetLabelColor (LAYERDC&, const GRE::LABELPLACE&, COLOR&);
		virtual INT64 v_GetNumElements (ELEMTYPE ElemType);
		virtual const RVC::OBJITEM& v_GetObjItem (int component) const;
		virtual ERRVALUE v_GetStyleInfo (ELEMTYPE ElemType, RVC::OBJITEM& ObjItem, RVC::OBJECTNAME& TableName);
	#ifdef USE_MGUI
		virtual void v_MenuAddRegionGenItems (MGUI::MENU& menu, ELEMTYPE ElemType);
	#endif
	#ifdef USE_MGUI
		virtual ERRVALUE v_PromptObject (bool OpenCtrlPanel, MDLGPARENT dlgparent);
	#endif
		virtual ERRVALUE v_ReportRawElemInfo (MISTRING& infostr, ELEMTYPE ElemType, ELEMNUM ElemNum);
		virtual const char* v_SerialGetTagName () const;
		virtual void v_SetElemDrawSelectionMode (ELEMTYPE ElemType, UINT16 SelectMode);
		virtual void v_SetElemDrawStyleMode (ELEMTYPE ElemType, UINT16 StyleMode);
		virtual void v_SetObjectDescriptor (int component, const RVC::DESCRIPTOR& descriptor);
		virtual ERRVALUE v_SetStyleObjItem (const RVC::OBJITEM& ObjItem);
		virtual int v_SnapPoint (DPOINT3D& ptObject, SNAPTO SnapTo, double MaxDistance, VIEW *view) const;
		virtual bool v_SnapPointIsAllowed (SNAPTO SnapTo) const;
		virtual int v_TestElementByRegion (ELEMTYPE ElemType, ELEMNUM ElemNum, const REGION2D& Region, TESTCRITERIA TestMode);
	#ifdef USE_MGUI
		virtual void v_TreeViewAddUpdateDetails (GRE::CTRL_TREEVIEW& TreeView, MGUI::CTRL_TREE::ITEMHND hndParent, MGUI::CTRL_TREE::ITEMHND hndInsertAfter);
		virtual MGUI::CTRL_TREE::ITEMHND v_TreeViewAddUpdateLegend (GRE::CTRL_TREEVIEW& TreeView, MGUI::CTRL_TREE::ITEMHND hndParent, MGUI::CTRL_TREE::ITEMHND hndInsertAfter);
	#endif
		virtual int v_UpdateExtents (bool forceupdate);

		LAYER_CAD (const LAYER_CAD&);
		LAYER_CAD& operator= (const LAYER_CAD&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//===================================================================================================================
}	// End namespace GRE

#endif	// INC_GRE_LRCAD_H
