/**
 * \file gre/lrraster.h
 * \brief GRE Raster Layer definitions
 *
 * \if NODOC
 * $Id: lrraster.h_v 1.83 2013/05/28 14:55:52 mju Exp $
 *
 * $Log: lrraster.h_v $
 * Revision 1.83  2013/05/28 14:55:52  mju
 * Add mcb_EditContrast.
 *
 * Revision 1.82  2013/02/14 20:36:53  mju
 * Undo previous.
 *
 * Revision 1.81  2013/02/14 20:11:46  mju
 * Make getCommonParm non-const as getGroup returns non-const.
 *
 * Revision 1.80  2012/12/11 21:19:56  mju
 * Add dispparm members for outlining contiguous areas.
 *
 * Revision 1.79  2012/09/18 13:46:51  mju
 * Attempt dox fix.
 *
 * Revision 1.78  2012/06/11 17:35:49  mju
 * Add dispparm::getNumColorComponents.
 *
 * Revision 1.77  2012/06/11 15:42:44  mju
 * Make ioparm::getRasterComponent public.
 *
 * Revision 1.76  2012/06/08 16:33:56  mju
 * Add more dispparm accessors.
 *
 * Revision 1.75  2012/06/07 21:25:38  mju
 * Add more dispparm accessor methods.
 *
 * Revision 1.74  2012/06/07 17:45:00  mju
 * Make all ioparm members private.
 * Change ioparm serialization so can save contrast settings for rgb24 and user-defined.
 *
 * Revision 1.73  2012/06/06 16:20:18  mju
 * Add ioparm accessor methods for all members that don't already have.
 *
 * Revision 1.72  2011/10/20 13:05:20  mju
 * Remove redundant hdrs.
 *
 * Revision 1.71  2011/10/06 15:22:31  mju
 * Remove dependencies, make textureMode uint8.
 *
 * Revision 1.70  2011/09/30 17:59:08  mju
 * Remove canDrawTiled.
 *
 * Revision 1.69  2011/09/30 14:57:36  mju
 * Move shadeparm to nest under layer_raster.
 *
 * Revision 1.68  2011/09/20 17:58:56  mju
 * Add updateContrastFullArea.
 *
 * Revision 1.67  2011/09/19 16:41:33  mju
 * Add ioparm::m_componentNum.
 *
 * Revision 1.66  2011/09/16 21:28:03  mju
 * Remove stuff for shading dialog.
 *
 * Revision 1.65  2011/09/15 21:30:47  mju
 * Add pDlgControlPanel.
 *
 * Revision 1.64  2010/10/19 21:27:09  mju
 * Make setupInitialPipelineStages public.
 *
 * Revision 1.63  2010/08/19 17:40:38  mju
 * Use elemnum type.
 *
 * Revision 1.62  2010/04/23 16:54:46  mju
 * Use unsignedset.
 *
 * Revision 1.61  2009/08/10 16:42:52  mju
 * Add v_getScaleMaxDetail.
 *
 * Revision 1.60  2009/06/23 17:47:53  mju
 * Add dispmode parm to ioparm contrast methods.
 *
 * Revision 1.59  2009/03/16 17:43:05  mju
 * Add setMaskNone.
 *
 * Revision 1.58  2008/08/07 14:50:00  mju
 * Rename texture member.
 *
 * Revision 1.57  2008/07/14 17:58:50  mju
 * Add v_canDrawAsTiles.
 *
 * Revision 1.56  2007/11/19 22:05:43  bcolombe
 * Added support for raster spatial filtering
 *
 * Revision 1.55  2007/10/10 20:32:25  mju
 * Add dispparm fields for enlarge/reduce methods.
 *
 * Revision 1.54  2007/10/10 17:49:34  mju
 * Add dispparm.m_cellLabelPlaces.
 *
 * Revision 1.53  2007/09/14 21:47:04  mju
 * Add more commonparms values.
 *
 * Revision 1.52  2007/09/11 14:02:59  mju
 * Add getStats.
 *
 * Revision 1.51  2007/09/10 20:57:17  mju
 * Keep stats in ioparm.
 * Add ioparm.getNumType.
 *
 * Revision 1.50  2007/09/10 17:49:20  mju
 * Add commonparms stuff.
 *
 * Revision 1.49  2007/09/06 21:56:46  mju
 * Add commonparm support.
 *
 * Revision 1.48  2007/08/15 14:30:57  mju
 * Add ioparm.getRasterComponent.
 *
 * Revision 1.47  2007/08/07 23:24:17  honerkamp
 * I forget...  did something for the raster theme mapping
 *
 * Revision 1.46  2007/08/06 13:22:57  mju
 * Add blendmode enum.
 *
 * Revision 1.45  2007/08/02 21:38:07  dwilliss
 * added method to get list of SML class members
 *
 * Revision 1.44  2007/07/05 22:52:04  honerkamp
 * Added a themeparmname variable
 *
 * Revision 1.43  2007/07/05 16:10:59  honerkamp
 * Added themeparm variables and functions
 *
 * Revision 1.42  2007/06/26 17:21:56  mju
 * Add ioparm.validateDataTip.
 *
 * Revision 1.41  2007/06/08 17:36:13  mju
 * Add legendtype_thumbnail.
 *
 * Revision 1.40  2007/04/09 17:48:25  dwilliss
 * Changed printer gamma structure.
 *
 * Revision 1.39  2007/03/30 17:47:47  mju
 * Use contrastparm.
 *
 * Revision 1.38  2007/03/12 17:42:46  mju
 * Add v_setObjctDescriptor.
 *
 * Revision 1.37  2007/02/06 18:04:06  mju
 * Add closePrivateObjects.
 *
 * Revision 1.36  2007/01/19 16:08:34  mju
 * Use color_array.
 *
 * Revision 1.35  2007/01/04 16:00:29  mju
 * Add parm to specify georeference when setup pipeline stages.
 *
 * Revision 1.34  2006/12/06 21:04:08  dwilliss
 * Added a parameter to v_DataTipGetString
 *
 * Revision 1.33  2006/11/21 16:37:24  mju
 * Add useValidity to setupStage.
 *
 * Revision 1.32  2006/11/20 16:43:36  mju
 * Exclude dlgGetObjects unless use_mgui.
 *
 * Revision 1.31  2006/11/20 15:40:48  mju
 * Add ioparm.validateObjItem.
 *
 * Revision 1.30  2006/11/15 22:26:35  mju
 * Major redesign to use image pipeline and eliminate legacy RVC usages.
 *
 * Revision 1.25  2006/10/02 17:48:09  mju
 * DataTip parms return as pointer.
 *
 * Revision 1.24  2006/09/28 13:16:42  mju
 * Use gre legendparms, datatip hdrs.
 *
 * Revision 1.23  2006/09/15 15:30:40  mju
 * Serializer changes.
 *
 * Revision 1.22  2006/09/07 21:26:14  mju
 * Remove opemode from openObject and prefix with v_
 *
 * Revision 1.21  2006/08/23 17:16:03  mju
 * Chg parms to setupPyramid.
 *
 * Revision 1.20  2006/06/23 16:32:14  mju
 * Declare mxdAnyCallbackStruct.
 *
 * Revision 1.18  2005/11/28 16:38:19  mju
 * Treeview legend update.
 *
 * Revision 1.17  2005/11/04 18:52:41  mju
 * enums in gre namespace.
 *
 * Revision 1.16  2005/10/19 20:17:48  mju
 * Use gre namespace instead of legacy typedefs.
 *
 * Revision 1.15  2005/10/14 17:31:39  mju
 * GRE namespace and treeview.
 *
 * Revision 1.14  2005/03/31 16:57:06  fileserver!dwilliss
 * Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 * Revision 1.13  2005/03/23 22:04:48  mju
 * Add v_closeObject.
 *
 * Revision 1.12  2005/03/01 14:22:14  mju
 * Add v_copy.
 *
 * Revision 1.11  2005/02/15 17:50:29  scowan
 * No real changes.
 *
 * Revision 1.10  2005/01/20 16:57:05  mju
 * Redesign layer type registration.
 *
 * Revision 1.9  2004/10/11 17:17:08  mju
 * Add nullmask instance to ioparm.
 *
 * Revision 1.8  2004/10/01 17:20:01  mju
 * Add dlgparent to assignObject.
 *
 * Revision 1.7  2004/09/20 17:38:17  vdronov
 * added fence and pedestal bound structures
 *
 * Revision 1.6  2004/09/16 22:25:31  vdronov
 * added BOUNDPARM
 *
 * Revision 1.5  2004/08/12 19:37:57  vdronov
 * added ClearPedestal method
 *
 * Revision 1.4  2004/06/30 13:29:12  mju
 * Add setobjregion.
 *
 * Revision 1.3  2004/06/28 17:53:52  mju
 * Convert to spatref system.
 *
 * Revision 1.2  2004/03/05 15:12:50  mju
 * Use virtual dtor for dispparm.
 *
 * Revision 1.1  2004/03/04 18:57:31  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_LRRASTER_H
#define	INC_GRE_LRRASTER_H

#ifndef	INC_GRE_LRIMAGE_H
	#include <gre/lrimage.h>
#endif

#ifndef	INC_GRE_LEGENDPARMS_H
	#include <gre/legendparms.h>
#endif

#ifndef	INC_GRE_DATATIP_H
	#include <gre/datatip.h>
#endif

#ifndef	INC_GRE_3DBOUND_H
	#include <gre/3dbound.h>
#endif

#ifndef	INC_MI32_CONTRASTPARM_H
	#include <mi32/contrastparm.h>
#endif

#ifndef	INC_MI32_COLORBAL_H
	#include <mi32/colorbal.h>
#endif

#ifndef	INC_MI32_COLORARRAY_H
	#include <mi32/colorarray.h>
#endif

#ifndef	INC_RVC_RASTERTHEMEPARM_H
	#include <rvc/rasterthemeparm.h>
#endif

#ifndef	INC_MI32_STATUNIVAR_H
	#include <mi32/statunivar.h>
#endif

#ifndef INC_MI32_RASTERFILTERPARMS_H
	#include <mi32/rasterfilterparms.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
struct McbAnyCallbackStruct;
struct MxdAnyCallbackStruct;
namespace MGD {
	class DEVICE_MEM24;
	}
namespace RVC {
	class RASTER;
	class NULLMASK;
	}

namespace SPATMOD {
	class GEOREFERENCE;
	namespace IMAGE{
		class SAMPLEITERATOR;
		}
	}
#endif


namespace GRE {
//===================================================================================================================

#define	RASTGRADMIN		  -127
#define	RASTGRADMAX			127
#define	RASTGRADOFFSET		128
#define	RASTGRADRANGE		256

//! Raster Layer class
class LAYER_RASTER : public GRE::LAYER_IMAGE {
	public:

		enum SUBTYPE {
			SUBTYPE_Single =				0,
			SUBTYPE_Hyperspectral =		1,
			SUBTYPE_MultiCI =				2,
			SUBTYPE_MultiRGB =			3,
			SUBTYPE_MultiHIS =			4,
			SUBTYPE_MultiHBS =			5,
			SUBTYPE_MultiRGBI =			6,
			SUBTYPE_MultiCMY =			7,
			SUBTYPE_MultiCMYK =			8,
			SUBTYPE_External =			9
			};

		enum COLORBLEND {
			COLORBLEND_RGBI_HIS =		1,
			COLORBLEND_RGBI_HBS =		2,
			COLORBLEND_RGBI_Ratio =		3,
			COLORBLEND_RGBI_Brovey =	4,
			COLORBLEND_RGBI_LAST =		4
			};

		//! Shading parameters.
		struct SHADEPARM {
			double AzimAngle;
			double ElevAngle;
			double ZScaling;
			UINT8 m_ShadeMethod;			//!< SPATMOD::IMAGE::FILTER_FOCALTOPOGRAPHIC::SHADING_METHOD
			UINT8 m_SurfaceFitMethod;	//!< SPATMOD::IMAGE::FILTER_FOCALTOPOGRAPHIC::SURFACE_FITTING_METHOD
			UINT8 spare[6];

			static const SERIALIZERITEM* SerialGetItemDef ();

			SHADEPARM (
				) : AzimAngle(45*DEGTORAD), ElevAngle(45*DEGTORAD), ZScaling(1.0), m_ShadeMethod(1), m_SurfaceFitMethod(0) { }
			};

		// LAYER_RASTER::COMMONPARM
		class COMMONPARM : public LAYER::COMMONPARM {
			public:

				//! Constructor.
				explicit COMMONPARM (
					GRE::GROUP *group
					);

				~COMMONPARM ();

				//! Determine if can use contrast parameters.
				//! Automatically determined based on raster layers present.
				bool CanUseContrastParm (
					) const { return (m_CanUseContrastParm); }

				//! Determine if can use shading parameters.
				//! Automatically determined based on raster layers present.
				bool CanUseShadingParm (
					) const { return (m_CanUseShadingParm); }

				//! Get contrast parameters.
				const CONTRASTPARM& GetContrastParm (
					int component
					) const { return (m_ContrastParm[component]); }

				//! Get whether to render "black" as transparent.
				STATE GetRenderBlackTransparent (
					) const { return (m_RenderBlackTransparent); }

				//! Get whether to render "null" as transparent.
				STATE GetRenderNullTransparent (
					) const { return (m_RenderNullTransparent); }

				//! Get whether to render "white" as transparent.
				STATE GetRenderWhiteTransparent (
					) const { return (m_RenderWhiteTransparent); }

				//! Get relief shading parameters.
				const SHADEPARM& GetShadingParm (
					) const { return (m_ShadingParm); }

				//! Get whether to use contrast parameters.
				bool GetUseContrastParm (
					) const { return (m_CanUseContrastParm && m_UseContrastParm); }

				//! Get whether to use shading parameters.
				bool GetUseShadingParm (
					) const { return (m_CanUseShadingParm && m_UseShadingParm); }

				//! Set contrast parameters.
				void SetContrastParm (
					int component,
					const CONTRASTPARM& ContrastParm
					);

				//! Get whether to render "black" as transparent.
				void SetRenderBlackTransparent (
					STATE RenderBlackTransparent
					) { m_RenderBlackTransparent = RenderBlackTransparent; }

				//! Get whether to render "null" as transparent.
				void SetRenderNullTransparent (
					STATE RenderNullTransparent
					) { m_RenderNullTransparent = RenderNullTransparent; }

				//! Get whether to render "white" as transparent.
				void SetRenderWhiteTransparent (
					STATE RenderWhiteTransparent
					) { m_RenderWhiteTransparent = RenderWhiteTransparent; }

				//! Set relief shading parameters.
				void SetShadingParm (
					const SHADEPARM& ShadingParm
					) { m_ShadingParm = ShadingParm; }

				//! Set whether to use contrast parameters.
				void SetUseContrastParm (
					bool UseContrastParm
					) { m_UseContrastParm = UseContrastParm; }

				//! Set whether to use shading parameters.
				void SetUseShadingParm (
					bool UseShadingParm
					) { m_UseShadingParm = UseShadingParm; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT

				typedef LAYER_RASTER::COMMONPARM THISCLASS;

				CONTRASTPARM m_ContrastParm[4];			// Contrast parameters
				SHADEPARM m_ShadingParm;				// Shading parameters
				bool m_CanUseContrastParm;
				bool m_CanUseShadingParm;
				bool m_IsContrastAuto;
				bool m_UseContrastParm;
				bool m_UseShadingParm;
				STATE m_RenderNullTransparent;
				STATE m_RenderBlackTransparent;
				STATE m_RenderWhiteTransparent;
				bool m_HasChangedLayers;

				static void GRECB_Group (const GRE::MSG *msg, void *vpCommonParm);
				void OnGreMsg (const GRE::MSG& msg);

				// Overridables from LAYER::COMMONPARM.
			#ifdef USE_MGUI
				virtual FORM* v_ConstructForm () const;
			#endif
				virtual const SERIALIZERITEM* v_SerialGetItemDef () const;
				virtual const char* v_SerialGetTagName () const;

				// Unimplemented
				COMMONPARM (const COMMONPARM& rhs);
				COMMONPARM& operator= (const COMMONPARM& rhs);

				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		// LAYER_RASTER::DISPPARM
		class DISPPARM : public LAYER::DISPPARM {
			public:

				class DLG;

				// LAYER_RASTER::DISPPARM::IOPARM
				struct IOPARM {
					public:

						static const SERIALIZERITEM* SerialGetItemDef ();
						static const SERIALIZERITEM* SerialGetItemDefMask ();

						IOPARM ();

						IOPARM (
							const IOPARM& rhs
							) { Copy(rhs); }

						~IOPARM ();

						//! Assignment
						IOPARM& operator= (
							const IOPARM& rhs
							) { if (this != &rhs) { Copy(rhs); } return (*this); }

						//! Access datatip parameters for direct update.
						DATATIPPARMS& AccessDataTip ()
							{ return (m_DataTip); }

						//! Access shading parameters for direct update.
						SHADEPARM& AccessShadeParm ()
							{ return (m_ShadeParm); }

						INT32 GetBand (
							) const { return (m_Band); }

						//! Determine if can do contrast enhancement.
						bool CanContrast () const;

						//! Clear contrast settings.
						void ClearContrast ();

						//! Clear georeference.
						void ClearGeoreference (
							) { m_ObjNameGeoref.Clear(); }

						//! Get RVC::IMAGE::CELLTYPE.
						UINT32 GetCellType (
							) const { return (m_CellType); }

						//! Get component for complex raster.
						UINT8 GetComponentComplex (
							) const { return (m_ComponentComplex); }

						//! Get contrast parameters.
						const CONTRASTPARM& GetContrastParm (
							) const { return (m_ContrastParm); }

						//! Get contrast parameters (non-const).
						//! For internal layer implementation only.
						CONTRASTPARM& GetContrastParmInternal (
							) { return (m_ContrastParm); }

						//! Get datatip parameters.
						const DATATIPPARMS& GetDataTip () const
							{ return (m_DataTip); }

						bool GetDoContrast () const
							{ return (m_DoContrast); }

						bool GetDoShading () const
							{ return (m_DoShading); }

						//! Get name of contrast subobject.
						const RVC::OBJECTNAME& GetObjNameContrast (
							) const { return (m_ObjNameContrast); }

						//! Get name of contrast subobject.
						RVC::OBJECTNAME& GetObjNameContrast (
							) { return (m_ObjNameContrast); }

						//! Get name of georeference subobject.
						const RVC::OBJECTNAME& GetObjNameGeoreference (
							) const { return (m_ObjNameGeoref); }

						//! Get number of bits per pixel.
						UINT16 GetNumBits () const;

						//! Get number of image columns.
						INT32 GetNumColumns (
							) const { return (m_NumColumns); }

						//! Get number of image rows.
						INT32 GetNumRows (
							) const { return (m_NumRows); }

						NUMTYPE GetNumType () const;

						//! Get currently set OBJITEM.
						const RVC::OBJITEM& GetObjItem (
							) const { return (m_ObjItem); }

						//! Access currently set OBJITEM.
						RVC::OBJITEM& GetObjItem (
							) { return (m_ObjItem); }

						//! Get cell value range.
						ERRVALUE GetRange (
							DOUBLE_RANGE& range
							) const;

						//! Get raster component.
						//! @return RVC::RASTER::COMPONENT enum value as UINT32.
						UINT32 GetRasterComponent () const;

						//! Get shading parameters.
						const SHADEPARM& GetShadeParm () const
							{ return (m_ShadeParm); }

						//! Get statistics for component.
						const STAT_UNIVARIATE& GetStats (
							) const { return (m_Stats); }

						//! Determine if is constant-value image.
						bool IsConstant (
							) const { return (m_IsConstant); }

						//! Determine if need contrast enhancement.
						bool NeedContrast (
							UINT8 dispmode
							) const;

						//! \internal
						void OnSerialRead();

						ERRVALUE SerialWrite (
							SERIALIZER& serializer,
							const char *tagname,
							const DISPPARM& dispparm
							) const;

						void SetBand (
							INT32 band
							) { m_Band = band; }

						//! Set component for complex raster.
						void SetComponentComplex (
							UINT8 ComponentComplex
							) { m_ComponentComplex = ComponentComplex; }

						//! \internal
						void SetComponentNum (
							INT8 ComponentNum
							) { m_ComponentNum = ComponentNum; }

						//! Set contrast to use specified object.
						void SetContrast (
							const RVC::OBJECTNAME& ObjNameContrast
							);

						//! Set contrast to use default settings.
						void SetContrastDefault (
							UINT8 dispmode
							);

						//! Set contrast parameters.
						void SetContrastParm (
							const CONTRASTPARM& ContrastParm
							);

						//! Set datatip parameters.
						void SetDataTip (
							const DATATIPPARMS& DataTip
							) { m_DataTip = DataTip; }

						//! Set DataTip to use default settings.
						void SetDataTipDefault (
							bool force
							);

						//! Set dimensions of "constant" image.
						//! Primarily for internal use only.
						void SetDimensions (
							INT32 NumColumns,
							INT32 NumRows
							) { if (m_IsConstant) { m_NumColumns = NumColumns; m_NumRows = NumRows; } }

						void SetDoContrast (
							bool doContrast
							) { m_DoContrast = doContrast; }

						void SetDoShading (
							bool doShading
							) { m_DoShading = doShading; }

						void SetGeoreference (
							const RVC::OBJECTNAME& ObjNameGeoref
							) { m_ObjNameGeoref = ObjNameGeoref; }

						//! Set OBJITEM.
						//! If item is unassigned then will set as "constant", otherwise it
						//! must refer to an "image" object or error will be returned.
						ERRVALUE SetObjItem (
							const RVC::OBJITEM& ObjItem,
							bool UpdateSaved
							);

						//! Set OBJITEM to actually save with serialization.
						//! Used when "editing" in temporary workspace.
						//! If not used then will save normal objitem.
						void SetSaveObjItem (
							const RVC::OBJITEM& ObjItem
							);

						//! Set shading parameters.
						void SetShadeParm (
							const SHADEPARM& ShadeParm
							) { m_ShadeParm = ShadeParm; }

						ERRVALUE SetupImageStage (
							SPATMOD::IMAGE::STAGE_AUTOPTR& StagePtr,
							const SPATMOD::GEOREFERENCE *pGeoreference,
							const GRE::LAYER_RASTER::COMMONPARM *pCommonParm,
							int component,
							bool UseValidity = true
							) const;

						void UpdateSaveInfo() const;

						//! Validate settings.
						//! Checks if various object-dependent settings are valid.
						//! Invalid settings will be fixed if possible or an error will be returned if not.
						ERRVALUE Validate (UINT8 dispmode);

						//! Validate contrast settings.
						//! Invalid settings will be fixed if possible or an error will be returned if not.
						ERRVALUE ValidateContrast (UINT8 dispmode);

						//! Validate datatip settings.
						//! Invalid settings will be fixed if possible or an error will be returned if not.
						ERRVALUE ValidateDataTip ();

						//! Validate objitem settings.
						ERRVALUE ValidateObjItem ();

					private:
						#ifndef GENERATING_DOXYGEN_OUTPUT
						RVC::OBJITEM m_ObjItem;
						mutable RVC::OBJITEM m_SaveObjItem;
						RVC::OBJECTNAME m_ObjNameGeoref;		// Georeference object name
						RVC::OBJECTNAME m_ObjNameContrast;	// Contrast table object name
						CONTRASTPARM m_ContrastParm;			// Contrast parameters for this raster
						SHADEPARM m_ShadeParm;					// Relief shading parameters for this raster
						DATATIPPARMS m_DataTip;					// DataTip parameters
						mutable DOUBLE_RANGE m_Range;			// Cell value range
						INT32 m_NumColumns;						// Total columns in image
						INT32 m_NumRows;							// Total rows in image
						UINT32 m_CellType;						// Image Cell Type (RVC::IMAGE::CELLTYPE)
						INT32 m_Band;								// Band number for hyperspectral set
						bool m_IsConstant;						// Treat raster as "constant", output value in contparm.outconstant
						bool m_HasSaveInfo;
						bool m_DoContrast;						// Perform contrasting on this raster, set by OpenRast
						bool m_DoShading;							// Perform relief shading on this raster
						UINT8 m_ComponentComplex;
						INT8 m_ComponentNum;						// Component number in ioparm array, -1 if not set
						mutable STAT_UNIVARIATE m_Stats;

						static ERRVALUE SerializerCB_OldContParmType (SERIALIZER&, const SERIALIZERITEM*, void *vgroup, SERIALIZERITEM::ACTION);

						ERRVALUE ComputeStats () const;
						void Copy (const IOPARM&);
						void InitContrast (UINT8 dispmode);

						#endif // GENERATING_DOXYGEN_OUTPUT
					};	// End of LAYER_RASTER::DISPPARM::IOPARM

				// LAYER_RASTER::DISPPARM::COLORSCALELABEL
				class COLORSCALELABEL {
					public:
						double CellValue;					//!< Cell value to label
						MISTRING Label;					//!< Label string

						//! Default constructor.
						COLORSCALELABEL (
							) :
							CellValue(0.0)
							{ }

						//! Copy constructor.
						COLORSCALELABEL (
							const COLORSCALELABEL& rhs
							) :
							CellValue(rhs.CellValue),
							Label(rhs.Label)
							{ }

						//! Destructor.
						~COLORSCALELABEL() {
							}

						//! Assignment
						COLORSCALELABEL& operator= (
							const COLORSCALELABEL& rhs
							) { if (this != &rhs) { CellValue = rhs.CellValue; Label = rhs.Label; } return (*this); }
					};	// End of LAYER_RASTER::DISPPARM::COLORSCALELABEL class.

				IOPARM ioparm[4];
				IOPARM maskioparm;
				UINT8 dispmode;						//!< Type of "display" being performed (RASTDISPMODE_xxxx)
				UINT8 ColorBlendMode;				//!< Color blending mode (for RGBI modes)
				UINT8 ColorBlendFactor;				//!< Color blending factor for ratio mode
				UINT8 Transparency;					//!< Layer transparency factor, 0=opaque
				UINT8 UseMask;							//!< Flag: Set if mask should be used
				UINT8 PreferredZoom1X;				//!< Set if layer preferred for 1X zoom
				bool LoadColMap;						//!< Load color palette using specified color palette name
				bool UseColMap;						//!< Set if color palette should be used
				COLOR_ARRAY colmap;					//!< Color palette
				RVC::OBJECTNAME m_ColorPaletteName;	//!< Name of color palette object to use / load
				bool m_UseThemeMap;
				RVC::RASTERTHEMEPARM m_themeparm;  //!<Theme Map
				RVC::OBJECTNAME m_themeparmname; //!< Name of theme map object to use / load
				bool m_OutlineContiguousAreas;			//!< Set to "outline" contiguous areas
				UINT8 m_OutlineInteriorTransparency;	//!< Transparency factor for interior when outlines used
				INT16 XXXdelete_OutTileCols;
				INT32 XXXdelete_OutTileLins;
				INT32 NumBands;						//!< Number of bands in hyperspectral mode
				CBPARMS ColorBalance;				//!< For adjusting color
				UINT8 LegendType;						//!< Type of legend
				UINT8 LegendSampleArea;				//!< Sample area
				UINT8 LegendScaleMinAtTop;			//!< Put minimum value at top
				UINT16 LegendScaleRangeCount;		//!< Number of range boxes for ColorScaleRanges
				LEGENDPARMS ColorSampleLegend;
				double xscale, yscale;				//!< Scale from object coords to meters if no georef info available
				MILIST<COLORSCALELABEL> ScaleLabels;
				double ScaleMin;						//!< Minimum cell value for color scale legend
				double ScaleMax;						//!< Maximum cell value for color scale legend
				double ScaleScale;					//!< Default label will be CellValue * ScaleScale + ScaleOffset
				double ScaleOffset;
				double ScaleInterval;				//!< CellValue interval for auto label generation
				bool ScaleMinOnTop;					//!< Flips the legend upside down
				bool ScaleIsAuto;						//!< True if scale is auto-generated, false if manually edited
				UINT8	ScalePlaces;					//!< Number of decimal places for scale
				UINT8 m_CellLabelPlaces;			//!< Number of decimal places for cell labels
				UINT8 m_EnlargeMethod;
				UINT8 m_ReduceMethod;
				UINT8 TextureFilterMode;			//!< GRE::TEXTUREFILTER::MODE value
				UINT8 AnisotropicLimit;				//!< 2 - 64
				UINT8 MipMapSharpness;				//!< 0 - 100
				GRE::BOUNDPARM m_Pedestal;
				GRE::BOUNDPARM m_Fence;
				RASTER_FILTER_PARMS RasterFilterParms;		//!< Settings user for raster spatial filtering

				DISPPARM ();
				DISPPARM (
					const DISPPARM& rhs
					): LAYER::DISPPARM(rhs) { Copy(rhs); }
				virtual ~DISPPARM ();

				DISPPARM& operator= (
					const DISPPARM& rhs
					) { if (this != &rhs) { Free(); Copy(rhs); } return (*this); }

				CBPARMS& AccessColorBalance ()
					{ return (this->ColorBalance); }

				COLOR_ARRAY& AccessColorPalette ()
					{ return (this->colmap); }

				bool CanAutoColorMap () const;

				bool CanColorBalance () const;

				bool CanContrast () const;
						
				bool CanThemeMap () const;

				bool CanUseColorPalette () const;

				void ClearColorPalette ();

				INT32 FindClosestBand (
					double wavelength
					) const;

				const GRE::BOUNDPARM& Get3dBoundFence () const
					{ return (m_Fence); }

				const GRE::BOUNDPARM& Get3dBoundPedestal () const
					{ return (m_Pedestal); }

				UINT8 GetCellLabelDecPlaces () const
					{ return (m_CellLabelPlaces); }

				UINT32 GetCellType (
					int component = 0
					) const { return (this->ioparm[component].GetCellType()); }

				const CBPARMS& GetColorBalance () const
					{ return (this->ColorBalance); }

				const COLOR_ARRAY& GetColorPalette () const
					{ return (this->colmap); }

				bool GetColorPaletteLoad () const
					{ return (this->LoadColMap); }

				const RVC::OBJECTNAME& GetColorPaletteName () const
					{ return (m_ColorPaletteName); }

				//! Get display mode (RASTDISPMODE_...).
				UINT8 GetDisplayMode () const
					{ return (this->dispmode); }

				UINT8 GetEnlargeMethod () const
					{ return (m_EnlargeMethod); }

				const RVC::OBJITEM& GetObjItemMask () const
					{ return (this->maskioparm.GetObjItem()); }

				int GetNumColorComponents () const;

				const RVC::OBJECTNAME& GetObjNameColorPalette (
					) const { return (m_ColorPaletteName); }

				//! Get component to use for shaded relief.
				//! @return Component index or -1 if none.
				int GetShadeComponent () const;

				UINT8 GetTextureAnisotropicLimit () const
					{ return (this->AnisotropicLimit); }

				UINT8 GetTextureFilterMode () const
					{ return (this->TextureFilterMode); }

				UINT8 GetTextureMipMapSharpness () const
					{ return (this->MipMapSharpness); }

				UINT8 GetTransparency () const
					{ return (this->Transparency); }

				bool IsColorPaletteUsed () const
					{ return (this->UseColMap); }

				bool IsPreferredZoomMaxDetail () const
					{ return (this->PreferredZoom1X != 0); }

				bool IsReliefShading () const;

				bool IsUsingMask () const
					{ return (this->UseMask != 0); }

				bool LegendCanDoColorScale () const;
				bool LegendCanDoColorSample () const;
				bool LegendCanDoColorThemeScale() const;

				//! Load color palette if needed.
				ERRVALUE LoadColorPalette ();

				void ResetColorBalance ();

				void SetColorPalette (
					const COLOR_ARRAY& ColorPalette
					) { this->colmap = ColorPalette; this->UseColMap = true; this->LoadColMap = false; }

				void SetColorPaletteNameLoad (
					const RVC::OBJECTNAME& ColorPaletteName,
					bool setLoadPalette = true
					) { m_ColorPaletteName = ColorPaletteName; this->LoadColMap = setLoadPalette; }

				void SetLegendType (
					UINT8 legendType
					) { this->LegendType = legendType; }

				void SetMaskObjItem (
					const RVC::OBJITEM& ObjItem
					) { this->maskioparm.SetObjItem(ObjItem,true); this->UseMask = true; }

				void SetMaskUsed (
					bool isUsed
					) { this->UseMask = isUsed; }

				//! Set to use specified object.
				ERRVALUE SetObject (
					const RVC::OBJITEM& objitem,		//!< Object to set
					bool keepsettings = false			//!< Retain current display settings
					);

				//! Set to use specified object(s).
				ERRVALUE SetObjects (
					const RVC::OBJITEMLIST& objitems,	//!< Objects to set
					bool keepsettings = false				//!< Retain current display settings
					);

				void SetTextureFilterMode (
					UINT8 mode
					) { this->TextureFilterMode = mode; }

				bool TestUsesTransparency ();

				//! Determine if mask is valid.
				//! @return true if current mask object is valid, false if not
				bool ValidateMask ();

			protected:
				virtual const SERIALIZERITEM* SerialGetItemDef (SERIALIZER& serializer) const;
				virtual ERRVALUE SerialRead (SERIALIZER& serializer);
				virtual ERRVALUE SerialWrite (SERIALIZER& serializer, const char *tagname = 0) const;

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				typedef DISPPARM THISCLASS;

				DOUBLE_ARRAY<double> m_BandWavelengths;

				static ERRVALUE SerializerCB_ColorPalette (SERIALIZER&, const SERIALIZERITEM*, void*, SERIALIZERITEM::ACTION);
				static ERRVALUE SerializerCB_ColorScaleLabels (SERIALIZER&, const SERIALIZERITEM*, void*, SERIALIZERITEM::ACTION);
				static ERRVALUE SerializerCB_Component (SERIALIZER&, const SERIALIZERITEM*, void*, SERIALIZERITEM::ACTION);
				static ERRVALUE SerializerCB_Mask (SERIALIZER&, const SERIALIZERITEM*, void*, SERIALIZERITEM::ACTION);

				void Copy (const DISPPARM&);
				void Free ();
				void UpdateSaveInfo () const;

				virtual ERRVALUE v_SetObjItem (const RVC::OBJITEM& objitem, bool replace);
				virtual ERRVALUE v_Validate ();

				friend class LAYER_RASTER;

				#endif // GENERATING_DOXYGEN_OUTPUT
			};	// End of LAYER_RASTER::DISPPARM

	#ifdef USE_MGUI
		static ERRVALUE DlgGetObjects (
			MDLGPARENT dlgparent,
			UINT8 dispmode,									//!< RASTDISPMODE_...
			RVC::OBJITEMLIST& ObjItemList
			);
	#endif

		//! Get common parameters for all layers of this type in group.
		static LAYER_RASTER::COMMONPARM* GetCommonParm (
			GROUP *group
			);

		//! Return the list of SML class members for this layer type.
		static const SML::CLASSMEMBER* GetSMLClassMembers ();

		//! Register this layer type
		static void RegisterType ();

		//! Method to install SML class, called by internal SML function.
		static void SMLClassInstall (
			SML::CONTEXT *context							//!< SML context
			);

		//! Construct layer and insert into list
		explicit LAYER_RASTER (
			GRE::GROUP *group,										//!< Group to contain layer
			CREATEFLAGS createflags = CREATEFLAG_None,	//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,		//!< Position in list
			LAYER *reflayer = 0								//!< Reference layer for insertion
			);

		virtual ~LAYER_RASTER ();

		//! Assign layer to use specified object(s)
		ERRVALUE AssignObjects (
			SUBTYPE subtype,								//!< Raster layer subtype
			const RVC::OBJITEMLIST& objitems			//!< Object item list
			);

		bool CanDoShading () const;

		void ClearPedestal ();

		//! Get common parameters for all layers of this type in group.
		LAYER_RASTER::COMMONPARM* GetCommonParm (
			) const { return (GetCommonParm(GetGroup())); }

		//! Retrieve layer-specific display parameters.
		//! @return current parameters
		const DISPPARM& GetDispParm () const
			{ return (m_dispparm); }

		//! Retrieve color-sample legend.
		GRE::LEGEND_COLORSAMPLE& GetColorSampleLegend (
			) {
			return (m_ColorSampleLegend);
			}

		bool IsPreferredZoomMaxDetail () const
			{ return (m_dispparm.IsPreferredZoomMaxDetail()); }

	#ifdef USE_MGUI
		//! Prompt user to select object(s) for raster layer.
		int PromptObjects (
			SUBTYPE subtype,								//!< Raster layer subtype
			bool OpenCtrlPanel = false,				//!< Whether or not to show control panel after selection
			MDLGPARENT dlgparent = 0					//!< Parent for dialogs
			);
	#endif

		//! Set color palette to use
		int SetColorMap (
			const COLOR_ARRAY& colmap,
			const RVC::OBJECTNAME& ColorPaletteName
			);

		//! Set layer-specific display parameters.
		//!
		//! This is the recommended method for changing how an existing layer is to be
		//! rendered.  Validation will be performed on the specified parameters and
		//! adjustments made to the layer's copy if necessary.  Use of this method
		//! usually result in a ChangeBegin / ChangeEnd message pair, and will also
		//! generate a ChangeObject message if the spatial object used in the layer
		//! is changed.
		ERRVALUE SetDispParm (
			const DISPPARM& dispparm					//!< Parameters to set
			);

		//! Set mask to use.
		ERRVALUE SetMask (
			const RVC::OBJITEM& objitem
			);

		//! Set to not use mask.
		void SetMaskNone ();

		//! Setup initial pipeline stages.
		ERRVALUE SetupInitialPipelineStages (
			SPATMOD::IMAGE::STAGE_AUTOPTR& StagePtr,
			const SPATMOD::GEOREFERENCE *pGeoreference
			);

		//! Update constrast to compute within specified region
		int UpdateContrastFromRegion (
			const REGION2D& region								//!< Region
			);

		//! Update constrast to use full area.
		int UpdateContrastFullArea (
			);

	protected:

		DISPPARM m_dispparm;								//!< Layer-specific display parameters

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		typedef LAYER_IMAGE BASECLASS;

		static void LegendMsgHandler (GRAPHICTREEITEM_MSG *msg);
	#ifdef X_NATIVE
		static void MCB_AutoColorMapApply (MxdAnyCallbackStruct*, LAYER_RASTER*);
		static void MCB_ColorBalance (MICALLBACKLIST::ANYCALLBACKSTRUCT*, LAYER_RASTER*);
		static void MCB_EditColorMap (MxdAnyCallbackStruct*, LAYER_RASTER*);
		static void MCB_EditContrast (McbAnyCallbackStruct*, LAYER_RASTER*);
	#endif

	#ifdef X_NATIVE
		MXDHANDLE m_DlgHndEditContrast;		// Dialog handle for contrast enhancement
		MXDHANDLE m_DlgHndEditColorMap;		// Dialog handle for color map editing
		MXDHANDLE m_DlgHndAutoColorMap;		// Dialog handle for automatic color map generation
		MXDHANDLE m_DlgHndColorBalance;		// Dialog handle for color balance
	#endif
		GRE::TEXTURE *m_pTexture;
		GRE::LEGEND_COLORSAMPLE m_ColorSampleLegend;
		GRE::LEGEND_COLORSCALE m_ColorScaleLegend;
		COMPINFO m_CompInfo[4];
	#ifdef USE_MGUI
		DISPPARM::DLG *m_pDlgControlPanel;
	#endif
		mutable RVC::RASTER *m_pObjRaster[4];
		mutable RVC::NULLMASK *m_pObjRasterNull[4];
		mutable RVC::RASTER *m_pObjMask;
		mutable RVC::NULLMASK *m_pObjMaskNull;

		void ClosePrivateObjects () const;
	#ifdef X_NATIVE
		void DestroyToolDialogs ();
	#endif
		ERRVALUE Draw2D (LAYERDC& layerdc);
		ERRVALUE Draw3D (LAYERDC& layerdc);
		ERRVALUE OpenPrivateObjects () const;
	#ifdef USE_MGUI
		void OnMenuTool (MGUI::ID, bool);
	#endif
		bool CB_ThemeFilter (const SPATMOD::IMAGE::SAMPLEITERATOR *it,const SPATMOD::IMAGE::SAMPLEITERATOR *itsrc, bool valid) const;

		ERRVALUE InsertSpatialFilter (
			SPATMOD::IMAGE::STAGE_AUTOPTR& StagePtr
			);

		void SetupLegend ();

		ERRVALUE SetupPyramid (
			const TRANS2D_MAPGEN& tmg,
			const LRECT2D& DispUsedRect,
			bool &needpyramid
			);

		// Overrides from LAYER.
		virtual ERRVALUE v_AssignObject (const RVC::OBJITEM& objitem, MDLGPARENT dlgparent);
		virtual bool v_CanShowDetails () const;
	#ifdef USE_MGUI
		virtual int v_ControlPanel (MDLGPARENT dlgparent, UINT32 flags = 0);
	#endif
		virtual LAYER* v_Copy (GRE::GROUP *group) const;
		virtual void v_DataTipGetDefaultPrefix (int component, DATATIPPARMS& DataTipParms) const;
		virtual const DATATIPPARMS* v_DataTipGetParms (int component) const;
		virtual int v_DataTipGetString (const DPOINT2D& point, MISTRING& string, SPATMOD::IMAGE::STAGE_ARRAY&, GRE::VIEW *view);
		virtual void v_DataTipSetDefault (int component, bool force);
		virtual ERRVALUE v_DataTipSetParms (int component, const DATATIPPARMS& DataTipParms);
		virtual ERRVALUE v_Draw (LAYERDC& layerdc);
		virtual ELEMNUM v_FindClosestElement (const DPOINT2D& point, double searchdist, double *distret, int component, const UNSIGNEDSET *pSearchSet, GRE::FINDFLAGS findflags);
		virtual int v_GetComponent (ELEMTYPE ElemType) const;
		virtual MICONSTRUCT* v_GetComponentIcon (int component) const;
		virtual COMPINFO* v_GetComponentInfo (int component);
		virtual MISTRING v_GetComponentName (int component) const;
		virtual UINT16 v_GetElemDrawSelectionMode (ELEMTYPE ElemType) const;
		virtual ELEMTYPE v_GetElemType (int component) const;
		virtual bool v_GetGeorefName (RVC::OBJECTNAME& name, int component = 0) const;
		virtual const GRE::LEGEND* v_GetLegend (ELEMTYPE ElemType);
		virtual INT64 v_GetNumElements (ELEMTYPE ElemType);
		virtual const RVC::OBJITEM& v_GetObjItem (int component) const;
		virtual double v_GetScaleMaxDetail (const DPOINT2D& point, const SPATREF::COORDREFSYS& crs, double DisplayPixelSizeMM) const;
		virtual bool v_HasLegend () const;
		virtual bool v_HasNullImageCells () const;
	#ifdef USE_MGUI
		virtual void v_MenuAddLayerItems (MGUI::MENU& menu);
		virtual void v_MenuAddRegionGenItems (MGUI::MENU& menu, ELEMTYPE ElemType);
	#endif
	#ifdef USE_MGUI
		virtual ERRVALUE v_PromptObject (bool OpenCtrlPanel, MDLGPARENT dlgparent);
	#endif
		virtual const char* v_SerialGetTagName () const;
		virtual void v_SetDftName (bool ReplaceExisting);
		virtual void v_SetObjectDescriptor (int component, const RVC::DESCRIPTOR& descriptor);
	#ifdef USE_MGUI
		virtual void v_TreeViewAddUpdateDetails (GRE::CTRL_TREEVIEW& TreeView, MGUI::CTRL_TREE::ITEMHND hndParent, MGUI::CTRL_TREE::ITEMHND hndInsertAfter);
		virtual MGUI::CTRL_TREE::ITEMHND v_TreeViewAddUpdateLegend (GRE::CTRL_TREEVIEW& TreeView, MGUI::CTRL_TREE::ITEMHND hndParent, MGUI::CTRL_TREE::ITEMHND hndInsertAfter);
	#endif
		virtual ERRVALUE v_UpdateExtents (bool forceupdate);

		LAYER_RASTER (const LAYER_RASTER&);
		LAYER_RASTER& operator= (const LAYER_RASTER&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};	// End of LAYER_RASTER


#define	RASTERLEGENDTYPE_SampleArea				0	//!< Piece of the raster
#define	RASTERLEGENDTYPE_ColorScaleContinuous	1	//!< Color scale
#define	RASTERLEGENDTYPE_ColorSample				2	//!< Sample of each color, possibly with labels
#define	RASTERLEGENDTYPE_None						3	//!< No legend
#define	RASTERLEGENDTYPE_ColorScaleRanges  		4	//!< Ranged color scale
#define	RASTERLEGENDTYPE_Thumbnail					5	//!< Image thumbnail
#define  RASTERLEGENDTYPE_ColorScaleThemeRanges 6  //!< Ranged color scale amtching themeparms


//===================================================================================================================
}	// End namespace GRE

#endif	// INC_GRE_LRRASTER_H
