/**
 * \file viewable.h <gre/viewable.h>
 * \brief GRE::VIEWABLE class definition
 *
 * \if NODOC
 * $Id: viewable.h_v 1.59 2012/03/29 21:46:13 mju Exp $
 *
 * $Log: viewable.h_v $
 * Revision 1.59  2012/03/29 21:46:13  mju
 * Use stereorender class.
 *
 * Revision 1.58  2011/10/20 13:28:45  mju
 * Remove redundant hdr.
 *
 * Revision 1.57  2011/02/01 15:59:24  bcolombe
 * Added GetNoRenderLayerList
 *
 * Revision 1.56  2011/01/18 00:05:05  bcolombe
 * Add members to MAINSCRIPT for allowing encrypted scripts
 *
 * Revision 1.55  2010/01/27 18:17:40  bcolombe
 * Added DlgRenderToTileset
 *
 * Revision 1.54  2008/07/21 20:51:05  mju
 * Add stereo values to parm_rendertoraster.
 *
 * Revision 1.53  2008/07/21 17:51:23  mju
 * Add parm_rendertoraster and use in renderToRaster method.
 *
 * Revision 1.52  2008/04/10 16:39:47  ldyas
 * added render to pdf dialog
 *
 * Revision 1.51  2008/01/25 15:34:43  mju
 * Add clear() and v_clear.
 *
 * Revision 1.50  2007/11/26 18:43:07  mju
 * Add getActiveLayer, unhighlightAllElements.
 *
 * Revision 1.49  2007/11/15 16:50:48  ldyas
 * added param to RenderToRaster
 *
 * Revision 1.48  2007/01/19 16:08:36  mju
 * Use color_array.
 *
 * Revision 1.47  2006/10/05 20:01:20  mju
 * Use color hdr.
 *
 * Revision 1.46  2006/10/04 14:57:11  mju
 * Add dlgRenderToSVG.
 *
 * Revision 1.45  2006/09/21 17:48:38  mju
 * Don't use xdefns
 *
 * Revision 1.44  2006/09/15 15:31:42  mju
 * Serialzier changes.
 *
 * Revision 1.42  2006/09/14 20:30:15  mju
 * Use serializable hdr.
 *
 * Revision 1.41  2006/09/14 17:05:39  mju
 * Use smlbase hdr.
 *
 * Revision 1.40  2006/05/05 17:46:54  mju
 * Add dlgRenderToKML.
 *
 * Revision 1.39  2006/02/16 15:02:03  mju
 * Add dlgEditScaleRanges.
 *
 * Revision 1.38  2005/12/02 23:04:51  mju
 * Add dlgRenderToRaster.
 *
 * Revision 1.37  2005/11/28 18:54:00  mju
 * Add getMax/MinScaleVisibleAny.
 *
 * Revision 1.36  2005/11/04 18:53:21  mju
 * Remove stuff for showing internal tables and scale ranges.
 *
 * Revision 1.35  2005/11/03 19:43:41  mju
 * Remove stuff to get/set showInternalTables.
 *
 * Revision 1.34  2005/11/02 23:00:52  mju
 * Remove objmgr stuff.
 *
 * Revision 1.33  2005/10/19 19:57:34  mju
 * Use gre namespace.
 *
 * Revision 1.32  2005/10/14 20:43:09  mju
 * Specify gre namespace for mfc_viewable.
 *
 * Revision 1.31  2005/10/14 17:34:04  mju
 * GRE namespace.
 *
 * Revision 1.30  2005/09/21 17:33:49  mju
 * Add isTopViewable.
 *
 * Revision 1.29  2005/09/15 21:50:16  mju
 * Add objMgrShell interface.
 *
 * Revision 1.28  2005/09/15 16:40:17  mju
 * Fix unref parm warning.
 *
 * Revision 1.27  2005/07/11 19:33:40  mju
 * Add mainscript.destroy method.
 *
 * Revision 1.26  2005/05/02 17:44:47  mju
 * Add getTopViewable and highlightMode methods.
 *
 * Revision 1.25  2005/02/24 14:42:55  mju
 * Add mainscript.callFunction variant to allow passing arg ptrs directly.
 *
 * Revision 1.23  2004/09/07 15:11:58  dwilliss
 * Added two new script functions
 *
 * Revision 1.22  2004/09/03 22:50:25  dwilliss
 * Added some methods to MAINSCRIPT
 *
 * Revision 1.21  2004/08/10 13:20:42  mju
 * Add control script fns for onGroupDrawLabelsBegin/End.
 *
 * Revision 1.20  2004/02/19 22:03:25  mju
 * Add func_onViewDataTipShowText for ctrl script.
 *
 * Revision 1.19  2004/02/09 21:11:20  mju
 * Add ifdef for getcontrolscript.
 *
 * Revision 1.18  2004/02/09 18:53:03  mju
 * Make mainscript class public and add Has/CallFunction methods.
 *
 * Revision 1.17  2004/01/23 20:53:03  scowan
 * Fixed for non-mgui.
 *
 * Revision 1.16  2004/01/16 21:59:38  mju
 * More ControlScript stuff.
 *
 * Revision 1.15  2004/01/12 21:20:52  mju
 * Add MAINSCRIPT.
 *
 * Revision 1.14  2003/11/10 18:43:55  mju
 * Add SaveObjItem and methods to manage.
 *
 * Revision 1.12  2003/09/19 19:45:26  mju
 * Add pFallbackPixel to RenderToRaster.
 *
 * Revision 1.11  2003/09/15 13:48:59  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.10  2003/08/29 20:13:27  mju
 * Add rect ptr to RenderToRaster.
 * Add v_GetExtents.
 *
 * Revision 1.9  2003/08/26 21:02:17  dwilliss
 * Added static SMLClassInstall method
 *
 * Revision 1.8  2003/07/30 15:48:28  mju
 * Ignore private sections.
 *
 * Revision 1.7  2003/07/29 15:45:26  mju
 * Don't define scriptlist prototypes unless X.
 *
 * Revision 1.6  2003/07/29 15:34:58  mju
 * Add UpdateViewScriptUI and Clear...ScriptList.
 *
 * Revision 1.5  2003/06/24 16:25:01  mju
 * Support saving macro/toolscripts with viewable.
 *
 * Revision 1.2  2003/06/17 13:23:56  mju
 * Include simplear.h.
 *
 * Revision 1.1  2003/06/16 19:40:43  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_VIEWABLE_H
#define	INC_GRE_VIEWABLE_H

#ifndef	INC_GRE_OBJECTBASE_H
	#include <gre/objectbase.h>
#endif

#ifndef INC_MI32_SERIALIZERITEM_H
	#include <mi32/serializeritem.h>
#endif

#ifndef INC_MI32_SERIALIZABLE_H
	#include <mi32/serializable.h>
#endif

#ifndef	INC_MI32_SIMPLEAR_H
	#include <mi32/simplear.h>
#endif

#ifndef	INC_MI32_RECT_H
	#include <mi32/rect.h>
#endif

#ifndef	INC_MI32_COLOR_H
	#include <mi32/color.h>
#endif

#ifndef	INC_RVC_OBJITEM_H
	#include <rvc/objitem.h>
#endif

#ifndef INC_MI32_SMLBASE_H
	#include <mi32/smlbase.h>
#endif

#ifndef INC_MI32_MILIST_H
	#include <mi32/milist.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class STEREORENDER;
#endif

namespace GRE {
//===================================================================================================================

//! Base class for objects that can have "views" (GRE::LAYOUT, GRE::GROUP).
class VIEWABLE : public GRE::OBJECT {
	public:

	#ifdef USE_MGUI
		class MAINSCRIPT : public SERIALIZABLE {
			public:
				//! Externally callable functions.
				enum FUNC {
					FUNC_OnViewDataTipShowRequest,				//!< parm[GRE::VIEW,POINT2D,TOOLTIP] ret[-1=noshow,0=builtin,1=show]
					FUNC_OnViewDataTipShowText,					//!< parm[GRE::VIEW,POINT2D,TOOLTIP] ret[-1=noshow,0=builtin,1=show]
					FUNC_OnGroupDrawLabelsBegin,					//!< parm[GRE::GROUP,GRE::VIEW]
					FUNC_OnGroupDrawLabelsEnd,						//!< parm[GRE::GROUP,GRE::VIEW]
					FUNC_OnLayerDrawLabelsBegin,					//!< parm[GRE::LAYER,GRE::VIEW]
					FUNC_OnLayerDrawLabelsEnd,						//!< parm[GRE::LAYER,GRE::VIEW]
					FUNC_OnDrawLabel,									//!< parm[LABELPLACE, GRE::LAYERDC,COLOR,Num] ret[0=builtin,1=drawn by script]
					FUNC_COUNT
					};

				MAINSCRIPT (VIEWABLE *viewable);
				virtual ~MAINSCRIPT ();
				int CallFunction (
					FUNC func,
					SMLARG *ArgIn,
					int NumArg,
					SMLARG *ArgRet = 0
					);
				int CallFunction (
					FUNC func,
					int NumArg,
					void *ArgPtr1,
					void *ArgPtr2 = 0,
					void *ArgPtr3 = 0
					);
				void Destroy ();
				int DlgEdit (MDLGPARENT dlgparent);
				bool HasFunction (FUNC func) const;
				bool HasScript () const;
				ERRVALUE Initialize ();
				void Terminate ();
			private:
				class CONTEXT;
				VIEWABLE *m_viewable;
				CONTEXT *m_pContext;
				MISTRING m_String;
				MISTRING m_DisplayString;
				bool m_IsEncrypted;
				MISTRING m_ScriptPath;

				ERRVALUE CreateSMLContext (SMLCONTEXT *& smlcontext);
				static void GRECB_CtrlScript (const GRE::MSG *pMsg, void *vpCtrlScript);
				static const SERIALIZERITEM* SerialGetItemDef ();
				virtual ERRVALUE SerialRead (SERIALIZER& serializer);
				virtual ERRVALUE SerialWrite (SERIALIZER& serializer, const char *tagname = 0) const;
				friend class MAINSCRIPT::CONTEXT;
			};
	#endif

		class PARM_RENDERTORASTER {
			public:

				//! Construct with both raster dimension and cell size.
				PARM_RENDERTORASTER (
					COLORSPACE ColorSpace,
					INT32 RasterColumns,
					INT32 RasterRows,
					double CellSize
					);

				//! Construct with raster dimensions.
				PARM_RENDERTORASTER (
					COLORSPACE ColorSpace,
					INT32 RasterColumns,
					INT32 RasterRows
					);

				//! Construct with cell size.
				PARM_RENDERTORASTER (
					COLORSPACE ColorSpace,
					double CellSize
					);

				~PARM_RENDERTORASTER ();

				const COLOR& GetBackgroundColor () const
					{ return (m_BackgroundColor); }

				double GetCellSize () const
					{ return (m_CellSize); }

				const COLOR_ARRAY* GetColorPalette () const
					{ return (m_pPalette); }

				COLORSPACE GetColorSpace () const
					{ return (m_ColorSpace); }

				const DRECT2D& GetExtentsView () const
					{ return (m_ExtentsView); }

				MGD::PIXEL GetFallbackPixel () const
					{ return (m_FallbackPixel); }

				INT32 GetImageColumns () const
					{ return (m_RasterColumns); }

				INT32 GetImageRows () const
					{ return (m_RasterRows); }

				bool GetSaveGeoreference () const
					{ return (m_SaveGeoreference); }

				const STEREORENDER* GetStereoValues () const
					{ return (m_pStereoValues); }

				bool GetUseClosestColorIfNotInPalette () const
					{ return (m_UseClosestIfNotInPalette); }

				int GetViewNum () const
					{ return (m_ViewNum); }

				bool HasColorPalette () const
					{ return (m_pPalette != 0); }

				bool IsDrawingStereo () const
					{ return (m_StereoDrawing && m_pStereoValues != 0); }

				void SetBackgroundColor (
					const COLOR& BackgroundColor
					) { m_BackgroundColor = BackgroundColor; }

				void SetExtents (
					const DRECT2D& ExtentsView
					) { m_ExtentsView = ExtentsView; }

				void SetPalette (
					const COLOR_ARRAY* pPalette,					//!< Pointer to palette, will be retained, NULL to dynamically create
					bool UseClosestIfNotInPalette = false,		//!< Use closest palette entry if not exact match
					MGD::PIXEL *pFallbackPixel = 0				//!< Pointer to fallback pixel, NULL to use 'transparent'
					);

				void SetSaveGeoreference (
					bool SaveGeoreference
					) { m_SaveGeoreference = SaveGeoreference; }

				void SetStereoDrawing (
					bool StereoDrawing = true
					) { m_StereoDrawing = StereoDrawing; }

				void SetStereoValues (
					const STEREORENDER& StereoValues
					);

				void SetViewNum (
					int ViewNum
					) { m_ViewNum = ViewNum; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				COLORSPACE m_ColorSpace;
				INT32 m_RasterColumns;
				INT32 m_RasterRows;
				double m_CellSize;
				COLOR m_BackgroundColor;
				int m_ViewNum;
				bool m_SaveGeoreference;
				bool m_UseClosestIfNotInPalette;
				const COLOR_ARRAY *m_pPalette;
				MGD::PIXEL m_FallbackPixel;
				DRECT2D m_ExtentsView;
				STEREORENDER *m_pStereoValues;
				bool m_StereoDrawing;

				void InitCommon ();
				#endif
			};

		//! Install SML class, called by internal SML function.
		static void SMLClassInstall (
			SMLCONTEXT* context
			);

		//! Destructor.
		virtual ~VIEWABLE (
			);

		//! Clear contents of viewable, resetting to default state.
		void Clear (
			);

		//! Clear save location specification.
		void ClearSaveLocation (
			) { m_SaveObjItem.Clear(); }

	#ifdef USE_MGUI
		// Open dialog to edit 'control script' if allowed.
		ERRVALUE ControlScriptEdit (
			MDLGPARENT dlgparent
			);
	#endif

		//! Destroy all views of this object.
		void DestroyAllViews (
			);

		//! Draw all views of this object.
		ERRVALUE DrawAllViews (
			GRE::DRAWCONDITION condition = GRE::DRAWCONDITION_Always
			) { return (v_DrawAllViews(condition)); }

	#ifdef USE_MGUI
		//! Get reference to "display control script".
		MAINSCRIPT& GetControlScript (
			) { return (m_CtrlScript); }
	#endif

		//! Get active layer for viewable.
		GRE::LAYER* GetActiveLayer (
			) const { return (v_GetActiveLayer()); }

		//! Get overall viewable extents.
		const DRECT2D& GetExtents (
			) const { return (v_GetExtents()); }

		//! Find layer using unique ID.
		virtual GRE::LAYER *FindLayerByID (
			UINT32 LayerID
			) const = 0;

		//! Get 'design scale' for layout or group.
		double GetDesignScale (
			) const { return (v_GetDesignScale()); }

		//! Get element highlighting mode.
		GRE::HIGHLIGHTMODE GetElemHighlightMode (
			) const { return (GetTopViewable()->m_ElemHighlightMode); }

		//! Get first view of this object.
		//! @return Pointer to view or NULL if no views of this object.
		//!
		//! To loop through all views for an object use this method to get the first view.
		//! Then use the GRE::VIEW::GetNext() to obtain the next view, checking for NULL as shown:
		//!
		//!		for (GRE::VIEW *view = object->GetFirstView(); (view != 0); view = view->GetNext()) {
		//!			...
		//!			}
		GRE::VIEW* GetFirstView (
			) const;

		//! Get full object name.
		virtual void GetFullName (
			MISTRING& name									//!< Name returned
			) const = 0;

		//! Get name of object to use in Ini file access.
		const char* GetIniName (
			) const { return (m_IniName); }

	#ifdef USE_MGUI
		//! Get MacroScript list.
		GRE::MACROSCRIPTLIST& GetMacroScriptList (
			) { return (*m_pMacroScriptList); }
	#endif

		//! Get maximum visible scale for any item in viewable.
		//! @return 0 if at least one item has no maximum scale set.
		double GetMaxScaleVisibleAny (
			) const { return (v_GetMaxScaleVisibleAny()); }

		//! Get minimum visible scale for any item in viewable.
		double GetMinScaleVisibleAny (
			) const { return (v_GetMinScaleVisibleAny()); }

		//! Get next available unique ID.
		virtual UINT32 GetNextLayerID (
			) const = 0;

		//! Get list of layers that can't be rendered
		void GetNoRenderLayerList (MILIST<GRE::LAYER*>& LayerList);

		//! Get path to file where viewable saved.
		const FILEPATH& GetSaveFilePath (
			) const { return (m_SaveObjItem.GetFilePath()); }

		//! Get object item reference to save location.
		const RVC::OBJITEM& GetSaveObjItem (
			) const { return (m_SaveObjItem); }

		//! Compute dimensions of object based on scale and georeference.
		virtual ERRVALUE GetSizeMeters (
			DPOINT2D& sizemeters							//!< Size returned in meters
			) = 0;

	#ifdef USE_MGUI
		//! Get ToolScript list.
		GRE::TOOLSCRIPTLIST& GetToolScriptList (
			) { return (*m_pToolScriptList); }
	#endif

		//! Get topmost containing viewable (const).
		const VIEWABLE* GetTopViewable (
			) const;

		//! Get topmost containing viewable (non-const).
		VIEWABLE* GetTopViewable (
			);

		//! Get view of this object given view number.
		//! @return Pointer to view or NULL if no matching view.
		GRE::VIEW* GetView (
			int ViewNum
			) const;

		//! Determine if object has any layers.
		virtual bool HasLayers (
			) const = 0;

		//! Determine if save location has been specified.
		bool HasSaveLocation (
			) const;

		//! Determine if changes have been made since last save.
		bool IsModified (
			) const { return (m_IsModified); }

		//! Determine if is topmost viewable.
		bool IsTopViewable (
			) const { return (this == GetTopViewable()); }

		//! Render to raster.
		ERRVALUE RenderToRaster (
			RVC::OBJITEM& objitem,							//!< Target object
			const PARM_RENDERTORASTER& parm				//!< Rendering parameters
			);

		//! Reset "is modified" status to "false".
		void ResetModified (
			) { m_IsModified = false; }

		//! Set 'design scale' for layout or group.
		void SetDesignScale (
			double DesignScale,							//!< Design scale to set
			bool notify = true							//!< Send notification if changed
			) { v_SetDesignScale(DesignScale,notify); }

		//! Set element highlighting mode.
		void SetElemHighlightMode (
			GRE::HIGHLIGHTMODE HighlightMode
			);

		//! Set base name of object for use in Ini file acces.
		void SetIniBase (
			const char* name
			);

	#ifdef USE_MGUI
		//! Set MacroScript list.
		void SetMacroScriptList (
			const GRE::MACROSCRIPTLIST& ScriptList
			);
	#endif

		//! Set path to file where viewable saved.
		//! Also resets object such that expects to save to non-RVC file.
		void SetSaveFilePath (
			const FILEPATH& SaveFilePath
			);

		//! Set object item reference to save location.
		void SetSaveObjItem (
			const RVC::OBJITEM& SaveObjItem
			) { m_SaveObjItem = SaveObjItem; }

	#ifdef USE_MGUI
		//! Set ToolScript list.
		void SetToolScriptList (
			const GRE::TOOLSCRIPTLIST& ScriptList
			);
	#endif

	#ifdef USE_MGUI
		//! Get all TreeViews which may contain this viewable.
		void TreeViewGetAll (
			SIMPLE_ARRAY<GRE::CTRL_TREEVIEW*>& TreeViewPtrs
			) const;
	#endif

	#ifdef USE_MGUI
		//! Set TreeView pointer.
		//! Used by GRE CTRL_TREEVIEW implementation, should not be called directly.
		void TreeViewSetPtr (
			GRE::CTRL_TREEVIEW *pTreeView
			) { m_pTreeView = pTreeView; }
	#endif

		//! Update user interface for changes to view scripts.
		void UpdateViewScriptUI (
			);

		//! Unhighlight all "marked" elements.
		void UnhighlightAllElements (
			bool redrawviews
			) { v_UnhighlightAllElements(redrawviews); }

	protected:

		bool m_IsModified;							//!< Changes have been made since last save

		//! Constructor.
		explicit VIEWABLE (
			OBJECT::TYPE type,					//!< Object type
			VIEWABLE *container					//!< Containing viewable
			);

		// Initialize 'control script' if exists and is allowed.
		ERRVALUE ControlScriptInitialize (
			);

		//! Get unique index for building Ini name.
		UINT8 GetIniIndex (
			) const { return (m_IniIndex); }

		//! Get serialization item definition array.
		static const SERIALIZERITEM* SerialGetItemDef (
			SERIALIZER&
			) { return (TemplateGetItemDef()); }

		//! Set index for building Ini name.
		void SetIniIndex (
			UINT8 IniIndex
			);

		//! Get serialization item definition array.
		static const SERIALIZERITEM* TemplateGetItemDef (
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		VIEWABLE *m_Container;						// Container viewable, 0 if at top
		char m_IniName[20];							// Name of object for use in Ini file access
		UINT8 m_IniIndex;								// Unique index of object for use in Ini file access
		UINT8 m_IniBaseLen;							// Length of base name not including appended index number
		RVC::OBJITEM m_SaveObjItem;				// Object item where saved
		GRE::HIGHLIGHTMODE m_ElemHighlightMode;
	#ifdef USE_MGUI
		MAINSCRIPT m_CtrlScript;
		GRE::CTRL_TREEVIEW *m_pTreeView;
		GRE::TOOLSCRIPTLIST *m_pToolScriptList;		// ToolScripts saved with this object
		GRE::MACROSCRIPTLIST *m_pMacroScriptList;		// MacroScripts saved with this object
	#endif

	#ifdef USE_MGUI
		static ERRVALUE SerialCB_MacroScript (SERIALIZER&, const SERIALIZERITEM*, VIEWABLE *viewable, SERIALIZERITEM::ACTION);
		static ERRVALUE SerialCB_ToolScript (SERIALIZER&, const SERIALIZERITEM*, VIEWABLE *viewable, SERIALIZERITEM::ACTION);
	#endif

		// VIEWABLE overridables.
		virtual void v_Clear () = 0;
		virtual ERRVALUE v_DrawAllViews (GRE::DRAWCONDITION condition) = 0;
		virtual GRE::LAYER* v_GetActiveLayer () const = 0;
		virtual double v_GetDesignScale () const = 0;
		virtual const DRECT2D& v_GetExtents () const = 0;
		virtual double v_GetMaxScaleVisibleAny () const = 0;
		virtual double v_GetMinScaleVisibleAny () const = 0;
		virtual void v_SetDesignScale (double DesignScale, bool notify) = 0;
		virtual void v_UnhighlightAllElements (bool redraw) = 0;

		VIEWABLE (const VIEWABLE&);
		VIEWABLE& operator= (const VIEWABLE&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};	// End of VIEWABLE

//===================================================================================================================

//! Dialog to edit scale ranges for items in viewable.
//! @return TRUE if any changes, FALSE if not, < 0 if error or cancel
int DlgEditScaleRanges (
	MDLGPARENT dlgparent,			//!< Parent for dialog
	VIEWABLE *viewable				//!< Viewable
	);

//! Dialog to render viewable to raster.
ERRVALUE DlgRenderToRaster (
	MDLGPARENT dlgparent,			//!< Parent for dialog
	VIEWABLE *viewable				//!< Viewable to render
	);

//! Dialog to render viewable to KML for Google Earth.
ERRVALUE DlgRenderToKML (
	MDLGPARENT dlgparent,			//!< Parent for dialog
	VIEWABLE *viewable				//!< Viewable to render
	);

//! Dialog to render viewable to PDF for Acrobat.
ERRVALUE DlgRenderToPDF (
	MDLGPARENT dlgparent,			//!< Parent for dialog
	VIEWABLE *viewable				//!< Viewable to render
	);

//! Dialog to render viewable to SVG.
ERRVALUE DlgRenderToSVG (
	MDLGPARENT dlgparent,			//!< Parent for dialog
	VIEWABLE *viewable				//!< Viewable to render
	);

//! Dialog to render viewable to tileset.
ERRVALUE DlgRenderToTileset (
	MDLGPARENT dlgparent,			//!< Parent for dialog
	VIEWABLE *viewable				//!< Viewable to render
	);

//===================================================================================================================

}	// End namespace GRE


#endif	// INC_GRE_VIEWABLE_H
