/**
 * \file gre/viewcommon.h
 * \brief GRE::VIEWCOMMON class definitions.
 *
 * \if NODOC
 * $Id: viewcommon.h_v 1.4 2011/10/20 17:14:23 mju Exp $
 *
 * $Log: viewcommon.h_v $
 * Revision 1.4  2011/10/20 17:14:23  mju
 * Remove unneeded antialias/thinhint members/methods.
 *
 * Revision 1.3  2010/09/13 16:47:38  mju
 * Add setting for whether to auto-redraw locator.
 *
 * Revision 1.1  2007/10/04 21:07:08  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_VIEWCOMMON_H
#define	INC_GRE_VIEWCOMMON_H

#ifndef	INC_MI32_MG2IMAGEFILEWRITER_H
	#include <mi32/mg2imagefilewriter.h>
#endif

#ifndef	INC_MI32_FILEPATH_H
	#include <mi32/filepath.h>
#endif

#ifndef	INC_MI32_COLOR_H
	#include <mi32/color.h>
#endif

namespace GRE {
//===================================================================================================================

//! Options which apply to all views in current process
class VIEWCOMMON {
	public:

		class FORM;									//!< Common options form, defined in <gre/uiviewcommon.h>
		class FORM_SNAPSHOT;						//!< Common options form for snapshot settings only, defined in <gre/uiviewcommon.h>

		VIEWCOMMON ();

		~VIEWCOMMON ();

	#ifdef USE_MGUI
		//! Dialog to edit all values.
		ERRVALUE DlgEditValues (
			MDLGPARENT dlgparent
			);
	#endif
		
	#ifdef USE_MGUI
		//! Dialog to setup snapshot.
		ERRVALUE DlgSnapshotSetup (
			MDLGPARENT dlgparent
			);
	#endif
		
		bool GetAutoOpen3dViewCtrl () const { return (m_AutoOpen3dViewCtrl); }
		bool GetAutoRaise () const { return (m_AutoRaise); }
		int GetAutoRedrawDelay () const { return (m_AutoRedrawDelay); }
		bool GetDefaultGeoLocked () const { return (m_DefaultGeoLocked); }
		bool GetLocatorRedrawOnAnyChange () const { return (m_LocatorRedrawOnAnyChange); }
		bool GetRedrawOnAnyChange () const { return (m_RedrawOnAnyChange); }
		UINT16 GetRefreshInterval () const { return (m_RefreshInterval); }
		bool GetSnapshotAddGeoFiles () const { return (m_SnapshotAddGeoFiles); }
		const FILEPATH& GetSnapshotFolder () const { return (m_SnapshotFolder); }
		const MGD::IMAGEFILEWRITER::SETTINGS& GetSnapshotFormat () const { return (m_SnapshotFormat); }
		UINT32 GetSnapshotIndex () const { return (m_SnapshotIndex); }
		bool GetSnapshotLaunchViewer () const { return (m_SnapshotLaunchViewer); }
		UINT32 GetSnapshotMaxIndex () const { return (m_SnapshotMaxIndex); }
		const MISTRING& GetSnapshotPrefix () const { return (m_SnapshotPrefix); }
		bool GetSnapshotUseSubFolder () const { return (m_SnapshotUseSubFolder); }
		bool GetTrackCursor () const { return (m_TrackCursor); }
		const COLOR& GetTrackCursorColor () const { return (m_TrackCursorColor); }
		bool GetUpdateAfterDrawLayer () const { return (m_UpdateAfterDrawLayer); }
		bool GetZoomPanWaitForRMB () const { return (m_ZoomPanWaitForRMB); }

		//! Increment snapshot index.
		//! Will update value in Ini file also.
		void IncrementSnapshotIndex (
			);

		//! Read settings from Ini file.
		//! By default, settings are shared among all processes.
		//! To retain settings separately, use GRE::SYSTEM::SetCommonIniGroup() during application initialization.
		void IniRead ();

		//! Write settings to Ini file.
		//! By default, settings are shared among all processes.
		//! To retain settings separately, use GRE::SYSTEM::SetCommonIniGroup() during application initialization.
		void IniWrite () const;

		void SetAutoOpen3dViewCtrl (bool AutoOpen3dViewCtrl) { m_AutoOpen3dViewCtrl = AutoOpen3dViewCtrl; }
		void SetAutoRedrawDelay (int AutoRedrawDelay) { m_AutoRedrawDelay = AutoRedrawDelay; }
		void SetAutoRaise (bool AutoRaise) { m_AutoRaise = AutoRaise; }
		void SetDefaultGeoLocked (bool DefaultGeoLocked) { m_DefaultGeoLocked = DefaultGeoLocked; }
		void SetLocatorRedrawOnAnyChange (bool RedrawOnAnyChange) { m_LocatorRedrawOnAnyChange = RedrawOnAnyChange; }
		void SetRedrawOnAnyChange (bool RedrawOnAnyChange) { m_RedrawOnAnyChange = RedrawOnAnyChange; }
		void SetRefreshInterval (UINT16 RefreshInterval) { m_RefreshInterval = RefreshInterval; }
		void SetSnapshot (const FILEPATH& Folder, const MGD::IMAGEFILEWRITER::SETTINGS& Format, bool AddGeoFiles, bool UseSubFolder, const MISTRING& Prefix, UINT32 Index = 0);
		void SetSnapshotFolder (const FILEPATH& Folder) { m_SnapshotFolder = Folder; }
		void SetSnapshotLaunchViewer (bool LaunchViewer) { m_SnapshotLaunchViewer = LaunchViewer; }
		void SetSnapshotMaxIndex (bool MaxIndex) { m_SnapshotMaxIndex = MaxIndex; }
		void SetTrackCursor (bool TrackCursor) { m_TrackCursor = TrackCursor; }
		void SetTrackCursorColor (const COLOR& TrackCursorColor) { m_TrackCursorColor = TrackCursorColor; }
		void SetUpdateAfterDrawLayer (bool UpdateAfterDrawLayer) { m_UpdateAfterDrawLayer = UpdateAfterDrawLayer; }
		void SetZoomPanWaitForRMB (bool ZoomPanWaitForRMB) { m_ZoomPanWaitForRMB = ZoomPanWaitForRMB; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		bool m_RedrawOnAnyChange;					// Redraw view when any change made to object in view
		bool m_AutoRaise;								// Auto-raise view when activate viewable
		bool m_UpdateAfterDrawLayer;				// Update display after each layer is drawn
		bool m_TrackCursor;							// Track cursor between views
		bool m_ZoomPanWaitForRMB;					// Wait for right mouse button (or <Enter>) to activate zoom/pan tool
		bool m_AutoOpen3dViewCtrl;					// Automatically open viewpoint controls for 3D view
		bool m_DefaultGeoLocked;					// Default views to be geolocked
		bool m_SnapshotAddGeoFiles;				// Add georeference files for non-RVC formats
		bool m_SnapshotUseSubFolder;				// Use sub-folder for quick snapshot if adding georeference files
		bool m_SnapshotLaunchViewer;				// Launch snapshot viewer (eg Google Earth)
		bool m_LocatorRedrawOnAnyChange;			// Redraw locator views on any change
		UINT16 m_RefreshInterval;					// Refresh interval for updates while layer is being drawn
		COLOR m_TrackCursorColor;					// Color of "tracking" cursor
		int m_AutoRedrawDelay;						// Delay for auto-redraw after change (milliseconds)
		FILEPATH m_SnapshotFolder;					// Folder for quick-snapshot files
		MGD::IMAGEFILEWRITER::SETTINGS m_SnapshotFormat;
		MISTRING m_SnapshotPrefix;					// Prefix for snapshot file/subfolder name
		UINT32 m_SnapshotIndex;						// Index of next snapshot
		UINT32 m_SnapshotMaxIndex;					// Maximum snapshot index before reuse, 0 to not reuse

		void IniWriteSnapshot () const;
		friend class VIEW;
		friend class VIEW_LEGACY;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================
}	// End namespace GRE

#endif	// INC_GRE_VIEWCOMMON_H
