/**
 * \file mgui/dlgmisc.h
 * \brief Miscellaneous MGUI dialog functions
 *
 * \if NODOC
 * $Id: dlgmisc.h_v 1.11 2013/01/22 22:30:26 mju Exp $
 *
 * $Log: dlgmisc.h_v $
 * Revision 1.11  2013/01/22 22:30:26  mju
 * Add dlgEditTextFileObject.
 *
 * Revision 1.10  2012/08/29 17:42:55  mju
 * Need const in prototype.
 *
 * Revision 1.9  2012/08/29 17:29:19  mju
 * Add dlgRetryFileRawOpen.
 *
 * Revision 1.8  2012/04/26 14:22:32  mju
 * Add dlgViewHttpStatistics.
 *
 * Revision 1.7  2011/07/20 14:01:58  mju
 * Remove unused fn.
 *
 * Revision 1.6  2011/05/03 13:33:00  mju
 * Forward-declare mistring.
 *
 * Revision 1.5  2011/05/02 19:14:28  ldyas
 * use register instead of set for httpauthdlg
 *
 * Revision 1.4  2011/05/02 18:44:07  ldyas
 * move in to MGUI name space
 *
 * Revision 1.3  2011/05/02 15:34:12  ldyas
 * added set http authentication dialog.
 *
 * Revision 1.2  2011/04/15 16:13:10  mju
 * Add dlgGetSystemFonts.
 *
 * Revision 1.1  2010/05/27 14:16:07  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_DLGMISC_H
#define	INC_MGUI_DLGMISC_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifndef	INC_MI32_FILEACCESS_H
#include <mi32/fileaccess.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations
class FILEPATH;
class MISTRING;
class AUTORETRY_FILE_RAW;
#endif

namespace MGUI {

ERRVALUE DlgEditTextFileObject (
	MDLGPARENT dlgparent,			//!< Parent for dialog
	bool IsModal						//!< Whether dialog is modal
	);

//! Dialog to select system font list.
int DlgGetSystemFonts (
	MDLGPARENT dlgparent				//!< Parent for dialog
	);

//! Show dialog for user to set host/realm specific username and password.
int DlgRegisterHTTPAuthentication (	 //!< uses GetDlgParent() default for parent
	const MISTRING& host,
	const MISTRING& realm
	);

//! Dialog to retry FILE_RAW::Open operation.
ERRVALUE DlgRetryFileRawOpen (
	const AUTORETRY_FILE_RAW& AutoRetry,
	const FILEPATH& filepath,
	FILE_ACCESS access,
	FILE_SHARE share
	);

//! Show dialog for viewing HTTP statistics.
void DlgViewHttpStatistics ();

}	// End namespace MGUI

#endif	// INC_MGUI_DLGMISC_H
