/** 
 * \file mgui/formgooglesearch.h 
 * \brief Definitions for MGUI::FORM_GOOGLESEARCH class.
 *
 * \if NODOC
 * $Id: formgooglesearch.h_v 1.8 2011/08/26 13:34:26 mju Exp $
 *
 * $Log: formgooglesearch.h_v $
 * Revision 1.8  2011/08/26 13:34:26  mju
 * Use formradiobuttons hdr.
 *
 * Revision 1.7  2011/03/28 12:42:11  mju
 * Need combobox hdr.
 *
 * Revision 1.6  2010/11/02 22:35:37  bcolombe
 * Use timer to update
 *
 * Revision 1.5  2010/10/20 20:34:15  bcolombe
 * *** empty log message ***
 *
 * Revision 1.4  2010/09/20 22:48:33  bcolombe
 * Added filtering
 *
 * Revision 1.3  2010/09/17 20:57:49  bcolombe
 * Have threads update form_list
 *
 * Revision 1.2  2010/09/13 20:46:32  bcolombe
 * *** empty log message ***
 *
 * Revision 1.1  2010/08/31 17:39:02  bcolombe
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_FORM_GOOGLESEARCH_H
#define	INC_MGUI_FORM_GOOGLESEARCH_H

#ifndef INC_MGUI_COMBOBOX_H
#include <mgui/combobox.h>
#endif

#ifndef INC_MGUI_EDIT_H
#include <mgui/edit.h>
#endif

#ifndef INC_MGUI_FORMLIST_H
#include <mgui/formlist.h>
#endif

#ifndef INC_MGUI_FORMTABBED_H
#include <mgui/formtabbed.h>
#endif

#ifndef INC_MGUI_TOOLTIP_H
#include <mgui/tooltip.h>
#endif

#ifndef INC_MGUI_TIMER_H
#include <mgui/timer.h>
#endif

#ifndef INC_MI32_MITHREAD_H
#include <mi32/mithread.h>
#endif

#ifndef INC_MI32_CRITSEC_H
#include <mi32/critsec.h>
#endif

#ifndef INC_MGUI_FORMEXTENTS_H
#include <mgui/formextents.h>
#endif

#ifndef INC_MGUI_FORMRADIOBUTTONS_H
#include <mgui/formradiobuttons.h>
#endif

#ifndef INC_MGUI_GRID_H
#include <mgui/grid.h>
#endif

#ifndef INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

#ifndef INC_SOA_GEONAMES_H
#include <soa/geonames.h>
#endif

//#define DEBUG_SEARCH 1

#ifdef DEBUG_SEARCH
#ifndef INC_MGUI_HTML_H
#include <mgui/html.h>
#endif
#endif


namespace MGUI {

class FORM_GOOGLESEARCH : public MGUI::FORM_COMPOSITE {
	public:
		typedef fastdelegate::FastDelegate<void(const MISTRING& name, const MISTRING& url)> DELEGATE_ONDOUBLECLICK;

		FORM_GOOGLESEARCH ();
		virtual ~FORM_GOOGLESEARCH ();

		//! Create form
		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane		//!< Parent pane
			);

		void SetDelegateOnDoubleClick(DELEGATE_ONDOUBLECLICK delegate) {m_delegate = delegate;}

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		
		int m_FieldStatus;
		int m_FieldName;
		int m_FieldURL;
		int m_FieldPath;
		int m_FieldDescription;
		int m_FieldAuthor;
		int m_FieldElements;
		int m_FieldExtents;
		int m_Offset;
		int m_PageText;
		int m_PagePlaces;
		int m_PageExtents;
		int m_PageElements;
		int m_PageResults;
		int m_FilteredResults;

		#ifdef DEBUG_SEARCH
		MGUI::FORM_HTML m_formhtml;
		#endif
		MGUI::FORM_TABBED m_TabForm;
		MGUI::CTRL_EDIT_STRING m_SearchText;
		MGUI::CTRL_PUSHBUTTON m_SearchBtn;
		MGUI::CTRL_LABEL m_ResultsLabel;
		MGUI::CTRL_LABEL m_FilteredResultsLabel;
		MGUI::FORM_LIST m_URLList;
		MGUI::CTRL_PUSHBUTTON m_NextBtn;
		MGUI::CTRL_PUSHBUTTON m_PrevBtn;
		DELEGATE_ONDOUBLECLICK m_delegate;

		class MyJob;
		MILIST<MyJob*> m_Jobs;
		MITHREAD::CRITICALSECTION m_JobSync;
		MITHREAD::POOL m_Pool;
		MITHREAD::QUEUE m_Queue;
		void OnJobDone (MITHREAD::ID id);
		bool m_StopJobs;
		class KMLData;
		MILIST<KMLData> m_KMLData;
		MGUI::TIMER m_timer;

		bool OnDraw(MGUI::FORM_LIST::CUSTOMDRAW& draw);
		bool OnDoubleClick(int itemnum, int fieldnum, const MGUI::POINT& point, MGUI::POINTERBUTTON button, MGUI::KEYSTATE keystate);
		void OnPageActivate ();
		bool OnPointerButtonDown (int itemnum, int fieldnum, const MGUI::POINT& point, POINTERBUTTON button, KEYSTATE keystate);
		bool OnPointerButtonUp (int itemnum, int fieldnum, const MGUI::POINT& point, POINTERBUTTON button, KEYSTATE keystate);
		bool OnToolTip (int itemnum, int fieldnum, MGUI::CTRL_TOOLTIP& tooltip);
		void OnTimer ();

		//*** PLACES & EXTENTS ***
		MISTRING m_GoogleKey;
		SOA::GEONAMES m_Geonames;
		MILIST<SOA::GEONAMES::RECORD> m_GeonamesList;
		DPOINT2D m_GeonamesLocation;
		MGUI::FORM_RADIOBUTTONS m_GeonamesRadio;
		MGUI::CTRL_EDIT_STRING m_GeonamesText;
		MGUI::CTRL_PUSHBUTTON m_GeonamesButton;
		MGUI::CTRL_GRID m_GeonamesGrid;
		MGUI::CTRL_LABEL m_GeonamesResults;
		MGUI::FORM_EDIT_NUMBER m_BufferEditNumber;
		void OnGetGoogleKey();
		void OnSetBufferSize();
		void OnSearchGeonames();
		void OnSelectGrid(int row, int col);
		void SearchPaneSetSensitive();

		class GOOGLEKEYSHELL : public MGUI::DLGSHELL {
			public:
				GOOGLEKEYSHELL ();

				~GOOGLEKEYSHELL ();
				
				void GetKey(MISTRING& key) {key = m_Key.GetValue();}

			private:
				MGUI::CTRL_LABEL m_Label;
				MGUI::CTRL_EDIT_STRING m_Key;
				MGUI::CTRL_PUSHBUTTON m_Button;
				void OnGetKey();

				ERRVALUE v_CreateContent(); 
			};

		GOOGLEKEYSHELL m_GoogleKeyShell;

		MGUI::CTRL_TOGGLEBUTTON m_ExtentsToggle;
		MGUI::FORM_COMBOBOX m_ContainmentCombo;
		MGUI::CTRL_TOGGLEBUTTON m_NoGlobalExtentsToggle;
		MGUI::CTRL_LABEL m_ContainmentLabel;
		MGUI::FORM_EDIT_EXTENTS m_FormExtents;
		MGUI::FORM_RADIOBUTTONS m_MatchExtentsRadio;
		//*** PLACES & EXTENTS***

		MGUI::CTRL_COMBOBOX m_ElementsCombo;
		MGUI::FORM_EDIT_NUMBER m_ElementsValue;

		void Search();
		void NextSearch();
		void PrevSearch();
		
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}	// End of MGUI namespace

#endif
