/**
 * \file mgui/toolregion.h
 * \brief MGUI::TOOL_REGION class definition
 *
 * \if NODOC
 * $Id: toolregion.h_v 1.9 2011/12/09 20:23:36 mju Exp $
 *
 * $Log: toolregion.h_v $
 * Revision 1.9  2011/12/09 20:23:36  mju
 * No context in v_draw.
 *
 * Revision 1.8  2011/06/21 14:13:26  mju
 * *** empty log message ***
 *
 * Revision 1.7  2008/12/01 21:13:33  scowan
 * Added generateRegion virtual method override.
 *
 * Revision 1.6  2008/02/01 15:01:36  mju
 * Change button handlers to return bool.
 *
 * Revision 1.5  2008/01/04 18:59:40  mju
 * Generalize pointer button methods.
 *
 * Revision 1.1  2003/01/10 23:09:13  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_TOOLREGION_H
#define	INC_MGUI_TOOLREGION_H

#ifndef	INC_MGUI_TOOLBASE_H
#include <mgui/toolbase.h>
#endif

#ifndef	INC_MI32_REGION2D_H
#include <mi32/region2d.h>
#endif

namespace MGUI {

//=====================================================================================================================

//! Region tool.
class TOOL_REGION : public TOOL_BASE {
	public:

		//! Constructor.
		explicit TOOL_REGION (
			TOOLCANVAS *canvas
			);

		//! Destructor.
		virtual ~TOOL_REGION ();

		//! Assignment from REGION2D
		TOOL_REGION& operator= (
			const REGION2D& rhs
			);

		//! Get reference to region.
		const REGION2D& GetRegion (
			) const { return (m_region); }

		//! Assign region to tool.
		void SetRegion (
			const REGION2D& region
			) { operator=(region); };

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		// Tool position values
		REGION2D m_region;

		// TOOL_BASE overrides.
		virtual void v_DoDraw ();
		virtual int v_GenerateRegion (REGION2D& region) const;
		virtual bool v_OnPointerButtonDown (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);
		virtual void v_OnPointerMove (MGUI::POINT point, POINTERBUTTON button, KEYSTATE keystate);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_TOOLREGION_H
