/*****************************************************************************
 *
 * \file editmenu.h <mi32/editmenu.h>
 * \brief EDITMENU class
 *
 * \if NODOC
 * $Log: editmenu.h_v $
 * Revision 1.6  2013/01/23 16:17:38  mju
 * Remove unused method declaration.
 *
 * Revision 1.5  2011/09/06 21:28:58  mju
 * Rename define_enum macro.
 *
 * Revision 1.4  2010/04/07 18:31:08  mju
 * Remove some methods to mgui::clipboard.
 *
 * Revision 1.3  2007/06/14 22:15:30  dwilliss
 * Major rewrite
 *
 * Revision 1.2  2007/03/23 22:48:53  dwilliss
 * added methods for syncing wysiwyg toggle with ours
 *
 * Revision 1.1  2005/05/13 21:03:03  dwilliss
 * Initial revision
 *
 * \endif
 * **************************************************************************/

#ifndef INC_MI32_EDITMENU_H
#define INC_MI32_EDITMENU_H

#ifdef X_NATIVE
#ifndef INC_MI32_TEXTED_H
#include <mi32/texted.h>
#endif


struct MENUITEM;
struct BUTTONITEM;

#endif

#ifndef INC_MI32_DELEGATE_H
#include <mi32/delegate.h>
#endif

namespace MGUI {
	class CTRL_EDIT_BASE;
	class MENU;
	class MENU_EDIT;
	}

class MISTRING;


class EDITMENU {
	public:

		enum FLAGS {
			FLAG_None =				0x0000,
			FLAG_NoSearch =		0x0001,
			FLAG_NoReplace =		0x0002,
			FLAG_NoInsertFile = 	0x0004,
			FLAG_AllowFormat =  	0x0008	//!< For if widget is WYSIWYG
			};

		EDITMENU ();
		EDITMENU (const EDITMENU&);
		virtual ~EDITMENU ();

		void Cut();
		void Copy();
		void Paste();
		void Clear();

#ifdef X_NATIVE

		void AddCallbacks (Widget xetext);

		// Note: the WYSIWYG toggle had to be created to begin with
		// We oly have WYSIWYG editing in X11 code
		void SetWYSIWYG (bool value);
		void SetWYSIWYGEnabled (bool value);

		const BUTTONITEM* CreateButtonItems (
			FLAGS flags,
			void (*undofunc)(Widget, void*) = 0,
			void* undodata = 0
			);

		const MENUITEM* CreateMenuItems (
			FLAGS flags,
			void (*undofunc)(Widget, void*) = 0,
			void* undodata = 0,
			Widget shell = 0
			);

		static void RegisterTextWidget (Widget);
		static EDITMENU* AddPopupEditMenu (Widget);
#endif 

		void SetCurrentCtrl (
			MGUI::CTRL_EDIT_BASE *EditCtrl
			);

#ifdef X_NATIVE
		void SetCurrentWidget (
			Widget widget
			);
#endif

	private:

		class PRIV;
		PRIV* m_Priv;

		friend class MGUI::MENU_EDIT;
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(EDITMENU::FLAGS)
#endif

#endif
