/**
*	\file <mi32/fixedstr.h> - FIXEDSTRING<> template class.
*
*	\if NODOC
*	$Id: fixedstr.h_v 1.72 2011/10/10 20:48:12 mju Exp $
*
*	$Log: fixedstr.h_v $
*	Revision 1.72  2011/10/10 20:48:12  mju
*	Remove unneeded hdr.
*
*	Revision 1.71  2008/11/17 15:20:49  mju
*	Comment out redundant methods as can rely on cast to const miunicode*.
*
*	Revision 1.70  2008/11/17 14:14:38  mju
*	Fix some comments.
*
*	Revision 1.69  2008/11/14 15:33:18  mju
*	Use ucchar hdr.
*
*	Revision 1.68  2007/09/24 14:54:29  scowan
*	Added const to sub array in generateUniqueName to fix g++ warning.
*
*	Revision 1.67  2007/05/07 13:40:24  mju
*	Just use inibase hdr.
*	Remove errcodes hdr as not needed by this hdr.
*
*	Revision 1.66  2007/05/07 13:30:14  mju
*	Move fixedstrchar to own hdr.
*
*	Revision 1.65  2007/04/19 19:45:54  mju
*	Deprecate inclusion of errcodes hdr as not needed by this or any dependent headers.
*
*	Revision 1.64  2006/06/19 13:08:55  mju
*	Still need ucstring hdr as many uc... fns used in template.
*
*	Revision 1.63  2006/06/14 14:30:28  mju
*	Use charencoding hdr.
*
*	Revision 1.62  2006/04/07 13:22:49  mju
*	Use int instead of size_t for size, 4GB is probably enough.
*
*	Revision 1.59  2005/07/21 15:35:26  scowan
*	Missed a place to call the delegate instead of the default function.
*
*	Revision 1.58  2005/07/21 14:08:52  scowan
*	Added isValidName and makeValidName using delegates.
*
*	Revision 1.52  2004/03/31 18:15:28  dwilliss
*	Added Assign() variant that takes a CHAR_ENCODING to make it match the
*	one that MISTRING has
*
*	Revision 1.48  2003/04/15 17:06:40  scowan
*	Make sure string is cleared to zero from the terminator to the end in generate unique name.
*
*	Revision 1.1  1999/08/26 16:11:28  scowan
*	Initial revision
*	\endif
**/

#ifndef INC_MI32_FIXEDSTR_H
#define INC_MI32_FIXEDSTR_H

#ifndef	INC_MI32_UCSTRING_H
	#include <mi32/ucstring.h>
#endif

#ifndef	INC_MI32_UCCHAR_H
	#include <mi32/ucchar.h>
#endif

#ifndef INC_MI32_MEMBUF_H
	#include <mi32/membuf.h>
#endif

#ifndef INC_MI32_INIBASE_H
	#include <mi32/inibase.h>
#endif

#ifndef INC_MI32_MISTRING_H
	#include <mi32/mistring.h>
#endif

#ifndef	INC_MI32_DELEGATE_H
	#include <mi32/delegate.h>
#endif

#ifndef INC_MEMORY_H
	#include <memory.h>		// For memcpy
	#define INC_MEMORY_H
#endif


inline bool DftValidNameChar (
	MIUNICODE c
	) { return (c =='_'||(c >='A'&& c <='Z')||( c >='a'&& c <='z')||( c >='0'&& c <='9')|| c >=160); }


//! Fixed-size array-based MIUNICODE string.
//!	This class is designed to replace fixed size MIUNICODE array string constructs in other classes and structures.
//! It is not meant as a STRING class implementation.  Therefore, the class cannot have virtual methods nor any
//!	other members except for the string storage itself.
template <int _CT> class FIXEDSTRING {
	public:

		//! Default constructor, clears the string.
		FIXEDSTRING (									
			) { Clear(); }

		//! Copy constructor.
		FIXEDSTRING (									
			const FIXEDSTRING& rhs
			) { memcpy(m_String, rhs.m_String, sizeof(m_String)); }

		//! Copy constructor
//		template <int _T2> FIXEDSTRING (
//			const FIXEDSTRING<_T2>& rhs				//!< String to concatinate from
//			) {
//			Clear();
//			ucstrncpy(m_String, rhs.GetReference(), _CT-1);
//			}

		//! Construct from MIUNICODE string.
		FIXEDSTRING (									
			const MIUNICODE* string
			) { 
			Clear(); 
			if (string != 0) ucstrncpy(m_String, string, _CT - 1);
			}

		//! Construct from MISTRING string.
		FIXEDSTRING (									
			const MISTRING& string
			) { 
			Clear(); 
			ucstrncpy(m_String, string, _CT - 1);
			}

		//! Construct from TEXTID.
		FIXEDSTRING (
			TEXTID textid
			) { Clear(); MISTRING str(textid); ucstrncpy(m_String,str,_CT-1); }

		//! Destructor.
		~FIXEDSTRING (									
			) { m_String[0] = 0; }

		//! Assignment operator.
		FIXEDSTRING& operator= (						
			const FIXEDSTRING& rhs				//!< String to assign from
			) {
			if (this != &rhs) {
				memcpy(m_String, rhs.m_String, sizeof(m_String));
				}
			return (*this);
			}

//		//! Assignment operator.
//		template <int _T2> FIXEDSTRING<_CT>& operator= (
//			const FIXEDSTRING<_T2>& rhs				//!< String to concatinate from
//			) {
//			Clear();
//			ucstrncpy(m_String, rhs.GetReference(), _CT-1);
//			return (*this);
//			}

		//! Assign from MIUNICODE string.
		FIXEDSTRING& operator= (					
			const MIUNICODE* rhs					//!< String to assign from
			) {
			if (m_String != rhs) {
				Clear();
				if (rhs != 0) ucstrncpy(m_String, rhs, _CT - 1);
				}
			return (*this);
			}

		//! Assign from MISTRING string.
		FIXEDSTRING& operator= (					
			const MISTRING& rhs					//!< String to assign from
			) {
			Clear();
			ucstrncpy(m_String, rhs, _CT - 1);
			return (*this);
			}

		//! Assign from TEXTID.
		FIXEDSTRING& operator= (
			TEXTID textid							//!< Message text ID
			) { Clear(); MISTRING str(textid); ucstrncpy(m_String,str,_CT-1); return(*this); }

		//! Concatenate operator.
//		template <int _T2>
//		FIXEDSTRING<_CT>& operator+= (
//			const FIXEDSTRING<_T2>& rhs				//!< String to concatinate from
//			) {
//			ucstrncat(m_String, static_cast<const MIUNICODE*>(rhs), _CT - (ucstrlen(m_String) + 1));
//			return (*this);
//			}

		//! Concatenate from MIUNICODE string.
		FIXEDSTRING& operator+= (						
			const MIUNICODE* rhs					//!< String to concatinate from
			) {
			if (rhs != 0) ucstrncat(m_String, rhs, _CT - (ucstrlen(m_String) + 1));
			return (*this);
			}
			
		//! Cast to <const MIUNICODE*>.
		operator const MIUNICODE* (
			) const { return (m_String); }

		//! Equality.
//		bool operator== (
//			const FIXEDSTRING& rhs
//			) const { return (Compare(rhs) == 0); }

		//! Equality.
		bool operator== (
			const MIUNICODE *rhs
			) const { return (Compare(rhs) == 0); }

		//! Inequality.
//		bool operator!= (
//			const FIXEDSTRING& rhs
//			) const { return (!(*this == rhs)); }

		//! Inequality.
		bool operator!= (
			const MIUNICODE *rhs
			) const { return (!(*this == rhs)); }

		//! Less Than
//		bool operator< (
//			const FIXEDSTRING& rhs
//			) const { return (Compare(rhs) < 0); }

		//! Less Than
		bool operator< (
			const MIUNICODE *rhs
			) const { return (Compare(rhs) < 0); }

		//! Less Than or Equal to
//		bool operator<= (
//			const FIXEDSTRING& rhs
//			) const { return (Compare(rhs) <= 0); }

		//! Less Than or Equal to
		bool operator<= (
			const MIUNICODE *rhs
			) const { return (Compare(rhs) <= 0); }

		//! Greater Than
//		bool operator> (
//			const FIXEDSTRING& rhs
//			) const { return (Compare(rhs) > 0); }

		//! Greater Than
		bool operator> (
			const MIUNICODE *rhs
			) const { return (Compare(rhs) > 0); }

		//! Greater Than or Equal to
//		bool operator>= (
//			const FIXEDSTRING& rhs
//			) const { return (Compare(rhs) >= 0); }

		//! Greater Than or Equal to
		bool operator>= (
			const MIUNICODE *rhs
			) const { return (Compare(rhs) >= 0); }

		//! Append from ASCII character string.
		void Append (								 	
			const char* name						//!< Character string to append from
			) {
			MIUNICODE *p = ucstrchr(m_String, 0);
			strntouc(p, name, _CT - (p - m_String) - 1);
			return;
			}

		//! Assign from ASCII character string.
		void Assign (
			const char* name						//!< Character string to assign from
			) { strntouc(m_String, name, _CT - 1); }

		//! Assign from ASCII string with maximum specified length.
		void Assign (								 	
			const char* name,						//!< Character string to assign from
			int len
			) { strntouc(m_String, name, MIN(_CT - 1, len)); }

		//! Assign from character string with specified encoding.
		void Assign (								 	
			const char* name,						//!< Character string to assign from
			CHAR_ENCODING encoding				//!< Encoding to use
			) { 
			if (encoding == CHAR_ENCODING_ASCII) {
				strtouc(m_String, name); 
				}
			else {
				MISTRING mistr(name, encoding);
				Clear();
				ucstrncpy(m_String, mistr, _CT - 1);
				}
			}

		//! Assign from MIUNICODE string with a max length given
		void Assign (								 	
			const MIUNICODE* name,						//!< MIUNICODE string to assign from
			int len
			) { ucstrncpy(m_String, name, MIN(_CT - 1, len)); }

		//! Clear the whole string.
		void Clear (									
			) { memset(m_String, 0, sizeof(m_String)); }
		
		//! Case-sensitive comparison, length limited (similar to strncmp).
		//! @return 0 if strings equal, -1 if this < str, 1 if this > str.
//		int Compare (
//			const FIXEDSTRING& str						//!< String to compare to
//			) const { return (ucstrcmp(m_String, str.m_String)); }

		//! Case-sensitive comparison, length limited (similar to strncmp).
		//! @return 0 if strings equal, -1 if this < str, 1 if this > str.
		int Compare (
			const MIUNICODE *str						//!< String to compare to
			) const { return (ucstrcmp(m_String, str)); }

		//! Case-insensitive comparison (similar to stricmp).
		//! @return 0 if strings equal, -1 if this < str, 1 if this > str.
//		int CompareNoCase (
//			const FIXEDSTRING& str						//!< String to compare to
//			) const { return (ucstricmp(m_String, str.m_String)); }

		//! Case-insensitive comparison (similar to stricmp).
		//! @return 0 if strings equal, -1 if this < str, 1 if this > str.
		int CompareNoCase (
			const MIUNICODE *str						//!< String to compare to
			) const { return (ucstricmp(m_String, str)); }
			
		//! Unique name filter class to determine if the name given is unique					
		class UNIQUENAMEFILTER {
			public:
				UNIQUENAMEFILTER () {}

				//! Filter called to determine if the name is unique
				//! @return '1' if unique, '0' if not, < 0 error
				int Filter (const FIXEDSTRING<_CT>& string) { return (v_Filter(string)); }
			private:
				virtual int v_Filter (const FIXEDSTRING<_CT>& string) = 0;
			};

		//! Generate an unique name based on the filter passed in
		ERRVALUE GenerateUniqueName (
			FastDelegate<int(const FIXEDSTRING<_CT>&)> delegate
			) {
			ERRVALUE err;
			if ((err = delegate(m_String)) < 0) return (err);
			if (err > 0) return (0);		// String is already unique

			// Initialize string and pointer for character replacement	
			int k = 0, len = GetLength();
			memset(&m_String[len], 0, (_CT - len) * sizeof(MIUNICODE));			// Make sure rest of m_String is set to 0
			MIUNICODE* where = &m_String[len];
			if (len == (_CT-1)) where --;
			MIUNICODE* start = where;
			UINT8 count[_CT];
			memset(count, 0, _CT);

			static const char *AddArray = "1234567890abcdefghijklmnopqrstuvwxyz_?";
			do {
				*where = static_cast<MIUNICODE>(AddArray[k++]);
				if (*where == static_cast<MIUNICODE>('?')) {
					k = 1;
		 			*where = static_cast<MIUNICODE>(AddArray[0]);
					MIUNICODE *p;
					for (p = where - 1;(p >= start);--p) {
						*p = static_cast<MIUNICODE>(AddArray[count[p-m_String]++]);
						if (*p == static_cast<MIUNICODE>('?')) {
							*p = static_cast<MIUNICODE>(AddArray[0]);
							count[p-m_String] = 0;
							}
						else break;
						}
					if (p < start) {		// Reset system and increment where
						++where;
						if (where - start == 4) return (EProgramError); // Reality check, if need to increase, it can be. 37 ^ 4 tried!
						memset(count, 0, _CT);
						if (where - m_String == (_CT-1)) {		// As far out a possible, retreat until a match is not found
							start --;
							where = start;
							if (start < m_String) return (EProgramError);		// Out of 37 ^ (_CT-1) possible combinations, none found
							}	
						for (p = where;(p >= start);--p) *p = static_cast<MIUNICODE>(AddArray[0]);
						}
					}
				err = delegate(m_String);
				} while (err == 0);
			if (err < 0) return (err);
			return (0);
			}
			
		//! Generate an unique name based on the filter passed in
		ERRVALUE GenerateUniqueName (
			UNIQUENAMEFILTER& FilterInst
			) {
			FastDelegate<int(const FIXEDSTRING<_CT>&)> delegate = FastDelegate<int(const FIXEDSTRING<_CT>&)>(&FilterInst, &FIXEDSTRING<_CT>::UNIQUENAMEFILTER::Filter);
			return (GenerateUniqueName(delegate));
			}

		//! Get string in specified encoding.
		//! @return Encoded string caller must free.
		void* GetEncoded (
			CHAR_ENCODING encoding					//!< Encoding
			) const {
			MISTRING tstr(m_String);
			return (tstr.GetEncoded(encoding));
			}

		//! Return length of string, not including terminator.
		int GetLength (						  		
			) const { return (ucstrlen(m_String)); }

		//! Return maximum string size, not including terminator.
		static int GetMaxSize (						  		
			) { return (_CT-1); }
			
		//! Return const string pointer, used for sprintf type parameters
		const MIUNICODE* GetReference (
			) const { return (m_String); }
			
		//! Get tail of string having up to specified number of characters.
		const MIUNICODE* GetTail (
			int maxlen
			) const { int len = ucstrlen(m_String); return ((len <= maxlen) ? m_String : (m_String + (len - maxlen))); }
			
		//! Find out if the string is empty.	
		bool IsEmpty (									
			) const { return (m_String[0] == 0); }
			
		//! Set the string according to values in ini file
		//! @return: 'true' if entry found and information read, false if empty field or field does not exist
		bool IniRead (
			INIHANDLE inih,
			const char *group,
			const char*const field
			) { return (_iniReadString(inih, group, field, INITYPE_Unicode, m_String, _CT * sizeof(MIUNICODE)) > 0); }

		//! Save the string to values in ini file
		void IniWrite (
			INIHANDLE inih,
			const char *group,
			const char*const field
			) const { _iniWriteString(inih, group, field, INITYPE_Unicode, m_String); }

		bool IsValidName (
			FastDelegate<bool(MIUNICODE)> delegate = FastDelegate<bool(MIUNICODE)>(DftValidNameChar)
			) const {
			const MIUNICODE* p = m_String;
			if (*p == 0 || (*p >= '0' && *p <= '9') || !delegate(*p)) return (false);

			for (++p;(*p != 0);++p) {
				if (!delegate(*p)) return (false);
				}
			return (true);
			}
		
		void MakeValidName (
			FastDelegate<bool(MIUNICODE)> delegate = FastDelegate<bool(MIUNICODE)>(DftValidNameChar)
			) {
			if (IsEmpty()) {
				Assign("_1");		// Need to have something
				return;
				}
			if (IsValidName(delegate)) return;
			
			/* First remove any space characters from the end */
			MIUNICODE *p = ucstrchr(m_String, 0) - 1;
			memset(p+1, 0, ((_CT-1) - (p - m_String))*sizeof(MIUNICODE));			/* Zero out rest of the string */
			while (p >= m_String && ucisspace(*p)) *p-- = 0;

			/* Now for the remaining characters not valid, replace with '_' */
			for (p = m_String;(*p != 0);++p) {
				if (!delegate(*p)) *p = '_';
				}

			/* Deal with case of a numeric character in the front of the name */
			if (m_String[0] >= '0' && m_String[0] <= '9') {
				for (p = ucstrchr(m_String, 0);(p != m_String);--p) *p = *(p-1);
				*p = (MIUNICODE)'_';
				m_String[_CT-1] = 0;			/* Extreme case */
				}
			return;
			}	

		//! Pad the string with a char to the end of the buffer
		void PadToEnd (
			char item
			) {
			for (int j = ucstrlen(m_String);(j < _CT-1);++j) m_String[j] = item;
			m_String[_CT-1] = 0;
			}
			
		//! Set the string to lower case letters
		void SetLowerCase (
			) { ucstrlwr(m_String); }

		//! Set the string to upper case letters
		void SetUpperCase (
			) { ucstrupr(m_String); }
			
		//! Swap bytes of the string, used for RVC strings
		void SwapBytes (
			) { ::SwapBytes(m_String, _CT); }

		//! Force termination of string at the end.
		void Terminate (								
			) { m_String[_CT - 1] = 0; }

		//! Truncate the string, has no effect if the string is shorter than NewLength.
		void Truncate (									
			int NewLength
			) { if (NewLength < (_CT - 1)) m_String[NewLength] = 0; }

	private:
	 	#ifndef GENERATING_DOXYGEN_OUTPUT
		MIUNICODE m_String[_CT];
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#endif		// INC_MI32_FIXEDSTR_H
