/**
 * \file geom3d.h <mi32/geom3d.h>
 * \brief 3-D geometry functions
 *
 * \if NODOC
 * $Id: geom3d.h_v 1.10 2003/09/15 13:49:56 fileserver!dwilliss Exp $
 *
 * $Log: geom3d.h_v $
 * Revision 1.10  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.9  2003/09/08 20:33:57  dwilliss
 * Added doxygen tags for start/end of function groups
 *
 * Revision 1.8  2003/01/24 17:20:22  scowan
 * Fixed g++ warning.
 *
 * Revision 1.7  2001/02/09 00:21:13  vdronov
 * added Clip3DFlatPolygon
 *
 * Revision 1.6  2000/06/08 19:08:39  sparsons
 * Genitor documentation.
 *
 * Revision 1.5  2000/06/01 21:07:10  mju
 * Compile in isolation.
 *
 * Revision 1.3  1999/01/05 19:12:22  scowan
 * Swapped multiple keywords
 *
 * Revision 1.2  1997/04/04 18:07:23  mju
 * Add Clip3D prototypes and constants.
 *
 * Revision 1.1  1996/12/20  22:27:19  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_GEOM3D_H
#define	INC_MI32_GEOM3D_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

struct DPOINT3D;

//----------------------------------------------------------------------------
//!		Constants																					
//----------------------------------------------------------------------------

#define	OUTCODE3D_Top			0x01
#define	OUTCODE3D_Bottom		0x02
#define	OUTCODE3D_Right		0x04
#define	OUTCODE3D_Left			0x08
#define	OUTCODE3D_Back			0x10
#define	OUTCODE3D_Front		0x20

#define	CLIP3D_Perspective	0x0001	//!<  Perspective view 
#define	CLIP3D_ClipBackPlane	0x0002	//!<  Clip against back plane 

//----------------------------------------------------------------------------
//!		Function prototypes																		
//----------------------------------------------------------------------------

//!:Associate with "Geometric Functions (3D)"
//!\addtogroup geom3d Geometric Functions 3D
//!@{

#if defined(__cplusplus)
extern "C" {
#endif

//! Compute Cohen-Sutherland "out-codes" used in clipping.
void Clip3DCompOutCodes (
	INT32 numpoints, 
	DPOINT3D *point, 
	UINT8 *outcode, 
	double zmin, 
	UINT32 flags
	);

//!	Clip line against canonical view volume.
int Clip3DLine (
	DPOINT3D *p0, 								//!< Line starting point passed/returned.
	DPOINT3D *p1, 								//!< Line ending point passed/returned.
	double zmin, 
	UINT32 flags
	);

//!	Clip line against canonical parallel-projection view volume.
//!
//! @return 1 if line accepted (and clipped).
int Clip3DLineParallel (
	DPOINT3D *p0, 								//!< Line starting point passed/returned 
	DPOINT3D *p1, 								//!< Line ending point passed/returned
	UINT32 flags
	);

//!	Clip line against canonical perspective-projection view volume.
//!
//! @return 1 if line accepted (and clipped).
int Clip3DLinePerspective (
	DPOINT3D *p0, 								//!< Line starting point passed/returned.
	DPOINT3D *p1, 								//!< Line ending point passed/returned.
	double zmin, 
	UINT32 flags
	);

//!	Clip triangle to canonical view volume.
//!
//! @return Number of boundary points, 0 if outside view volume.
int Clip3DTriangle (
	DPOINT3D **tp, 
	UINT8 *outcode, 
	DPOINT3D *opoint, 						//!< Boundary points returned.
	double ViewPlaneDist, 
	UINT32 flags
	);

//!	Clip flat polygon to canonical view volume.
//!
//! @return Number of boundary points, 0 if outside view volume.
int Clip3DFlatPolygon (
	DPOINT3D *ipoint, 
	UINT8 *outcode,
	INT32 numIPoints,
	DPOINT3D *opoint, 						//!< Boundary points returned.(Should be twice more than numIPoints)
	double ViewPlaneDist, 
	UINT32 flags
	);

#if defined(__cplusplus)
}
#endif

//!@}

#endif
