/******************************************************************************
 * \file mi32/mg2binary.h
 * \brief A base class for all binary devices
 *
 * \if NODOC
 * $Log: mg2binary.h_v $
 * Revision 1.4  2011/11/11 14:30:07  mju
 * Make context nested and clean up.
 *
 * Revision 1.3  2011/10/26 17:57:45  mju
 * Add frame buffer support.
 *
 * Revision 1.2  2003/12/15 20:37:00  dwilliss
 * compute region should be const
 *
 * Revision 1.1  2003/12/15 17:55:32  dwilliss
 * Initial revision
 * \endif
 */
 
#ifndef INC_MI32_MG2BINARY_H
#define INC_MI32_MG2BINARY_H

#ifndef INC_MI32_MGD2_H
#include <mi32/mgd2.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class BINARYTOREGIONINTERFACE;
#endif


namespace MGD {

//! Base implementation of MGD::DEVICE for 1-bit visuals.
class DEVICE_BINARY : public MGD::DEVICE {
	public:

		//! constructor
		DEVICE_BINARY ();

		//! Destructor
		virtual ~DEVICE_BINARY ();

		//! Clear the whole image to 0.
		void ClearAll (
			) { SetAll(false); }

		// Compute a region which encloses all the 1 values.
		ERRVALUE ComputeRegion (
			BINARYTOREGIONINTERFACE &bintoreg
			) const;
			
		bool GetRange (
			INT32 x,
			INT32 y,
			INT32 x2,				//!< Maximum ending position
			INT32& endx			//!< Ending position (inclusive) RETURNED
			) const {
			return (v_GetRange(x, y, x2, endx));
			}
			
		bool GetValue (
			INT32 x,
			INT32 y
			) const { return (v_GetValue(x, y)); }
			
		//! Set the whole image to 1 or 0
		void SetAll (
			bool value = true
			) { v_SetAll(value); }

	protected:

		class CONTEXT : public MGD::CONTEXT {
			public:
				//! Destructor.
				virtual ~CONTEXT();

			protected:
				//! Default constructor.
				CONTEXT (
					MGD::DEVICE_BINARY *pDevice
					);
			private:
		   	#ifndef GENERATING_DOXYGEN_OUTPUT
				typedef MGD::CONTEXT BASECLASS;
				#endif
			};

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT
		DEVICE_BINARY (const DEVICE_BINARY&);
		DEVICE_BINARY& operator= (const DEVICE_BINARY&);

		// MGD::DEVICE overrides.
		virtual MGD::PIXEL v_GetColorPixel (const COLOR& color);
		virtual COLORSPACE v_GetColorSpace () const;
		virtual bool v_UseAntiAliasing () const;
		#endif // GENERATING_DOXYGEN_OUTPUT

		// Overridables.

		virtual bool v_GetRange (
			INT32 x,
			INT32 y,
			INT32 x2,			//!< Maximum ending position
			INT32& endx			//!< Ending position (inclusive) RETURNED
			) const = 0;
		
		virtual bool v_GetValue (
			INT32 x,
			INT32 y
			) const = 0;
			
		//! Set the whole image to 1 or 0
		virtual void v_SetAll (
			bool value
			) = 0;

	};


}	// End of namespace

#endif	// INC_MI32_MG2BINARY_H
