/******************************************************************************,
 * \file mi32/mgd2.h
 * \brief MicroImages Graphics Device
 *
 * \if NODOC
 * $Log: mgd2.h_v $
 * Revision 1.202  2012/03/15 17:33:31  mju
 * Add typeid support to allow device type identification.
 * Don't inline copyRect.
 *
 * Revision 1.201  2012/03/08 16:29:57  mju
 * Make getAllPlanesMask public.
 *
 * Revision 1.200  2012/01/26 22:28:30  mju
 * Remove unused drawPointPattern variant.
 *
 * Revision 1.199  2012/01/26 14:34:30  mju
 * Update some context virtual methods.
 *
 * Revision 1.198  2012/01/25 15:30:03  mju
 * Clean up context member definitions.
 *
 * Revision 1.197  2012/01/13 16:02:27  mju
 * Add useVirtualBuffer for getDepthBuffer.
 *
 * Revision 1.196  2011/12/19 15:58:22  mju
 * Use stipplemask class.
 *
 * Revision 1.195  2011/12/15 22:46:44  mju
 * Add stippleGetMaskRow/Test methods.
 *
 * Revision 1.194  2011/12/15 18:56:42  mju
 * Add base stipple support.
 *
 * Revision 1.193  2011/11/10 22:25:12  mju
 * Allow separate depth buffer for each frame buffer, and avoid implying only for 3D.
 * Add device::getNull so can hide null device defns.
 * Don't inline any virtual methods.
 * Change hatchMBR so don't need to include rvc header.
 *
 * Revision 1.192  2011/10/31 15:07:42  mju
 * In getImagePointer put useVirtualWindow parm last to reduce ambiguity.
 *
 * Revision 1.191  2011/10/28 21:14:48  mju
 * Require frame buffer parm for getImagePointer/const and writeImageBuffer.
 *
 * Revision 1.190  2011/10/26 17:58:25  mju
 * Frame buffer support.
 *
 * Revision 1.189  2011/10/12 17:05:14  mju
 * Use stylepattern_base in multiPassTestStyle.
 *
 * Revision 1.186  2010/09/16 22:50:31  bcolombe
 * Added method to textblock for turning off parsing of format chars
 *
 * Revision 1.185  2010/05/05 13:45:25  mju
 * Add device::getVirtualOffset and hasVirtualWindow protected methods.
 *
 * Revision 1.183  2009/11/12 17:14:55  mju
 * Add drawIcon using iconid.
 *
 * Revision 1.182  2009/09/28 17:45:13  mju
 * Add drawSimpleArrowHead and drawSimpleFrame.
 *
 * Revision 1.180  2008/09/23 14:47:51  mju
 * No default for prompt parm in dlgGetScalableFont.
 *
 * Revision 1.178  2008/08/07 13:29:38  mju
 * Add setNoFillRightPolyPixels.
 *
 * Revision 1.177  2008/05/01 17:38:19  bcolombe
 * Changed GetLineWidth to return double - everything that is using it expects double
 *
 * Revision 1.172  2007/07/24 20:41:19  dwilliss
 * Added method to reset virtual window
 *
 * Revision 1.167  2007/07/10 14:35:50  dwilliss
 * Added Transparency Mode
 *
 * Revision 1.166  2007/06/15 21:47:45  ldyas
 * added DrawLines Dashed which takes a polyline.
 *
 * Revision 1.150  2007/03/15 15:11:06  scowan
 * Added method to set aliasing to text block.
 * Fixed some textblock comments.
 *
 * Revision 1.133  2006/11/30 20:53:44  dwilliss
 * Added Images pointer to TEXTBLOCK
 *
 * Revision 1.120  2006/09/08 20:34:58  dwilliss
 * Added methods to fill/draw rounded rectangles
 *
 * Revision 1.105  2005/03/30 20:32:35  dwilliss
 * Added MultiPass rendering methods
 *
 * Revision 1.1  2001/07/11 17:33:41  dwilliss
 * Initial revision
 * \endif
 */

#ifndef INC_MI32_MGD2_H
#define INC_MI32_MGD2_H

#ifndef INC_MI32_MG2BASE_H
#include <mi32/mg2base.h>
#endif

#ifndef INC_MI32_ELEMSTYL_H
#include <mi32/elemstyl.h>
#endif

#ifndef INC_MI32_DISPINFO_H
#include <mi32/dispinfo.h>
#endif

#ifndef INC_MI32_RECT_H
#include <mi32/rect.h>
#endif

#ifndef INC_MI32_SCRNUNIT_H
#include <mi32/scrnunit.h>
#endif

#ifndef INC_MI32_TEXTBASE_H
#include <mi32/textbase.h>
#endif

#ifndef	INC_MI32_COLORARRAY_H
#include <mi32/colorarray.h>
#endif

#ifndef INC_MI32_MG2CMS_H
#include <mi32/mg2cms.h>
#endif

#ifndef INC_MI32_MG2ENUMS_H
#include <mi32/mg2enums.h>
#endif

#ifndef INC_MI32_ICONID_H
#include <mi32/iconid.h>
#endif

#ifndef INC_MI32_STIPPLEMASK_H
#include <mi32/stipplemask.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
class MISTRING;
class LABELSTYLE;
class POLYLINE;
class POLYLINELIST;
class REGION2D;
class SUBREGION2D;
struct TEXTMETRICS;
struct FTFACE;
struct COLORCOMPSTRUCT;
struct MICONSTRUCT;
class BINARYTOREGIONINTERFACE;
class SAVEFONTFLAGS;
namespace SPATMOD {
	namespace IMAGE {
		class STAGE_ARRAY;
		}
	}
namespace HTML {
	class HTMLDOC;
	}
namespace RVC {
	class STYLE;
	class RASTER;
	class STYLEPATTERN_BASE;
	class STYLEPATTERN_BITMAP;
	class STYLEPATTERN_LINE;
	class STYLEPATTERN_HATCH;
	class STYLEPATTERN_SYMBOL;
	}
namespace {
	class LOCALOPENFONTS;		// Located in mg2text.c
	}
namespace MGD {
	class OTPRIV;
	struct STRINGDATA;
	class CONTEXT;
	class DEVICE_MEM1;
	class DEVICE_MEM1_SPARSE;
	class DEVICE_MEM8G;
	}
#endif // GENERATING_DOXYGEN_OUTPUT


namespace MGD {

//=====================================================================================================================
// Special characters in array of Glyph IDs (GID) returned from
// FONT.UCtoGID().  When rendering or computing string width,
// These shoul be passed through to let the rendering engine handle them
// XXX These should be private to mg2textt.c and mgdtext.c
#define  MgdTEXTGID_ChangeFace		0xFFFF	
#define	MgdTEXTGID_LeftToRight		0xFFFE
#define	MgdTEXTGID_RightToLeft		0xFFFD
#define	MgdTEXTGID_NextGlyph0Width 0xFFFC
#define	MgdTEXTGID_FallbackFace		0xFFFB
#define	MgdTEXTGID_SpecialCodeMin	0xFFFB	// For quicker comparison

//=====================================================================================================================
//! Information about single glyph in a font.
struct GLYPHINFO {
	double lbearing;							//!< Pixels between glyph origin and left edge
	double rbearing;							//!< Pixels between glyph origin and right edge
	double rspacing;							//!< Pixels between glyph origin and next character
	double ascent;								//!< Pixels between glyph origin and top
	double descent;							//!< Pixels between glyph origin and bottom
	};

//=====================================================================================================================
//! Font metrics.
struct FONTINFO {
	double ascent;								//!< Ascent above baseline in pixels
	double descent;							//!< Descent below baseline in pixels
	double linespace;							//!< Spacing between baselines
	UINT16 NumGlyphs;							//!< Number of glyphs in font
	UINT16 InitChar;							//!< Initial character having glyph defined
	UINT16 LastChar;							//!< Last character having glyph defined
	UINT16 NullChar;							//!< Character to user for undefined glyphs
	};

//=====================================================================================================================
//! Font definition and implementation.
class FONT {
	public:

		enum MODE {
			MODE_MIUNICODE,							//!< Using Unicode
			MODE_RAWGID,							//!< Glyph IDs instead of Unicode
			MODE_GID									//!< Glyph IDs encoded by UCtoGID member function
			};

		enum FONTTYPE {
			FONTTYPE_OF,
			FONTTYPE_TTF
			};

		//! Read a TrueType font into memory.
		//!
		//! The "name" parameter is expected to be the filename, minus the path
		//! or extention.
		//!
		//! The FaceNum parameter is for TrueType Collections (ttc).  A ttc file
		//! contains multiple ttfs
		//! Since TrueType fonts are embeded in resource files on the Macintosh
		//! (even in OS X), a simple "filename" isn't all that useful.  This
		//! function handles reading the whole thing.
		static ERRVALUE ReadTTF (
			const MIUNICODE* name,
			SIMPLE_ARRAY<UINT8>& data
			);

		FONT();

		virtual ~FONT();
		
		virtual int DrawGlyph (
			MGD::CONTEXT* context, 
			double dx, 
			double dy, 
			double baseangle, 
			const MIUNICODE* ucstr, 
			MGD::FONT::MODE mode
			) = 0;

		virtual int DrawGlyph (
			MGD::CONTEXT* context, 
			double dx, 
			double dy, 
			double baseangle, 
			const STRINGDATA& string,
			int n
			) = 0;

		virtual double NextPosition (
			MGD::CONTEXT* context, 
			double* retx, 
			double* rety, 
			double baseangle, 
			const STRINGDATA& string,
			int n,
			double SpaceExtra = 0.0,	//!< Extra pixels per space to insert
			double GlyphExtra = 0.0
			) = 0;

		virtual int GetFontMetrics (
			MGD::CONTEXT* context, 
			MGD::FONTINFO& fontinfo
			) = 0;
		
		virtual int GetGlyphMetrics (
			MGD::CONTEXT* context, 
			MIUNICODE ch, 
			MGD::GLYPHINFO& glyphinfo, 
			MGD::FONT::MODE mode
			) = 0;

		virtual int GetGlyphExtents (
			MGD::CONTEXT* context, 
			const double& dx, 
			const double& dy, 
			const double& baseangle, 
			MIUNICODE ch, 
			DRECT2D& extents, 
			MGD::FONT::MODE mode
			) = 0;

		virtual bool IsEmbeddable (
			) const = 0;

		virtual int UCtoGID (
			MGD::CONTEXT* context, 
			const MIUNICODE* uc, 
			MGD::STRINGDATA& strdata
			) = 0;

		const MIUNICODE* GetFontName (
			) const {
			return (m_fontname);
			}

		virtual MGD::FONT::FONTTYPE GetFontType (
			) const = 0;
		
		//! Get the outline of a glyph.  This only works for TrueType
		//! fonts, but the caller won't have access to the TrueType-derived
		//! MGD::FONT_TTF.  So we make it virtual and let it return an
		//! error by default.  We could easily implement it for OF fonts
		//! too if it was ever needed.
		virtual ERRVALUE GetGlyphOutline (
			UINT16 GlyphIndex,
			REGION2D& region,
			UINT32 FontFlags
			);

		MIUNICODE GetInitChar (
			) const {
			return (m_InitChar);
			}
		
		MIUNICODE GetLastChar (
			) const {
			return (m_LastChar);
			}
		
		MIUNICODE GetNULLChar (
			) const {
			return (m_NullChar);
			}
		
		MIUNICODE GetNumGlyphs (
			) const {
			return (m_NumGlyphs);
			}

	protected:
		MISTRING m_fontname;
		FILEPATH m_filename;
		UINT16 m_NumGlyphs;					//!< Number of glyphs in font
		MIUNICODE m_InitChar;					//!< Initial character having glyph defined
		MIUNICODE m_LastChar;					//!< Last character having glyph defined
		MIUNICODE m_NullChar;					//!< Character to user for undefined glyphs
	};

//=====================================================================================================================
//! Text drawing parameters.
struct TEXTSETTINGS : public TEXTSTYLE {
	PIXEL ForegroundPixel;
	PIXEL BackgroundPixel;
	PIXEL UnderlinePixel;
	
	TEXTSETTINGS (
		) { Reset(); }
		
	double GetBoldness (
		) const { return (BoldnessWidth); }

	void Reset (
		);
		
	};

//=====================================================================================================================
//! Text block.
struct TEXTBLOCK {

	TEXTBLOCK();
	TEXTBLOCK(const TEXTBLOCK&);

	~TEXTBLOCK();

	ERRVALUE ComputeExtents (
		MGD::CONTEXT* gc,
		DRECT2D& rect
		);

	int Draw (
		MGD::CONTEXT* gc
		);

	double GetBaseAngle (
		) const { return (m_baseangle); }

	const DPOINT2D& GetBasePoint (
		) const { return (m_basepoint); }

	TEXTBASEORIGIN GetBaseOrigin (
		) const { return (m_baseorigin); }

	//! Get the first line of the text block
	ERRVALUE GetFirstString (
		MISTRING& string				//!< String returned
		);

	ERRVALUE GetFirstString (
		MGD::CONTEXT* gc,
		MISTRING& string				//!< String returned
		);

	//! Returns the HTMLDOC of the text block.
	//! If the block wasn't HTML, this returns NULL
	//! Do not free the returned pointer, but you can modify it if you like.
	HTML::HTMLDOC* GetHTMLDOC (
		);

	const TEXTSTYLE& GetStyle (
		) const { return m_Style; }

	const MISTRING& GetText (
		) const {
		return (m_Text);
		}

	bool IsAntialiased (
		) const { return m_bIsAntialiased; }

	void SetAntialiasing (
		bool value
		) { m_bIsAntialiased = value; }

	void SetBaseAngle (
		double angle = 0		//!< Base angle in radians
		);

	//! Set the base point, angle of rotation and base origin 
	void SetBasePoint (
		const DPOINT2D& pt,
		TEXTBASEORIGIN baseorigin = TEXTBASEORIGIN_FirstLine		//!< Origin of basepoint within block
		);

	//! Parse formatting codes out of text
	//! By default, textblock will look for \t, \n in text and convert them to tab, newline characters
	//! This method will turn this off
	void SetDoFormatting (bool bDoFormatting);

	//! Set the list of images referenced by the text block.
	//! Text with, for example, "{~IMG 0}" in it will use the device
	//! in Images[0] as the source of the image.
	//! IMPORTANT: The text block does not copy the devices and assumes they
	//! will stay valid for the life of the block
	void SetImages (
		const SIMPLE_ARRAY<MGD::DEVICE*>& Images
		);

	void SetStyle (
		const TEXTSTYLE& Style
		);

	//! Set the text for the text block. 
	void SetText (
		const MISTRING& text
		);

	//! Set the source file path.
	//! If the text is HTML and refers to images with relative paths, the source
	//! path is used to know what those relative paths are relative to.
	void SetSourcePath (
		const FILEPATH& filepath
		);

	bool UsesTransparency (
		) const { return m_bUsesTransparency; }

#ifndef GENERATING_DOXYGEN_OUTPUT
	private:
		class PRIV;
		friend class PRIV;
		PRIV* m_Priv;

		MISTRING m_Text;								//!< Encoded text string.
		TEXTSTYLE m_Style;							//!< Initial style
		DPOINT2D m_basepoint;						//!< Base point in display coordinates
		double m_baseangle;							//!< Rotation angle in radians
		TEXTBASEORIGIN m_baseorigin;				//!< Origin of basepoint within block
		bool m_bIsAntialiased; 						//!< set when SetUp is called.  Dave, WTH is Setup?
		bool m_bUsesTransparency; 					//!< set when SetUp is called.  Dave, WTH is Setup?
#endif
	};

//=====================================================================================================================
//! Base class for general drawing "device".
class DEVICE {
	public:

		//! Get reference to "null" device.
		//! A null device is useful for creating CONTEXT instances used for calculation of text size, etc.
		//! Any attempt to draw using the null device will be ignored.
		static DEVICE& GetNull ();

		//! Destructor
		virtual ~DEVICE ();

		//! Returns true if the derived class is based on an implementation
		//! where fill spans are considered "bad".  This is used by the
		//! implementation of text drawing functions to know if they should
		//! be drawn as fillspans of call FillPolygons
		bool AvoidFillSpans (
			) const { return (v_AvoidFillSpans()); }

		ERRVALUE Clear (
			const COLOR& color
			) { return (v_Clear(color)); }

		//! Clear the ICM Profile of the destination device.
		void ClearICMDestProfile (
			) { m_DestProfile.Free(); }

		//! Clear the ICM Profile of the proofing device
		void ClearICMProofProfile ()
			{ m_ProofProfileParms.m_bProofToScreen = false; }

		//! Compute a region which encloses all pixels with alpha above or below some treshold.
		//!
		//! Note: this method only works on devices having 8 bits per component.
		//! 	Binary devices have their own implementation.
		//! \param bintoreg The interface to use to create the region
		//! \param component Color component
		//! \param threshold The threshold value to compare to
		//! \param above If true, the region will include all pixels with value
		//!		greater than the threshold.  If false, the region will include 
		//!		all pixels with value less than the threshold
		ERRVALUE ComputeRegion (
			BINARYTOREGIONINTERFACE& bintoreg,
			COMPONENT component = COMPONENT_Alpha,
			int threshold = 128,
			bool above = true		
			) const;

		//! Change value from one SCREENUNIT to another using display for pixel size.
		//! @return Converted value.
		double ConvertScreenUnits (
			double value,								//!< Value to convert
			int iunits,									//!< Input units (SCREENUNIT_...)
			int ounits									//!< Output units (SCREENUNIT_...)
			) const { return (::ConvertScreenUnits(value,GetPixelSizeMM(),1.0,iunits,ounits)); }

		//! Create a context for drawing.
		MGD::CONTEXT* CreateContext ();

		//! Initialize depth buffer for each frame buffer.
		ERRVALUE DepthInit ();

		//! Initialize depth buffer for specified frame buffer.
		ERRVALUE DepthInit (
			UINT16 FrameBuffer					//!< Frame buffer index
			);

		//! Release depth buffer for each frame buffer.
		void DepthRelease ();

		//! Release depth buffer for specified frame buffer.
		void DepthRelease (
			UINT16 FrameBuffer					//!< Frame buffer index
			);

		//! Get pixel mask for all planes.
		PIXEL GetAllPlanesMask (
			) const { return (m_AllPlanesMask); }

		//! Return capabilities of this device.
		CAPABILITY GetCapabilities () const
			{ return (v_GetCapabilities()); }

		//! Get the ColorMap for the device.
		//! Will return NULL if the device doesn't use one.
		const COLOR_ARRAY* GetColorMap (
			) const { return (v_GetColorMap()); }

	#ifndef NO_DEPRECATED
		//! Return (closest) matching PIXEL value for COLOR.
		//! Does NOT take ICM into account.  The one in context does.
		//! Deprecated because you should call the one in the context
		//! \deprecated Call the version in the context
		DEPRECATED PIXEL GetColorPixel (
			const COLOR& color
			) { return (v_GetColorPixel(color)); }
	#endif

		//! Return the "native" COLORSPACE used by the display
		//! Image operations using this colorspace will be more efficient.
		//! Default implementation determines the colorspace from the 
		//! DeviceInfo
		COLORSPACE GetColorSpace (
			) const { return (v_GetColorSpace()); }

		//! Return the depth at a given pixel location.  
		float GetDepth (
			INT32 x, 
			INT32 y,
			UINT16 FrameBuffer
			) const;

		const DISPLAYINFO& GetDeviceInfo (
			) const { return (m_devinfo); }

		//! Get device extents.
		const LRECT2D& GetExtents (
			) const { return (m_Extents); }

		//! Get device height in pixels.
		INT32 GetHeight (
			) const { return (GetExtents().GetHeight()); }

		const PROFILE& GetICMDestProfile (
			) const { return (m_DestProfile); }

		PROFILE::INTENT GetICMIntent (
			) const { return (m_ICMIntent); }

		//! Get number of bytes per image pixel.
		int GetImageBytesPerPixel (
			) const { return (m_ImageBytesPerPixel); }

		//! Return a pointer to a piece of the image buffer with option to use virtual window.
		//! This only has to be implemented if HasCapability(CAPABILITY_GetImage)
		//! returns true.  The CopyRect() routines use it to read the source image.
		//! Note that this may return a pointer directly into the screen buffer
		//! or a temporary buffer into which it has read the requested data.  It's
		//! up to the implementation to decide how to handle this.
		UINT8* GetImagePointer (
			INT32 x,
			INT32 y,
			INT32 len,
			UINT16 FrameBuffer,
			bool UseVirtualWindow
			) {
			if (UseVirtualWindow && m_bHasVirtualWindow) {
				x -= m_offset.x;
				y -= m_offset.y;
				}
			return (v_GetImagePointer(x,y,len,FrameBuffer));
			}

		//! Return a pointer to a piece of the image buffer.
		UINT8* GetImagePointer (
			INT32 x,
			INT32 y,
			INT32 len,
			UINT16 FrameBuffer
			) { return (v_GetImagePointer(x,y,len,FrameBuffer)); }

		//! Determine if full image lines are needed (no tiled output).
		bool GetNeedFullImageLine (
			) const { return (v_GetNeedFullImageLine()); }

		//! Get number of display frame buffers.
		UINT16 GetNumFrameBuffers () const
			{ return (v_GetNumFrameBuffers()); }

	#ifndef NO_DEPRECATED
		//! Given a pixel value, return the color it represents
		//! \deprecated Call the one in the context instead
		DEPRECATED void GetPixelColor (
			PIXEL pixel,
			COLOR& color
			) const {
			v_GetPixelColor(pixel, color);
			}
	#endif

		//! Return the pixel size in millimeters, assuming square pixels.
		//! This method will cope with "this" being NULL by returning the
		//! pixel size of a NULL device
		double GetPixelSizeMM () const;

		//! Return the pixel size in millimeters without assuming squareness.
		//! This method will cope with "this" being NULL by returning the
		//! pixel size of a NULL device
		void GetPixelSizeMM (
			double& xsize,
			double& ysize
			) const;

		//! Return the pixel size in DPI, assuming square pixels.
		double GetPixelSizeDPI (
			) const { return ((1000.0 / METERSTOINCHES) / GetPixelSizeMM()); }

		//! Return the pixel size in DPI without assuming squareness
		void GetPixelSizeDPI (
			double& xsize,
			double& ysize
			) const {
			double x, y;
			GetPixelSizeMM(x, y);
			xsize = (1000.0 / METERSTOINCHES) / x;
			ysize = (1000.0 / METERSTOINCHES) / y;
			}

		//! Get device type ID.
		UINT32 GetTypeId () const
			{ return (m_TypeId); }

		INT32 GetWidth (
			) const { return (GetExtents().GetWidth()); }

		bool HasCapability (
			MGD::CAPABILITY capability
			) const { return ((v_GetCapabilities() & capability) != 0); }

		//! Determine if has depth buffer for specified frame buffer.
		bool HasDepth (
			UINT16 FrameBuffer					//!< Frame buffer index
			) const { return (FrameBuffer < m_DepthBuffers.GetNumItems() && m_DepthBuffers[FrameBuffer] != 0); }

		//! Reset any virtual window set on the device.
		//! This clears the effects of SetVirtualWindow()
		void ResetVirtualWindow ();
		
		//! Set the depth at a given pixel location.
		void SetDepth (
			INT32 x, 
			INT32 y,
			float z,
			UINT16 FrameBuffer
			);

		//! Set the ICM Profile of the destination device.
		void SetICMDestProfile (
			const PROFILE& profile,
			PROFILE::INTENT intent = PROFILE::INTENT_Perceptual
			);

		//! Set the ICM Profile of the destination device.
		void SetICMDestProfile (
			const PROFILEPARMS_PRINTER& profile
			);

		//! Set the ICM Profile of the destination device to the currently
		//! selected Screen profile (Selected in the TNTmips preferences)
		void SetICMDestProfileScreen ();

		//! Set the ICM Profile of the proofing device
		//! If set, will cause images and colors to be mapped to the gamut of
		//! the proofing device (printer) before being mapped to the gamut of
		//! the Dest device.  This way, the Dest device (screen) won't show any
		//! colors which the Proofing device (printer) is incapable of rendering.
		void SetICMProofProfile (
			const PROFILEPARMS_PRINTER& profile
			);

		//! Set the pixel size in millimeters without assuming squareness
		void SetPixelSizeMM (		
			double xsize,
			double ysize
			) { m_mmPerPixel.Set(xsize,ysize); }

		//! Setup a virtual window.
		//! Allows an MGD::DEVICE to behave as a subset of a larger device
		//! even though the real buffer only covers a part of it. dx and dy should
		//! be the coordinates of the upper-left corner of the raster in the
		//! virtual space and fullheight/fullwidth is the size of the virtual
		//! space. 
		void SetVirtualWindow (
			INT32 dx,
			INT32 dy,
			INT32 fullwidth,
			INT32 fullheight
			);
		
		//! Determine if this device supports a specific opacity blending mode.
		bool SupportsOpacityBlendMode (
			MGD::OPACITY_MODE mode
			) const { return (v_SupportsOpacityBlendMode(mode)); }

	protected:

   	#ifndef GENERATING_DOXYGEN_OUTPUT
		LRECT2D m_Extents;				// All implementations need this
		PIXEL m_AllPlanesMask;
		int m_ImageBytesPerPixel;
		DPOINT2D m_mmPerPixel;
		LPOINT2D m_TileSize;				// Tile size hint (0's if tiles not helpful)
		#endif

		//! Allocate device type ID.
		static UINT32 AllocTypeId ()
			{ ++s_LastTypeId; return (s_LastTypeId); }

		//! Constructor.
		//! Because we could get an error while allocating an MGD::DEVICE,
		//! and we don't deal with exception handling code, 
		//! the constructor should only initialize the class members.
		//! Each derived class should have a Create() method to 
		//! actually allocate memory. That would be a pure-virtual 
		//! function, but the parameters necessary are
		//! implementation dependant.  For example, the raster implementation
		//! would need to take an RVC::RASTER object or something.
		//! An X implementation would require a Drawable, etc...
		DEVICE ();

		//! Get a pointer to the depth buffer for a given location.
		//! Since this is protected, it's assumed that you've already
		//! validated the coordinates.  If the depth buffer is all in memory,
		//! this returns a pointer into that buffer.  If it's being spooled
		//! to a temporary file, it reads the requested part into a buffer
		//! and returns that buffer.  In the temporary raster case, each call
		//! to GetDepthBuffer() reuses the same buffer.  After modifying the
		//! depth buffer, you should call WriteDepthBuffer().  WriteDepthBuffer()
		//! does nothing unless the depth buffer is from a temporary raster, in
		//! which case it writes it back out.
		//! This function will return NULL if the depth buffer is not initialized
		//! or if reading from the temporary raster would cause an error.
		float* GetDepthBuffer (
			INT32 x,
			INT32 y,
			INT32 len,
			UINT16 FrameBuffer,
			bool UseVirtualWindow
			);

		//! Same as GetImagePointer() but returns a const pointer.
		const UINT8* GetImagePointerConst (
			INT32 x,
			INT32 y,
			INT32 len,
			UINT16 FrameBuffer,
			bool UseVirtualWindow
			) { return (GetImagePointer(x,y,len,FrameBuffer,UseVirtualWindow)); }

		//! Same as GetImagePointer() but returns a const pointer.
		const UINT8* GetImagePointerConst (
			INT32 x,
			INT32 y,
			INT32 len,
			UINT16 FrameBuffer
			) { return (GetImagePointer(x,y,len,FrameBuffer)); }

		//! Get image pointer without reading current contents.
		UINT8* GetImagePointerNoRead (
			INT32 x,
			INT32 y,
			INT32 len,
			UINT16 FrameBuffer,
			bool UseVirtualWindow
			) {
			if (UseVirtualWindow && m_bHasVirtualWindow) {
				x -= m_offset.x;
				y -= m_offset.y;
				}
			return (v_GetImagePointerNoRead(x,y,len,FrameBuffer));
			}

		UINT8* GetImagePointerNoRead (
			INT32 x,
			INT32 y,
			INT32 len,
			UINT16 FrameBuffer
			) { return (v_GetImagePointerNoRead(x,y,len,FrameBuffer)); }

		const LPOINT2D& GetRealSize (
			) const { return (m_RealSize); }

		const LPOINT2D& GetVirtualOffset () const
			{ return (m_offset); }

		bool HasVirtualWindow () const
			{ return (m_bHasVirtualWindow); }

		//! Sets the bits, shift, and mask parameters of the DEVICEINFO.
		void SetDeviceColorSpace (
			COLORSPACE colorspace
			);

		//! To be called by Create() method to set stuff up.
		ERRVALUE SetDeviceInfo (
			const DISPLAYINFO& dispinfo
			);

		void SetRealSize (
			int height,
			int width
			) { m_RealSize.Set(width,height); }

		//! Set device type ID.
		//! Should only be done in constructor.
		void SetTypeId (UINT32 TypeId)
			{ m_TypeId = TypeId; }

		//! Update the Depth Buffer. 
		//! Does nothing unless the Depth Buffer is initialized and 
		//! is using a raster for depth buffering.  
		ERRVALUE WriteDepthBuffer (
			INT32 x,
			INT32 y,
			INT32 len,
			const float *zbuf,
			UINT16 FrameBuffer
			);

		//! Write an image buffer to the display.
		//! Should be called after using GetImagePointer... if contents are changed.
		ERRVALUE WriteImageBuffer (
			INT32 x,
			INT32 y,
			INT32 len,
			UINT8 *buf,
			UINT16 FrameBuffer
			) { return (v_WriteImageBuffer(x,y,len,buf,FrameBuffer)); }

		//! Return a list of the capabilities of this device.
		//! Derived class should bitwise-OR with result of base method.
		virtual CAPABILITY v_GetCapabilities (
			) const;

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT
		// Unimplemented.
		DEVICE (const DEVICE&);
		DEVICE& operator= (const DEVICE&);

		friend class CONTEXT;

		class ZBUFFER;

		static UINT32 s_LastTypeId;

		DISPLAYINFO m_devinfo;
		PROFILE m_DestProfile;
		PROFILE::INTENT m_ICMIntent;
		PROFILEPARMS_PRINTER m_ProofProfileParms;
		SIMPLE_ARRAY<ZBUFFER*> m_DepthBuffers;
		LPOINT2D m_offset;
		LPOINT2D m_RealSize;
		UINT32 m_TypeId;
		bool m_bHasVirtualWindow;

		static void MCB_ScreenProfileChanged (MICALLBACKLIST::ANYCALLBACKSTRUCT*, void*);
		#endif // GENERATING_DOXYGEN_OUTPUT

		//! Returns true if the derived class is based on an implementation
		//! where fill spans are considered "bad".  This is used by the
		//! implementation of text drawing functions to know if they should
		//! be drawn as fillspans of call FillPolygons.
		//! Default implementation returns false.
		virtual bool v_AvoidFillSpans () const;

		virtual ERRVALUE v_Clear (
			const COLOR& color
			);
		
		//! Create CONTEXT for drawing.  
		//! Derived classes may override this and make it return anything
		//! derived from MGD::CONTEXT.  MGD::CONTEXT's constructor is private, so
		//! this is the only way for calling code to create a GC.
		virtual MGD::CONTEXT* v_CreateContext ();

		//! Fill a horizontal span.
		//! Many of the default method implementations call this to do all the work.
		//! Derived classes that implement this do not need to worry about clipping,
		//! as the default implementation in CONTEXT does clipping and then calls this version.
		//!
		//! If your implementation handles bitmap pattern fills in a way that's
		//! optimized for your dest device, you should return CAPABALITY_BitmapFill
		//! in GetCapabilities.
		//! Default implementation does nothing.
		virtual ERRVALUE v_FillSpan (
			MGD::CONTEXT *gc,
			INT32 x1,
			INT32 x2,
			INT32 y
			);

		//! Get color palette, default implementation returns NULL.
		virtual const COLOR_ARRAY* v_GetColorMap (
			) const;

		//! Get closest matching PIXEL to COLOR.
		//! This should NOT take ICM into account.  The context will
		//! have already translated the color through the ICM transform (if any)
		//! before calling this function.
		//! This method is not const because some derived classes may update a
		//! color map or something in the process.
		virtual PIXEL v_GetColorPixel (
			const COLOR& color
			);

		//! Return the "native" COLORSPACE used by the display.
		//! Image operations using this colorspace will be more efficient.
		//! Default implementation determines the colorspace from the 
		//! DeviceInfo
		virtual COLORSPACE v_GetColorSpace () const;

		//! Return a pointer to a piece of the image buffer.
		//! This only has to be implemented if HasCapability(CAPABILITY_GetImage)
		//! returns true.  The CopyRect() routines use it to read the source image.
		//! Note that this may return a pointer directly into the screen buffer
		//! or a temporary buffer into which it has read the requested data.  It's
		//! up to the implementation to decide how to handle this.
		virtual UINT8* v_GetImagePointer (
			INT32 x,
			INT32 y,
			INT32 len,
			UINT16 FrameBuffer
			);

		//! If GetImagePointer returns a pointer directly into the frame buffer.
		//!   this function does not need to be overridden.
		//! If GetImagePointer has to actually read something (say via an X
		//!   call to the display or reading out of a raster, then this version
		//!   should just return a temporary buffer big enough to hold a single
		//!   line.
		virtual UINT8* v_GetImagePointerNoRead (
			INT32 x,
			INT32 y,
			INT32 len,
			UINT16 FrameBuffer
			);

		//! Determine if full image lines are needed (no tiled output).
		//! Default implementation returns false.
		virtual bool v_GetNeedFullImageLine () const;

		//! Get number of frame buffers.
		//! Default implementation returns 1.
		virtual UINT16 v_GetNumFrameBuffers () const;

		//! Given a pixel value, return the color it represents.
		//! This should NOT take ICM into account.  The context will translate
		//! the color back through the inverse ICM transform (if any)
		virtual void v_GetPixelColor (
			PIXEL pixel,
			COLOR& color
			) const;

		virtual bool v_SupportsOpacityBlendMode (
			MGD::OPACITY_MODE
			) const;

		//! Determine if the device wants low-level drawing code to anti-alias using transparency.
		//! @return true if you want anti-aliased drawing, false if not. Default implementation returns true.
		//! Binary devices should override this to return false.
 		virtual bool v_UseAntiAliasing () const;
				
		//! Write an image buffer to the display.
		//! If GetImagePointer reutrns a pointer directly into the frame buffer,
		//! this method can just return 0.  If GetImagePointer has to actually
		//! read something, this method is used to write the result back out.
		//! Typically used when drawing to a raster
		virtual ERRVALUE v_WriteImageBuffer (
			INT32 x,
			INT32 y,
			INT32 len,
			UINT8 *buf,
			UINT16 FrameBuffer
			);

	};

//=====================================================================================================================
//! Base for general drawing context.
class CONTEXT {
	public:
		// Note, default Constructor is protected

		//! Base class for private data.
		//! Basically just a class with a pure virtual destructor so that we
		//!   can delete one without knowing what it is.  Used for Get/SetPrivData
		class PRIVDATA {
			public:
				PRIVDATA() {}
				virtual ~PRIVDATA() {}
			};

		//! Allocate a private data index.
		//! This is uses as an argument to GetPrivData and SetPrivData
		static int AllocPrivDataIndex ();

		static ERRVALUE TextGetFontName (
			int fonthandle,
			MISTRING& fontname
			);

		//! Return copy of font info for specified font handle
		static ERRVALUE TextGetFontInfo (
			int fonthandle,
			MGD::FONT **font
			);

		//! Open a font (.ttf or .of).
		static int TextOpenFont (
			const MIUNICODE *fontname
			);
	
		//! Destructor.
		virtual ~CONTEXT();

		//! Add to the clip mask drawing exclusion area using a rectangle.
		//! Areas OUTSIDE the rectangle will be clipped (i.e., they won't draw)
		//! The default method twiddles bits in a clip device which is used
		//! as a bitmask through which all FillSpans operate.
		//! ( Currently only used in autolabl.c, and only)
		//! Should be replaced by IntersectClipRegion()
		ERRVALUE AddExteriorToClip (
			const LRECT2D& rect
			);

		//! Add to the clip mask drawing exclusion area using a DRECT2D.
		//! Areas added will NOT draw (i.e. they WILL be clipped).
		//! The default method clears bits in a clip device which is used
		//! as a bitmask through which all FillSpans operate.
		ERRVALUE AddToClip (
			const DRECT2D& rect
			);

		//! Add to the clip mask drawing exclusion area using an LRECT2D.
		//! Areas added will NOT draw (i.e. they WILL be clipped).
		//! The default method clears bits in a clip device which is used
		//! as a bitmask through which all FillSpans operate.
		ERRVALUE AddToClip (
			const LRECT2D& rect
			);

		//! Add to the clip mask drawing exclusion area using a POLYLINE.
		//! Areas added will NOT draw (i.e. they WILL be clipped).
		//! The default method clears bits in a clip device which is used
		//! as a bitmask through which all FillSpans operate.
		ERRVALUE AddToClip (
			const POLYLINE& polyline
			);

		//! Add to the clip mask drawing exclusion area using a REGION2D.
		//! Areas added will NOT draw (i.e. they WILL be clipped).
		//! The default method clears bits in a clip device which is used
		//! as a bitmask through which all FillSpans operate.
		ERRVALUE AddToClip (
			const REGION2D& region
			);

		//! Calculate 3d depth function for given three points
		bool Calculate3dDepthFunction (
			const DPOINT3D& p1,					//!<	Points defining a plane
			const DPOINT3D& p2,
			const DPOINT3D& p3,
			const double viewPlaneDistance,	//!< View plane distance
			const bool isPerspective,			//!< Perspective or orthogonal projection
			const MAT3X3& matrix					//!< Transformation from destination rectangle to canonical view plane
			);

		//! Clear the clip mask drawing exclusion area
		void ClearClipMask ();

		//! Set the ICM Profile of the source image or colorspace
		void ClearICMSourceProfile (
			) {
			m_SourceProfile.Free();
			SetupICMTransform();
			}

		//! Compute the extents of a label without actually drawing it.
		//! This works for labels with frames.
		//!
		//! If the bCheckClip parameter is supplied and true and the label is
		//! completely outside the current clipping rectangle, this function
		//! will return 0 and the extents region won't be computed (why bother?)
		//! If the label may be visible in the current clipping rectangle,
		//! the region will be filled in and the function will return 1.
		//! If bCheckClip is false (the default), this function will always
		//! generate the extents region and return 0 or an error code < 0
		//!
		//! If the frame parameter is supplied and not NULL, the frame region
		//! will be returned in the supplied region.  When there is a frame
		//! for a label, the frame region is the same as the extents region.
		//! however, if the label has no frame, the frame region will be empty
		//! where as the extents region will just hold the extents of the text.
		ERRVALUE ComputeLabelExtents (
			REGION2D& extents,
			const MIUNICODE* text,
			const POLYLINE& BaseLine,
			const LABELSTYLE& Style,
			const POLYLINELIST& LeaderLines,
			bool bCheckClip = false,
			REGION2D* frame = 0
			);
			
		//! Compute the extents of a label without actually drawing it.
		//! This works for labels with frames.
		//!
		//! If the bCheckClip parameter is supplied and true and the label is
		//! completely outside the current clipping rectangle, this function
		//! will return 0 and the extents region won't be computed (why bother?)
		//! If the label may be visible in the current clipping rectangle,
		//! the region will be filled in and the function will return 1.
		//! If bCheckClip is false (the default), this function will always
		//! generate the extents region and return 0 or an error code < 0
		//!
		//! If the frame parameter is supplied and not NULL, the frame region
		//! will be returned in the supplied region.  When there is a frame
		//! for a label, the frame region is the same as the extents region.
		//! however, if the label has no frame, the frame region will be empty
		//! where as the extents region will just hold the extents of the text.
		ERRVALUE ComputeLabelExtents (
			REGION2D& extents,
			const TEXTBLOCK& textblock,
			const LABELSTYLE& Style,
			const POLYLINELIST& LeaderLines,
			bool bCheckClip = false,
			REGION2D* frame = 0
			);
			
		//! Compute the frame for a given label.
		//! Note that if the label does not have a frame, the region returned
		//! will be empty.
		//! If the label style indicates triangular leaders, they will be 
		//! merged into the frame region.  If plain lines are being used, you'll
		//! have to clip them to the frame and draw them yourself.
		ERRVALUE ComputeLabelFrame (
			REGION2D& frame,			//!< The frame returned
			const MIUNICODE* text,
			const POLYLINE& BaseLine,
			const LABELSTYLE& Style,
			const POLYLINELIST& LeaderLines
			);
			
		//! Change value from one SCREENUNIT to another using display for pixel size.
		//! @return Converted value.
		double ConvertScreenUnits (
			double value,								//!< Value to convert
			int iunits,									//!< Input units (SCREENUNIT_...)
			int ounits									//!< Output units (SCREENUNIT_...)
			) { return (::ConvertScreenUnits(value,m_dev->GetPixelSizeMM(),1.0,iunits,ounits)); }

		//! Copy all the settings from one CONTEXT to another.
		void Copy (
			const CONTEXT &rhs
			) { v_Copy(rhs); }

		//! Copy rectangular area from another device to this one.
		//! The upper-left corner of the source rectangle will be placed at the destination
		//! point.  Appropriate adjustments will be made for portions off the edge of the display.
		ERRVALUE CopyRect (
			const MGD::DEVICE *pDevSrc,
			const LRECT2D& RectSrc,
			const LPOINT2D& PtTgt,
			MGD::DEVICE_MEM8G *pDevAlpha = 0
			);

		//! Copy rectangular area from another device.
		//!
		//! This method is the same as the other CopyRect(), but takes a
		//! destination rectangle instead of just a point.  If necessary, the souce 
		//! image will be sampled to fit the destination rectangle.
		//!
		//! If the destination rectangle happens to be the same size as the source
		//! rectangle, it will call the simpler method.
		ERRVALUE CopyRectSampled (
			const MGD::DEVICE *pDevSrc,
			const LRECT2D& srcrect, 
			const LRECT2D& destrect,
			MGD::DEVICE_MEM8G *pDevAlpha = 0
			);

		void DepthTestDisable (
			) { SetDepthTest(MGD::DEPTHTEST_None); }

		void DepthTestEnable (
			) { SetDepthTest(MGD::DEPTHTEST_CheckAndSet); }

		//! Disable ICM Color Management.
		//!	No color management will be done until the source or destination
		//!	ICM profile is set again.
		void DisableICM (
			) { m_ICMTransform.Free(); }

		//! Draw single elliptical arc.
		ERRVALUE DrawArc (
			INT32 xcenter,								//!< X coordinate of center of arc
			INT32 ycenter,								//!< Y coordinate of center of arc
			INT32 xradius,								//!< X radius of ellipse (BEFORE ROTATION)
			INT32 yradius,								//!< Y radius of ellipse (BEFORE ROTATION)
			double startangle,						//!< Starting angle in RADIANS
			double sweepangle,						//!< Sweep angle in RADIANS
			double rotangle = 0,							//!< Rotation angle in RADIANS
			MGD::SHAPECLOSE closemode = SHAPECLOSE_None		
			) { return (v_DrawArc(xcenter, ycenter, xradius, yradius, startangle, sweepangle, rotangle, closemode)); }

		ERRVALUE DrawArc (
			double xcenter,								//!< X coordinate of center of arc
			double ycenter,								//!< Y coordinate of center of arc
			double xradius,								//!< X radius of ellipse (BEFORE ROTATION)
			double yradius,								//!< Y radius of ellipse (BEFORE ROTATION)
			double startangle,						//!< Starting angle in RADIANS
			double sweepangle,						//!< Sweep angle in RADIANS
			double rotangle = 0,							//!< Rotation angle in RADIANS
			MGD::SHAPECLOSE closemode = SHAPECLOSE_None		
			) { return (v_DrawArc(xcenter, ycenter, xradius, yradius, startangle, sweepangle, rotangle, closemode)); }

		//! Draw an arrow head.
		ERRVALUE DrawArrow (
			const DPOINT2D& EndPoint,		//!< The location of the tip of the arrow
			double angle,						//!< The direction the arrow points (radians)
			double length,						//!< Length of arrow head (pixels)
			double sweep = 30.0 * DEGTORAD,	//!< Sweep angle (radians)
			MGD::ARROWTYPE type = ARROWTYPE_FilledTriangle
			);

		//! Draw outline of a circle.
		ERRVALUE DrawCircle (
			INT32 xcenter,								//!< X coordinate of center of circle
			INT32 ycenter,								//!< Y coordinate of center of circle
			INT32 radius								//!< Radius of circle in pixels
			) { return (v_DrawCircle(xcenter, ycenter, radius)); }

		//! Draw circle outline.
		ERRVALUE DrawCircle (
			double xcenter,								//!< X coordinate of center of circle
			double ycenter,								//!< Y coordinate of center of circle
			double radius								//!< Radius of circle in pixels
			) { return (v_DrawCircle(xcenter, ycenter, radius)); }

		//! Draw outline of an ellipse.
		ERRVALUE DrawEllipse (
			INT32 xcenter,								//!< X coordinate of center of arc
			INT32 ycenter,								//!< Y coordinate of center of arc
			INT32 xradius,								//!< X radius of ellipse (BEFORE ROTATION)
			INT32 yradius,								//!< Y radius of ellipse (BEFORE ROTATION)
			double rotangle = 0							//!< Rotation angle in RADIANS
			) { return (v_DrawEllipse(xcenter, ycenter, xradius, yradius, rotangle)); }

		ERRVALUE DrawEllipse (
			double xcenter,								//!< X coordinate of center of arc
			double ycenter,								//!< Y coordinate of center of arc
			double xradius,								//!< X radius of ellipse (BEFORE ROTATION)
			double yradius,								//!< Y radius of ellipse (BEFORE ROTATION)
			double rotangle = 0							//!< Rotation angle in RADIANS
			) { return (v_DrawEllipse(xcenter, ycenter, xradius, yradius, rotangle)); }

		//! Draw single icon.
		ERRVALUE DrawIcon (
			INT32 x, 
			INT32 y, 
			MICONSTRUCT* icon, 
			ICONFLAGS flags
			) { return (v_DrawIcon(x, y, icon, flags)); }

		//! Draw single icon using ICONID and background color.
		ERRVALUE DrawIcon (
			INT32 x,
			INT32 y,
			ICONID iconid,
			const COLOR& bgcolor,
			ICONFLAGS flags
			);

		//! Draw a text label with optional frame and leader lines.
		//! @returns 1 if label was drawn, 0 if completely clipped, < 0 on error
		ERRVALUE DrawLabel (
			const MIUNICODE* text,
			const POLYLINE& BaseLine,
			const LABELSTYLE& Style,
			const POLYLINELIST& LeaderLines
			);
			
		//! Draw a text label with optional frame and leader lines.
		//! @returns 1 if label was drawn, 0 if completely clipped, < 0 on error
		ERRVALUE DrawLabel (
			const TEXTBLOCK& textblock,
			const LABELSTYLE& Style,
			const POLYLINELIST& LeaderLines
			);
			
		//! Draw a 2 point line.
		ERRVALUE DrawLine (
			INT32 x1, 
			INT32 y1, 
			INT32 x2, 
			INT32 y2
			) { return (v_DrawLine(x1, y1, x2, y2)); }

		//! Draw 3D line.
		ERRVALUE DrawLine3D (
			INT32 x1,									//!< Starting X coordinate
			INT32 y1,									//!< Starting Y coordinate
			float z1,									//!< Starting Z coordinate
			INT32 x2,									//!< Ending X coordinate
			INT32 y2,									//!< Ending Y coordinate
			float z2										//!< Ending Z coordinate
			) { return (v_DrawLine3D(x1, y1, z1, x2, y2, z2)); }

		//! Draw a polyline
		ERRVALUE DrawLines (
			const LPOINT2D* points,					//!< Point array
			INT32 numpoints,							//!< Number of points (>= 2)
			bool close = false						//!< Force closed polygon if true (Not very effecient)
			);

		//! Draw a polyline
		ERRVALUE DrawLines (
			const POLYLINE& line
			);

		//! Draw a polyline
		ERRVALUE DrawLines (
			const SIMPLE_ARRAY<LPOINT2D>& points,					//!< Point array
			bool close = false							//!< Force closed polygon if true (Not very effecient)
			) { return (DrawLines(points, points.GetNumItems(), close)); }

		//! Draw a dashed polyline.
		ERRVALUE DrawLinesDashed (
			const LPOINT2D* points,						//!< Point array
			INT32 numpoints,								//!< Number of polyline vertices
			const SIMPLE_ARRAY<double>& DashArray,	//!< Lengths and Distances of alternating dashes and open segments... 
			double DashOffset = 0.0,					//!< Start of first dash.
			bool close = false							//!< Force closed polygon if true
			) { return (v_DrawLinesDashed(points, numpoints, DashArray, DashArray.GetNumItems(), DashOffset, close)); }
		
		//! Draw a dashed polyline.
		ERRVALUE DrawLinesDashed (
			const SIMPLE_ARRAY<LPOINT2D>& points,	//!< Point array
			const SIMPLE_ARRAY<double>& DashArray,	//!< Lengths and Distances of alternating dashes and open segments... 
			double DashOffset = 0.0,					//!< Start of first dash.
			bool close = false							//!< Force closed polygon if true
			) { return (v_DrawLinesDashed(points, points.GetNumItems(), DashArray, DashArray.GetNumItems(), DashOffset, close)); }

		//! Draw a dashed polyline.
		ERRVALUE DrawLinesDashed (
			const LPOINT2D* points,						//!< Point array
			INT32 numpoints,								//!< Number of polyline vertices
			const double *DashArray,					//!< Lengths and Distances of alternating dashes and open segments... 
			INT32 DashCount,								//!< Number of items in DashArray
			double DashOffset = 0.0,					//!< Start of first dash.
			bool close = false							//!< Force closed polygon if true
			) { return (v_DrawLinesDashed(points, numpoints, DashArray, DashCount, DashOffset, close)); }
		
		//! Draw a dashed polyline.
		ERRVALUE DrawLinesDashed (
			const SIMPLE_ARRAY<LPOINT2D>& points,	//!< Point array
			const double *DashArray,					//!< Lengths and Distances of alternating dashes and open segments... 
			INT32 DashCount,								//!< Number of items in DashArray
			double DashOffset = 0.0,					//!< Start of first dash.
			bool close = false							//!< Force closed polygon if true
			) { return (v_DrawLinesDashed(points, points.GetNumItems(), DashArray, DashCount, DashOffset, close)); }

		//! Draw a dashed polyline.
		ERRVALUE DrawLinesDashed (
			const POLYLINE& line,						//!< Point array
			const double *DashArray,					//!< Lengths and Distances of alternating dashes and open segments... 
			INT32 DashCount,								//!< Number of items in DashArray
			double DashOffset = 0.0,					//!< Start of first dash.
			bool close = false							//!< Force closed polygon if true
			);
		
		//! Draw sample of line style in specified rectangle.
		ERRVALUE DrawLineStyleSample (
			const RVC::STYLE& styleobj,	//!< Open RVC STYLE object reference
			const LINESTYLE& style,			//!< Style
			const LRECT2D& rect,				//!< Rectangle to contain sample
			STYLESAMPLEFLAGS flags = STYLESAMPLE_Default
			);

		//! Draw a single pixel, using current color.
		//! This function ignores all 3D settings.  If you want 3D, call PutPixel
		ERRVALUE DrawPixel (
			INT32 x, 
			INT32 y
			);

		//! Draw a point, using current point style.
		ERRVALUE DrawPoint (
			INT32 x, 
			INT32 y
			) { return (v_DrawPoint(x, y)); }

		//! Draw a point, using current point style.
		ERRVALUE DrawPoint (
			const DPOINT2D& pt
			) { return (v_DrawPoint(pt)); }	

		//! Draw multiple points.
		ERRVALUE DrawPoints (
			const LPOINT2D* points, 
			INT32 NumPoints
			) { return (v_DrawPoints(points, NumPoints)); }

		//! Draw multiple points.
		ERRVALUE DrawPoints (
			const SIMPLE_ARRAY<LPOINT2D>& points					//!< Point array
			) { return (DrawPoints(points, points.GetNumItems())); }

		//! Draw sample of point style in specified rectangle.
		ERRVALUE DrawPointStyleSample (
			const RVC::STYLE& styleobj,	//!< Open RVC STYLE object reference
			const POINTSTYLE& style,		//!< Style
			const LRECT2D& rect,				//!< Rectangle to contain sample
			STYLESAMPLEFLAGS flags = STYLESAMPLE_Default
			);

		//! Draw sample of polygon style in specified rectangle.
		ERRVALUE DrawPolyStyleSample (
			const RVC::STYLE& styleobj,	//!< Open RVC STYLE object reference
			const POLYSTYLE& style,			//!< Style
			const LRECT2D& rect,				//!< Rectangle to contain sample
			STYLESAMPLEFLAGS flags = STYLESAMPLE_Default
			);

		//! Draw rectangle outline.
		ERRVALUE DrawRectangle (
			INT32 x1, 							//!< Upper left X
			INT32 y1, 							//!< Upper left Y
			INT32 x2, 							//!< Lower right X
			INT32 y2								//!< Lower right Y
			) { return (v_DrawRectangle(x1, y1, x2, y2)); }

		//! Draw rectangle outline.
		ERRVALUE DrawRectangle (
			const LRECT2D& rect
			) { return (v_DrawRectangle(rect)); }
		
		//! Draw rectangle outline.
		ERRVALUE DrawRectangle (
			const DRECT2D& rect
			) { return (v_DrawRectangle(rect)); }
		
		//! Draw a region.
		ERRVALUE DrawRegion (
			const REGION2D& polygons
			);

		//! Draw a rounded rectangle
		//! The if no radius is specified (or -1), the radius will be computed
		//! as 1/4 of the minimum dimension of the rectangle
		ERRVALUE DrawRoundedRectangle (
			const LRECT2D& rect,
			INT32 radius = -1
			);

		//! Draw muliple 2-point line segments.
		ERRVALUE DrawSegments (
			const LSEGMENT2D* segments, 
			INT32 NumSegments
			) { return (v_DrawSegments(segments, NumSegments)); }

		//! Draw muliple 2-point line segments.
		ERRVALUE DrawSegments (
			const SIMPLE_ARRAY<LSEGMENT2D>& segments		//!< Segment array
			) { return (v_DrawSegments(segments, segments.GetNumItems())); }

		//! Draw simple triangular arrowhead.
		//! Base of triangle will be at specified location
		void DrawSimpleArrowHead (
			INT32 xbase,						//!< X center of base
			INT32 ybase,						//!< Y center of base
			INT32 size,							//!< Size in pixels (height of triangle, 1/2 width of base)
			MGD::DIRECTION direction
			);

		//! Draw simple two-color frame using specified rectangle.
		void DrawSimpleFrame (
			const LRECT2D& rect,					//!< Rectangle for frame
			const COLOR& ColorTopLeft,
			const COLOR& ColorBottomRight
			);

		//! Draw a subregion.
		ERRVALUE DrawSubRegion (
			const SUBREGION2D& polygons
			);

		//! Erase rectangle to specified color, replacing pixels regardless of transparency.
		ERRVALUE EraseRectangle (
			const LRECT2D& rectangle,				//!< Rectangle to erase
			const COLOR& color						//!< Color to erase with
			) { return (v_EraseRectangle(rectangle,color)); }

		//! Fill single elliptical arc chord or wedge.
		ERRVALUE FillArc (
			INT32 xcenter,								//!< X coordinate of center of arc
			INT32 ycenter,								//!< Y coordinate of center of arc
			INT32 xradius,								//!< X radius of ellipse (BEFORE ROTATION)
			INT32 yradius,								//!< Y radius of ellipse (BEFORE ROTATION)
			double startangle,						//!< Starting angle in RADIANS
			double sweepangle,						//!< Sweep angle in RADIANS
			double rotangle = 0,							//!< Rotation angle in RADIANS
			MGD::SHAPECLOSE closemode = SHAPECLOSE_Direct
			) { return (v_FillArc(xcenter, ycenter, xradius, yradius, startangle, sweepangle, rotangle, closemode)); }

		ERRVALUE FillArc (
			double xcenter,								//!< X coordinate of center of arc
			double ycenter,								//!< Y coordinate of center of arc
			double xradius,								//!< X radius of ellipse (BEFORE ROTATION)
			double yradius,								//!< Y radius of ellipse (BEFORE ROTATION)
			double startangle,						//!< Starting angle in RADIANS
			double sweepangle,						//!< Sweep angle in RADIANS
			double rotangle = 0,							//!< Rotation angle in RADIANS
			MGD::SHAPECLOSE closemode = SHAPECLOSE_Direct
			) { return (v_FillArc(xcenter, ycenter, xradius, yradius, startangle, sweepangle, rotangle, closemode)); }

		//! Fill a circle
		ERRVALUE FillCircle (
			INT32 xcenter,								//!< X coordinate of center of circle
			INT32 ycenter,								//!< Y coordinate of center of circle
			INT32 radius								//!< Radius of circle in pixels
			) { return (v_FillCircle(xcenter, ycenter, radius)); }

		ERRVALUE FillCircle (
			double xcenter,								//!< X coordinate of center of circle
			double ycenter,								//!< Y coordinate of center of circle
			double radius								//!< Radius of circle in pixels
			) { return (v_FillCircle(xcenter, ycenter, radius)); }
		
		//! Fill an ellipse
		ERRVALUE FillEllipse (
			INT32 xcenter,								//!< X coordinate of center of arc
			INT32 ycenter,								//!< Y coordinate of center of arc
			INT32 xradius,								//!< X radius of ellipse (BEFORE ROTATION)
			INT32 yradius,								//!< Y radius of ellipse (BEFORE ROTATION)
			double rotangle = 0							//!< Rotation angle in RADIANS
			) { return (v_FillEllipse(xcenter, ycenter, xradius, yradius, rotangle)); }

		ERRVALUE FillEllipse (
			double xcenter,								//!< X coordinate of center of arc
			double ycenter,								//!< Y coordinate of center of arc
			double xradius,								//!< X radius of ellipse (BEFORE ROTATION)
			double yradius,								//!< Y radius of ellipse (BEFORE ROTATION)
			double rotangle = 0							//!< Rotation angle in RADIANS
			) { return (v_FillEllipse(xcenter, ycenter, xradius, yradius, rotangle)); }
		
		//! Fill a polygon.
		ERRVALUE FillPolygon (
			const LPOINT2D* points, 
			INT32 NumPoints, 
			MGD::POLYSHAPE shape = POLYSHAPE_Complex, 
			bool DrawOutline = false
			) { return (v_FillPolygon(points, NumPoints, shape, DrawOutline)); }

		//! Fill polygons.
		//! The first polygon is the polygon to fill, additional polygons are
		//! treated as islands (with possible islands inside the islands, etc)
		ERRVALUE FillPolygons (
			const LPOLYGON* polygons, 
			INT32 NumPolys, 
			bool DrawOutline = false
			) { return (v_FillPolygons(polygons, NumPolys, DrawOutline)); }
			
		//! Fill a rectangle
		ERRVALUE FillRectangle (
			INT32 x1, 		//!< Upper left X
			INT32 y1, 		//!< Upper left Y
			INT32 x2, 		//!< Lower right X
			INT32 y2			//!< Lower right Y
			) { return (v_FillRectangle(x1, y1, x2, y2)); }

		//! Fill a rectangle
		ERRVALUE FillRectangle (
			const LRECT2D& rect
			) { return (v_FillRectangle(rect)); }

		//! Fill a rectangle
		ERRVALUE FillRectangle (
			const DRECT2D& rect
			) { return (v_FillRectangle(rect)); }

		//! Fill a region.
		ERRVALUE FillRegion (
			const REGION2D& polygons,
			bool DrawOutline = false
			) { return (v_FillRegion(polygons, DrawOutline)); }

		//! Draw a rounded rectangle
		//! The if no radius is specified (or -1), the radius will be computed
		//! as 1/4 of the minimum dimension of the rectangle
		ERRVALUE FillRoundedRectangle (
			const LRECT2D& rect,
			INT32 radius,
			bool bDrawBorder = false
			);

		//! Fill a horizontal span.  
		ERRVALUE FillSpan (
			INT32 x1,
			INT32 x2,
			INT32 y
			);

		//! Fill a subregion.
		ERRVALUE FillSubRegion (
			const SUBREGION2D& polygons,
			bool DrawOutline = false
			) { return (v_FillSubRegion(polygons, DrawOutline)); }

		void Get3dDepthFunction (
			float &xcoeff,								//!< X coefficient
			float &ycoeff,								//!< Y coefficient
			float &zconst,								//!< Z constant
			bool &isPespective
			) const {
			xcoeff = m_dfx;
			ycoeff = m_dfy;
			zconst = m_dfz;
			isPespective = m_IsPerspective;
			}
			
		bool GetAntialiasing () const;

		//! Gets the current clipping region.
		void GetClipRegion (
			REGION2D& region
			) const;

		const LRECT2D& GetClipRectangle (
			) const { return (m_ClipRect); }

		//! Returns the current drawing color
		PIXEL GetColor (
			) const { return (m_DrawColorPixel); }

		bool GetColorLock (
			) const { return (m_bColorLocked); }

		//! Given a color value, return the pixel it represents.
		//!  Does ICM translation if source and dest ICM Profiles are set
		PIXEL GetColorPixel (
			const COLOR& color
			) const;

		//! Given a color value, return the pixel it represents (ignores ICM).
		PIXEL GetColorPixelNoICM (
			const COLOR& color
			) const;

		//! Return the depth at a given pixel location for current frame buffer.
		float GetDepth3D (
			INT32 x, 
			INT32 y
			) { return (m_dev->GetDepth(x,y,m_FrameBuffer)); }

		MGD::DEPTHTEST GetDepthTest () const
			{ return (m_DepthTest); }

		//! Return the Device for the context.
		MGD::DEVICE& GetDevice (
			) const { return (*m_dev); }
		
		//! Return the Device for the context.
		DEPRECATED_MSG("Use GetDevice instead") MGD::DEVICE* GetDisplay (
			) const { return (m_dev); }

		//! Get DISPLAYINFO for device attached to context.
		const DISPLAYINFO& GetDisplayInfo (
			) const { return (m_dev->GetDeviceInfo()); }

		FONT* GetFont (
			) const { return (m_font); }

		//! Get current frame buffer.
		UINT16 GetFrameBuffer () const
			{ return (m_FrameBuffer); }

		//! Get the outline of a glyph in the currently selected font.
		//! if bUseGlyphIndex it true, then uc is a TrueType glyph index rather
		//!	than a Unicode character code
		//! The returned region will have the glyph's base point at (0,0)
		ERRVALUE GetGlyphOutline (
			MIUNICODE uc,
			REGION2D& region,
			bool bUseGlyphIndex = false
			);

		//! Given a pixel value, return the color it represents.
		//!  Does ICM translation if source and dest ICM Profiles are set
		void GetPixelColor (
			PIXEL pixel,
			COLOR& color
			) const;

		//! Given a pixel value, return the color it represents (ignores ICM).
		void GetPixelColorNoICM (
			PIXEL pixel,
			COLOR& color
			) const;

		//! Return true if using drawing mode to only allow N/S/E/W movement when drawing a line
		bool GetLine4Connected () const
			{ return m_bLine4Connected; }

		MGD::LINECAP GetLineCap () const
			{ return m_LineCap; }
		
		MGD::LINEJOIN GetLineJoin () const
			{ return m_LineJoin; }

		//! Get whether or not point symbol is scaled to map.
		bool GetLineScaledToMap () const
			{ return (m_bLineScaledToMap); }

		double GetLineWidth () const
			{ return (m_LineWidth); }

	#ifndef NO_DEPRECATED
		//! Return the output map scale.
		//! \deprecated Use GetOutputScale instead.
		DEPRECATED double GetMapScale () const
			{ return (GetOutputScale()); }
	#endif

		//! Get the mode used when filling with translucent color.
		//! Note, this only applies to RGBA devices.
		OPACITY_MODE GetOpacityBlendMode (
			) const { return m_OpacityBlendMode; }

#ifndef GENERATING_DOXYGEN_OUTPUT
		OTPRIV* GetOpenTypePriv (
			);
#endif

		//! Get the current output drawing scale.
		double GetOutputScale () const
			{ return (this == 0) ? 1.0 : m_MapScale; }
		
		//! Get current pixel function.
		MGD::PIXELFUNC GetPixelFunction (
			) const { return m_PixelFunc; }

		double GetPixelSizeMM () const
			{ return (m_dev->GetPixelSizeMM()); }

		PIXEL GetPlaneMask () const
			{ return m_PlaneMask; }

		bool GetPlaneMaskLock () const
			{ return m_bPlaneMaskLocked; }

		//! Get whether or not point symbol is scaled to map.
		bool GetPointScaledToMap (
			) const {
			return (m_bPointScaledToMap);
			}

		//! Get Private data attached to the context.
		//! This is currently used by SML to attach extra data to the context.
		PRIVDATA* GetPrivData (
			int indx
			) const;
			
		//! Get filling style
		const POLYSTYLE& GetStyleFill (
			) const { return (m_FillStyle); }

		//! Get line style
		const LINESTYLE& GetStyleLine (
			) const { return (m_LineStyle); }

		//! Get point style
		const POINTSTYLE& GetStylePoint (
			) const { return (m_PointStyle); }

		MGD::TEXTSETTINGS& GetTextSettings ()
			{ return m_Text; }

		const MGD::TEXTSETTINGS& GetTextSettings () const
			{ return m_Text; }

		//! Get the current TextStyle 
		const TEXTSTYLE& GetTextStyle () const
			{ return (m_Text); }

		//! Determine if thin line width hinting is enabled.
		//! When enabled, lines less than 1 pixel wide will become partly
		//! transparent and get more transprent the thinner they are.
		bool GetThinLineWidthHint () const
			{ return (m_bThinLineWidthHint); }

		int GetTransparency () const
			{ return (m_Transparency); }

		bool GetTransparencyLock (
			) const { return (m_bTransparencyLocked); }
		
		//! Return true if using pattern/symbol for DrawPoint/DrawPoints
		bool GetUsePatternPoint (
			) const { return (m_bUsePatternPoint); }

		//! Return true if using line pattern for drawing functions
		bool GetUsePatternLine (
			) const { return (m_bUsePatternLine); }

		//! Return true if using pattern for fill functions
		bool GetUsePatternFill () const
			{ return (m_bUsePatternFill); }

		//! Returns true if the context has a clip region set which is more
		//! complex than a simple rectangle.  Some of the derived class 
		//! drawing methods fall back on the base-class methods if it is.
		bool HasComplexClip (
			) const {
			return (m_ClipRegion != 0);
			}

		//! Determine if transparency has been used for drawing.
		bool HasUsedTransparency (
			) const {
			return (m_bHasUsedTransparency);
			}

		//! Fill a polygon with a hatch pattern.
		//! This is a low-level version of the function used only by the
		//! hatch pattern editor.  Generally you'd set a style and just
		//! call FillPolygon
		ERRVALUE HatchPolygonStyle (
			const LPOINT2D *point,
			INT32 numpts
			);

		//! Tell the display what kind of data we'll provide to PutImageSpan().
		//! If it matches the colorspace of the display, PutImageSpan() will
		//! call ImagePutSpanDirect().  If not, the image will be translated
		//! to what the display wants first.
		ERRVALUE ImageBegin (
			INT32 x,										//!< X coordinate of left edge
			INT32 y,										//!< Y coordinate of top edge
			INT32 width,								//!< Width of image in pixels
			INT32 height,								//!< Height of image in pixels
			COLORSPACE colorspace,					//!< Colorspace of the data we'll be sending
			const COLOR_ARRAY* colormap = 0,			//!< Colormap for COLORSPACE_Indexed colorspaces
			UINT32 flags = 0
			);

		//! Tell the display what kind of data we'll provide to PutImageSpan().
		//! If it matches the colorspace of the display, PutImageSpan() will
		//! call ImagePutSpanDirect().  If not, the image will be translated
		//! to what the display wants first.
		ERRVALUE ImageBegin (
			const LRECT2D& rect,
			COLORSPACE colorspace,
			const COLOR_ARRAY* colormap = 0,
			UINT32 flags = 0
			) {
			return (ImageBegin(rect.xinit, rect.yinit, rect.GetWidth(), rect.GetHeight(), colorspace, colormap, flags));
			}

		//! Notify device that image is complete.
		//! Needed for some displays to flush output or terminate image operations
		ERRVALUE ImageEnd (
			UINT32 flags = 0
			);

		//! Draw "interface text" MIUNICODE* string.
		//!
		//!	This and related InterfaceText functions are used for rendering text through the
		//!	underlying graphic system.  For example, X text will be rendered using BDF fonts.
		//!	For user interfaces this will generally be much faster than using the Text or
		//!	TextBlock functions, but is subject to the limitations of the user-selected font
		//!	and style.
		//!
		//!	@param x X coordinate of basepoint of text
		//!	@param y Y coordinate of basepoint of text
		//!	@param string The string to draw
		//!	@param baseorigin Where to draw the text relative to x,y
		//!	@param bVisualOrder If true, the text is assumed to be in visual order and no BIDI flipping will be done.
		//!	@param style Interface text style
		//!
		//! Default implementation does nothing.
		//!	This method is only implemented on MGD::DEVICE_X and should not be
		//!	used anywhere WYSIWYG text is desired.  
		ERRVALUE InterfaceTextDrawString (
			INT32 x, 
			INT32 y, 
			const MIUNICODE* string, 
			TEXTBASEORIGIN baseorigin = TEXTBASEORIGIN_FirstLine,	//!< Origin location within text
			bool bVisualOrder = false,
			INTERFACETEXTSTYLE style = INTERFACETEXTSTYLE_Normal
			) {
			return (v_InterfaceTextDrawString(x, y, string, baseorigin, bVisualOrder, style));
			}

		//! Get extents of "interface text" MIUNICODE* string.
		//! Default implementation does nothing.
		//!	This method is only implemented on MGD::DEVICE_X and should not be
		//!	used anywhere WYSIWYG text is desired.  
		ERRVALUE InterfaceTextGetExtents (
			const MIUNICODE* string,					//!< String
			int *width,									//!< Width of string in pixels returned, 0 if don't need
			int *height = 0,							//!< Height in pixels returned, 0 if don't need
			int *ascent = 0,							//!< Ascent of first line in pixels returned, 0 if don't need
			INTERFACETEXTSTYLE style = INTERFACETEXTSTYLE_Normal
			) {
			return (v_InterfaceTextGetExtents(string, width, height, ascent, style));
			}

		//! Get extents of "interface text" MIUNICODE* string.
		//! Default implementation does nothing.
		//!	This method is only implemented on MGD::DEVICE_X and should not be
		//!	used anywhere WYSIWYG text is desired.  
		ERRVALUE InterfaceTextGetMetrics (
			const MIUNICODE* string,					//!< String
			TEXTMETRICS& metrics,
			INTERFACETEXTSTYLE style = INTERFACETEXTSTYLE_Normal
			) {
			return (v_InterfaceTextGetMetrics(string, metrics, style));
			}

		//! Get width of "interface text" MIUNICODE* string.
		//! Default implementation does nothing.
		//!	This method is only implemented on MGD::DEVICE_X and should not be
		//!	used anywhere WYSIWYG text is desired.  
		int InterfaceTextGetWidth (
			const MIUNICODE* string,					//!< String
			INTERFACETEXTSTYLE style = INTERFACETEXTSTYLE_Normal,
			bool bInkMetrics = true
			) {
			return (v_InterfaceTextGetWidth(string, style, bInkMetrics));
			}

	#ifndef NO_DEPRECATED
		//! Get width of "interface text" MIUNICODE* string.
		//! Default implementation does nothing.
		//!	This method is only implemented on MGD::DEVICE_X and should not be
		//!	used anywhere WYSIWYG text is desired.  
		DEPRECATED int InterfaceTextGetWidth (
			const MIUNICODE* string,					//!< String
			bool bInkMetrics
			) {
			return (v_InterfaceTextGetWidth(string, INTERFACETEXTSTYLE_Normal, bInkMetrics));
			}
	#endif

		//! Intersects the current clip region with a DRECT2D.
		//! The resulting region is made the new clip region.
		//! Only areas inside the clip region will be drawn.
		ERRVALUE IntersectClip (
			const DRECT2D& rect
			);

		//! Intersects the current clip region with an LRECT2D.
		//! The resulting region is made the new clip region.
		//! Only areas inside the clip region will be drawn.
		ERRVALUE IntersectClip (
			const LRECT2D& rect
			);

		//! Intersects the current clip region with an POLYLINE.
		//! The resulting region is made the new clip region.
		//! Only areas inside the clip region will be drawn.
		ERRVALUE IntersectClip (
			const POLYLINE& polyline
			);

		//! Intersects the current clip region with an REGION2D.
		//! The resulting region is made the new clip region.
		//! Only areas inside the clip region will be drawn.
		ERRVALUE IntersectClip (
			const REGION2D& region
			);

		bool Is3dEnabled (
			) const {
			return (GetDevice().HasCapability(CAPABILITY_3D));
			}

		//! Enable multi-pass rendering of patterns.
		void MultiPassBegin ();

		//! Determine if any more passes are necessary in multi-pass rendering.
		//! Will return true if no more passess are necessary or if not currently
		//! in multi-pass rendering mode.
		bool MultiPassIsDone () const;

		//! Call at the end of a pass in multipass rendering.
		//! If called at the end of the last pass, it wil reset multipass
		//! rendering.
		void MultiPassNext ();

		//! Reset multi-pass rendering mode.
		//! Note that this is automatically done by MultiPassNext at the end of
		//! the last pass.
		void MultiPassReset ();

		//! Return the current pass number in multi-pass rendering.
		//! Will return -1 if not doing multi-pass rendering.
		int MultiPassGetCurrentPass () const;

		//! Return the number of passes required for multi-pass rendering.
		//! This function will always return at least 1.  During the rendering
		//! of pass 0, the number of passes required may increase as new styles
		//! are seen which require more passes than previous styles.
		int MultiPassGetNumPasses () const;

	#ifndef NO_DEPRECATED
		//! Determine if a given style would cause anything to be rendered on this pass.
		//! Will always return true for the first pass or if not doing multi-pass rendering.
		//! @deprecated  Use the one that takes a RVC::STYLEPATTERN_BASE*.
		DEPRECATED bool MultiPassTestStyle (
			const LINESTYLE& style,
			const void* pattern,
			int patternbytes
			) const;
	#endif

		//! Determine if a given style would cause anything to be rendered on this pass.
		//! Will always return true for the first pass or if not doing multi-pass rendering.
		bool MultiPassTestStyle (
			const LINESTYLE& style,
			const RVC::STYLEPATTERN_BASE *pPattern = 0	//!< Pattern if used, must match pattern type specified in line style
			) const;

		//! Determine if a given style would cause anything to be rendered on
		//! this pass or not.
		//! Will always return true for the first pass or if not doing multi-pass
		//! rendering.
		bool MultiPassTestStyle (
			const POLYSTYLE& style,
			const void* pattern = 0,
			int patternbytes = 0
			) const;

		//! Put a span of image data.
		//! 
		//! \note  Before putting an image, you MUST call ImageBegin()
		//! and call ImageEnd() afterwards.  These are stub functions that
		//! do nothing in most implementations, but required by others
		//!
		//!	The mask, if provided, is one byte-per-pixel. 0 means totally transparent,
		//! 255 means totally opaque.
		ERRVALUE PutImageSpan (
			INT32 x, 
			INT32 y, 
			INT32 len, 
			const void* data, 
			const UINT8 *mask = 0 
			);

		//! Put single pixel on generic display.
		//! If depth testing is enabled, this will compute the current Z value
		//! and call PutPixel3D
		ERRVALUE PutPixel (
			INT32 x, 
			INT32 y, 
			PIXEL pixel
			);

		//! Put 3D pixel at specified coordinates.
		ERRVALUE PutPixel3D (
			INT32 x, 
			INT32 y, 
			float z, 
			PIXEL pixel
			);

		//! Reset all drawing parameters (except color) to defaults
		void Reset ()
			{ v_Reset(); }

		//! Reset check for transparency usage.
		void ResetUsedTransparency (
			) { m_bHasUsedTransparency = false; }

		//! Set depth function for 3D drawing.
		//! This sets the depth function to: \c Z = X * xcoeff + Y * ycoeff + zconst.
		//! Sets the function and calls DepthTestEnabled() if not already enabled.
		ERRVALUE Set3dDepthFunction (
			float xcoeff,								//!< X coefficient
			float ycoeff,								//!< Y coefficient
			float zconst,								//!< Z constant
			bool isPespective
			);

		//! Enable antialiased drawing.
		//!	Will be ignored on devices that don't support it.
		//!	Use GetAntialiasing() to determine the current setting.
		void SetAntialiasing (
			bool enable = true
			) { m_bAntialiasingEnabled = enable; }

		//! Set origin for bitmap pattern drawing.
		//! The origin will be SUBTRACTED from the screen coordinate when indexing into the bitmap pattern.
		//! If a rectangle is filled whose upper-left corner is the specified origin the upper-left corner
		//! of the pattern will be at the upper-left corner of the rectangle.
		void SetBitmapOrigin (
			INT32 xorigin,								//!< X origin
			INT32 yorigin								//!< Y origin
			) { v_SetBitmapOrigin(xorigin, yorigin); }

		//! Enable or disable clipping for PutXxx functions.
		//! If clipping is disabled the programmer is responsible for ensuring that the coordinates passed to
		//! the Put...() functions are within the display area.  Attempting to draw outside the display area
		//! may result in unpredictable behavior.
		void SetClipForPut (
			bool DoClipForPut = true				//!< true to perform clipping, false to disable
			) { m_bDoClipForPut = DoClipForPut; }

		//! Disable clipping.
		//! Sets the ClipRect to the device extents,
		//! calls ClearClipMask() and then UpdateClipNone()
		void SetClipNone ();

		//! Set the clipping rectangle.
		//!
		//! intersects the rectangle with the device extents
		//! and calls UpdateClipRect().  
		void SetClipRectangle (
			const LRECT2D & rect
			);
		
		//! Set the clipping rectangle.
		void SetClipRectangle (
			int xinit,
			int yinit,
			int xlast,
			int ylast
			) {
			SetClipRectangle(LRECT2D(xinit, yinit, xlast, ylast));
			}

		//! Set the clipping region.
		void SetClipRegion (
			const REGION2D& region
			);
		
		//! Set the current drawing color.
		//! Default method does nothing if the color is locked.
		//!   Then does a bitwise AND with the device's AllPlanesMask.
		//!   If the resulting color is different then the color already
		//!   set, it calls UpdateColor()
		void SetColor (
			PIXEL pixel
			);

		//! Set the current drawing color.
		//! Default method does nothing if the color is locked.
		//!   Calls SetTransparency() with the transparency componant of the color
		//!   Calls GetColorPixel() and calls the PIXEL version of SetColor with
		//!     the resulting pixel value
		//!   It then calls UpdateColor(const COLOR&)
		void SetColor (
			const COLOR& color
			);

		//! Set the current drawing color.
		//! Default method does nothing if the color is locked.
		//!   Constructs a COLOR from the COLOR::NAME and calls that version
		//!	This method is necessary to avoid compiler confusion.  Without
		//!	it, MSVC7 thinks a COLOR::NAME is the same as a PIXEL
		void SetColor (
			COLOR::NAME color
			) { SetColor(COLOR(color)); }

		//! Used during symbol rendering for vector formats like pdf/svg etc.
		//! to indicate that the color being rendered is the variable color for 
		//! the pattern/symbol.
		void SetColorIsPatternVariable (
			bool value
			) { v_SetColorIsPatternVariable(value); }

		//! Set "color locked" state.
		//! Requests to change the current drawing color or pixel will be ignored while the color is locked.
		void SetColorLock (
			bool ColorLocked						//!< true to lock, false to unlock
			) { m_bColorLocked = ColorLocked; }

		//! Set the current drawing color.
		//! Here only for compatibility with old MGD. Should be depricated. 
		//! \deprecated: Use MGD::CONTEXT::SetColor() instead
		DEPRECATED_MSG("Use SetColor() instead") void SetColorPixel (
			PIXEL pixel
			) { SetColor(pixel); }

		//! Set the depth at a given pixel location for current frame buffer.
		void SetDepth3D (
			INT32 x, 
			INT32 y,
			float z
			) { m_dev->SetDepth(x,y,z,m_FrameBuffer); }

		//! Set depth test mode.
		void SetDepthTest (
			MGD::DEPTHTEST testmode
			);
		
		void SetFont (
			MGD::FONT* font
			);

		//! Set current frame buffer.
		//! Has no effect if specified frame buffer is out of range for device.
		void SetFrameBuffer (
			UINT16 FrameBuffer					//!< Frame buffer index
			);

		//! Set the ICM Profile of the source image or colorspace
		void SetICMSourceProfile (
			const PROFILE& profile
			);

		//! Set drawing mode to only allow N/S/E/W movement when drawing a line
		void SetLine4Connected (
			bool state = true
			) { m_bLine4Connected = state; }

		//! Set the line cap and join styles.
		//! Records the settings, and calls UpdateLineAttributes() if they have changed.
		void SetLineAttributes (
			MGD::LINECAP capstyle,
			MGD::LINEJOIN joinstyle
			);

		//! Set whether or not line width is scaled to map.
		void SetLineScaledToMap (
			bool LineScaledToMap					//!< true if scaled, false if not
			) { m_bLineScaledToMap = LineScaledToMap; }

		//! Set the line width in pixels.
		//! Records the setting and calls  UpdateLineWidth() if it's changed.
		void SetLineWidth (
			double width
			);

		//! Set the line width in millimeters.
		//! Converts the width to pixels and calls SetLineWidth()
		void SetLineWidthMM (
			double width
			) { SetLineWidth(width / m_dev->GetPixelSizeMM()); }

		//! Set to not fill rightmost polygon pixels.
		void SetNoFillRightPolyPixels (
			bool NoFillRightPolyPixels
			) { m_bNoFillRightPolyPixels = NoFillRightPolyPixels; }
		
		//! Used by some code to cause fill span to output single-pixel spans
		void SetNoMultiPointSpan (
			bool state
			) { m_bNoMultiPointSpan = state; }

		//! Set the mode used when filling with translucent color.
		//! Note, this only effects RGBA devices.
		void SetOpacityBlendMode (
			OPACITY_MODE OpacityBlendMode
			) { m_OpacityBlendMode = OpacityBlendMode; }

		//! Set output scale for use by styles, symbols, etc.
		void SetOutputScale (
			double mapscale		//!< Inverse drawing scale (egL 24000 for 1:24000)
			);

		//! Set the pixel function.
		void SetPixelFunction (
			MGD::PIXELFUNC func
			);

		//! Set the pixel plane mask.
		//! Does nothing if the plane mask is locked (via SetPlaneMaskLock()).
		//! Otherwise, it records the given value bitwise ANDed with the device's
		//! AllPlanesMask and calls UpdatePlaneMask() if the setting changed.
		void SetPlaneMask (
			UINT32 mask
			);

		//! Set bitplane mask using separate RGB values.
		void SetPlaneMask (
			UINT8 redmask,
			UINT8 greenmask,
			UINT8 bluemask
			); 

		//! Set bitplane mask using separate RGB values.
		//! Same as the version of SetPlaneMask() that takes 3 values.
		//! This exists only for backwards compatibility with old code
		//! Use SetPlaneMask() instead.
		void SetPlaneMaskRGB (
			UINT8 redmask,
			UINT8 greenmask,
			UINT8 bluemask
			) { SetPlaneMask(redmask, greenmask, bluemask); }

		//! Lock or unlock bitplane mask.
		//! Requests to change the plane mask will be ignored while the mask is locked.
		void SetPlaneMaskLock (
			bool PlaneMaskLocked					//!< true to lock mask, false to unlock
			) { m_bPlaneMaskLocked = PlaneMaskLocked; }

		//! Set whether or not point symbol is scaled to map.
		//! @note This is set automatically when using SetStylePoint().
		void SetPointScaledToMap (
			bool PointScaledToMap					//!< true if scaled, false if not
			) { m_bPointScaledToMap = PointScaledToMap; }

		//! Set Private data attached to the context.
		//! This is currently used by SML to attach extra data to the context.
		void SetPrivData (
			int indx,
			PRIVDATA* data
			);

	#ifndef DEPRECATE_ALL
		//! Set filling style including pattern.
		//! @deprecated Use the version that takes RVC::STYLEPATTERN_BITMAP or RVC::STYLEPATTERN_HATCH
		DEPRECATED_MSG("Use the version that takes RVC::STYLEPATTERN_BITMAP or RVC::STYLEPATTERN_HATCH") 
		ERRVALUE SetStyleFill (
			const POLYSTYLE* style,		//!< Style to use. (can be NULL to use defaults)
			const void* pattern,		//!< Pattern data, if required
			INT32 patternbytes		//!< Number of bytes of pattern data
			);
	#endif

		//! Set filling style including pattern.
		ERRVALUE SetStyleFill (
			const POLYSTYLE* style,		//!< Style to use. 
			const RVC::STYLEPATTERN_BASE* Pattern = 0		//!< PATTERNTYPE_Bitmap or PATTERNTYPE_Hatch
			);

		//! Set the fill style to a built-in pattern fill.
		ERRVALUE SetStyleFill (
			MGD::BUILTINFILLSTYLE style,
			const COLOR& color
			);

		//! Set filling style including pattern.
		ERRVALUE SetStyleLine (
			const LINESTYLE* style,		//!< Style to use. 
			const RVC::STYLEPATTERN_BASE* Pattern = 0		//!< PATTERNTYPE_Line or PATTERNTYPE_Bitmap
			);

	#ifndef DEPRECATE_ALL
		//! Set line drawing style including pattern.
		//!
		//!	@note If the STYLEFLAG_ScaleToMap flag is set in style.StyleFlags the current output scale will be used
		//!	to scale the pattern.
		//!  @deprecated Use the version that takes RVC::STYLEPATTERN_LINE or RVC::STYLEPATTERN_BITMAP
		DEPRECATED_MSG("Use the version that takes RVC::STYLEPATTERN_LINE or RVC::STYLEPATTERN_BITMAP") 
		ERRVALUE SetStyleLine (
			const LINESTYLE* style,		//!< Style to use. (can be NULL to use defaults)
			const void* pattern,		//!< Pattern data, if required
			INT32 patternbytes		//!< Number of bytes of pattern data
			);
	#endif

		//! Set point style including optional point symbol.
		ERRVALUE SetStylePoint (
			const POINTSTYLE* style,		//!< Style to use. 
			const RVC::STYLEPATTERN_BASE* Symbol = 0	//!< Can be a STYLEPATTERN_SYMBOL or STYLEPATTERN_BITMAP
			);

	#ifndef DEPRECATE_ALL
		//! Set point drawing style including pattern.
		//!
		//!	If the STYLEFLAG_ScaleToMap flag is set in style.StyleFlags the current output scale will be used
		//!	to scale the pattern.  If no pattern is specified the points will be drawn as boxes or circles
		//!	depending on the StyleFlags.
		//!	@deprecated Use the version that takes RVC::STYLEPATTERN_SYMBOL
		DEPRECATED_MSG("Use the version that takes RVC::STYLEPATTERN_SYMBOL") 
		ERRVALUE SetStylePoint (
			const POINTSTYLE* style,	//!< Style to use. (can be NULL to use defaults)
			const void* pattern,		//!< Pattern data, if required
			INT32 patternbytes		//!< Number of bytes of pattern data
			);
	#endif

		// If you're looking for SetStyleText() it's named TextSetStyle() so that it sorts with
		// all the text methods.

		void SetTextSettings (
			const MGD::TEXTSETTINGS& settings
			) { m_Text = settings; }

		//! Set text styling from TEXTSTYLE structure.
		void SetTextStyle (
			const TEXTSTYLE& style							//!< Style parameters to use
			) { TextSetStyle(style); }

		//! Enable thin line width hinting.
		//! When enabled, lines less than 1 pixel wide will become partly
		//! transparent and get more transprent the thinner they are.
		//! This feature is off by default.
		//! It also only works if Antialiasing is enabled.
		void SetThinLineWidthHint (
			bool enable = true
			) { m_bThinLineWidthHint = enable; }

		//! Set transparency factor.
		//!
		//!	SetColor() will also set the transparency value (unless locked) using the high 8 bits of the
		//!	COLOR.flags field.
		//!
		//!	Not all generic displays support transparency.  
		//!
		//!	Does nothing if transparency is locked (via SetTransparencyLock()).
		//!	Otherwise, it records the setting and if not 0, sets a flag so that
		//! HasUsedTransparency() will return true.
		void SetTransparency (
			UINT8 transparency						//!< Transparency value, 0 = opaque, 255 = 100% transparent
			);

		//! Lock or unlock current transparency setting.
		//! Requests to change the transparency factor will be ignored while the value is locked.
		void SetTransparencyLock (
			bool TransparencyLocked				//!< true to lock, false to unlock
			) { m_bTransparencyLocked = TransparencyLocked; }

		//! Set whether to perform transparency blending in PutImage functions.
		//! This function will have no effect if not supported by the graphic display.
		void SetTranspForPutImage (
			bool DoTranspForPutImage				//!< true to perform transparency blending, false to not
			) { m_bDoTranspPutImage = DoTranspForPutImage; }

		//! Use pattern for fill functions
		void SetUsePatternFill (
			bool state = true
			) { m_bUsePatternFill = state; }

		//! Use line pattern for drawing functions
		void SetUsePatternLine (
			bool state = true
			) { m_bUsePatternLine = state; }

		//! Use pattern/symbol for DrawPoint/DrawPoints
		void SetUsePatternPoint (
			bool state = true
			) { m_bUsePatternPoint = state; }

		//! Get stipple mask.
		const STIPPLEMASK& StippleGetMask () const
			{ return (m_StippleMask); }

		//! Determine if stipple set to be used
		bool StippleIsUsed () const
			{ return (m_StippleIsUsed); }

		//! Set stipple mask.
		//! Will automatically set stipple to be used based on whether mask is defined.
		void StippleSetMask (
			const STIPPLEMASK& StippleMask	//!< Stipple mask
			);

		//! Set whether stipple set to be used.
		void StippleSetUsed (
			bool used					//!< Whether used or not, if true must have previously called StippleSetMask
			);

		//! Add to the clip mask drawing exclusion area using a string, baseline and font settings
		ERRVALUE TextAddToClip (
			const MIUNICODE *string,					//!< String to determine clip mask exclusion
			const POLYLINE& Baseline,				//!< String baseline
			MGD::BASELINE baseflags,
			MGD::FONT::MODE mode = MGD::FONT::MODE_MIUNICODE
			);

		ERRVALUE TextBlockComputeExtents (
			MGD::TEXTBLOCK& block,
			DRECT2D& extents				//!< extents in display pixels returned
			);

		ERRVALUE TextBlockDraw (
			MGD::TEXTBLOCK& block	
			);

		ERRVALUE TextBlockGetFirstString (
			MGD::TEXTBLOCK& block,
			MISTRING& string				//!< String returned
			);

		//! Draw single glyph at specified location.
		ERRVALUE TextDrawGlyph (
			double x,									//!< X coordinate of basepoint
			double y,									//!< Y coordinate of basepoint
			double baseangle,							//!< Baseline angle, counter-clockwise from horizontal in radians
			MIUNICODE character,						//!< Character whose glyph should be drawn
			MGD::FONT::MODE mode = MGD::FONT::MODE_MIUNICODE
			);

		//! Draw string along specified baseline.
		ERRVALUE TextDrawString (
			const DPOINT2D *basepoints,					//!< Points defining baseline
			int numbasepoints,						//!< Number of basepoints, if 1 will assume angle of 0
			MGD::BASELINE baseflags,							//!< Flags for determining how baseline fits basepoints
			const MIUNICODE *string,					//!< String to draw
			MGD::FONT::MODE mode = MGD::FONT::MODE_MIUNICODE
			);

		//! Draw partial string along specified baseline.
		ERRVALUE TextDrawStringPart (
			const DPOINT2D *basepoints,					//!< Points defining baseline
			int numbasepoints,						//!< Number of basepoints, if 1 will assume angle of 0
			MGD::BASELINE baseflags,							//!< Flags for determining how baseline fits basepoints
			const MIUNICODE *string,					//!< String to draw
			int startindex = 0,						//!< Starting character index in string (0 for beginning)
			int endindex = 0,							//!< Ending index, 0 to draw to end of string
			MGD::FONT::MODE mode = MGD::FONT::MODE_MIUNICODE
			);

		//! Draw string given starting point and baseline angle
		ERRVALUE TextDrawStringSimple (
			double x,									//!< X coordinate of basepoint
			double y,									//!< Y coordinate of basepoint
			double baseangle,							//!< Baseline angle, counter-clockwise from horizontal in radians
			const MIUNICODE *string,					//!< String to draw
			MGD::FONT::MODE mode = MGD::FONT::MODE_MIUNICODE
			);

		//! Retrieve metrics (ascent, descent, etc) for entire font.
		ERRVALUE TextGetFontMetrics (
			MGD::FONTINFO& fontinfo					//!< Font "metrics" returned
			);

		//! Compute extents for glyph as it would be drawn (assuming 0-width lines).
		ERRVALUE TextGetGlyphExtents (
			const double& x,							//!< X coordinate of starting point
			const double& y,							//!< Y coordinate of starting point
			const double& baseangle,				//!< Baseline angle, counter-clockwise from horizontal in radians
			MIUNICODE ch,									//!< Character used to select glyph
			DRECT2D& rect,								//!< Extents in display coordinates returned
			MGD::FONT::MODE mode = MGD::FONT::MODE_MIUNICODE
			) {
			return (GetFont()->GetGlyphExtents(this, x, y, baseangle, ch, rect, mode));
			}

		//! Retrieve metrics (ascent, descent, etc) for a specific glyph.
		ERRVALUE TextGetGlyphMetrics (
			MIUNICODE ch,									//!< Character used to select glyph
			MGD::GLYPHINFO& glyphinfo,				//!< Glyph metrics returned
			MGD::FONT::MODE mode = MGD::FONT::MODE_MIUNICODE
			) {
			return (GetFont()->GetGlyphMetrics(this, ch, glyphinfo, mode));
			}

		//! Compute extents for string as it would be drawn (assuming 0-width lines).
		ERRVALUE TextGetStringExtents (
			const DPOINT2D *basepoints,					//!< Points defining baseline
			int numbasepoints,						//!< Number of basepoints, if 1 will assume angle of 0
			MGD::BASELINE baseflags,							//!< Flags for determining how baseline fits basepoints
			const MIUNICODE *string,					//!< String to determine extents for
			DRECT2D& extents,							//!< Extents returned
			MGD::FONT::MODE mode = MGD::FONT::MODE_MIUNICODE
			);

		//! Compute extents extents for string allowing for justification.
		ERRVALUE TextGetStringExtentsJustified (
			const DPOINT2D *basepoints,					//!< Points defining baseline
			int numbasepoints,						//!< Number of basepoints, if 1 will assume angle of 0
			MGD::BASELINE baseflags,							//!< Flags for determining how baseline fits basepoints
			const MIUNICODE *string,					//!< String to determine extents for
			DRECT2D& extents,							//!< Extents returned
			MGD::FONT::MODE mode = MGD::FONT::MODE_MIUNICODE
			);

		//! Compute extents for string given starting point and baseline angle.
		ERRVALUE TextGetStringExtentsSimple (
			double x,									//!< X coordinate of basepoint
			double y,									//!< Y coordinate of basepoint
			double baseangle,							//!< Baseline angle, counter-clockwise from horizontal in radians
			const MIUNICODE *string,					//!< String to determine extents for
			DRECT2D& extents,							//!< Extents returned
			MGD::FONT::MODE mode = MGD::FONT::MODE_MIUNICODE
			);

		//! Compute extents for string given starting point and baseline angle allowing for justification.
		ERRVALUE TextGetStringExtentsSimpleJustified (
			double x,									//!< X coordinate of basepoint
			double y,									//!< Y coordinate of basepoint
			double baseangle,							//!< Baseline angle, counter-clockwise from horizontal in radians
			const MIUNICODE *string,					//!< String to determine extents for
			DRECT2D& extents,							//!< Extents returned
			MGD::FONT::MODE mode = MGD::FONT::MODE_MIUNICODE
			);

		//! Retrieve composite metrics (maximum ascent, descent etc.) about a given string.
		ERRVALUE TextGetStringMetrics (
			const MIUNICODE *string,					//!< String to determine metrics for
			MGD::GLYPHINFO& glyphinfo,				//!< Composite glyph information returned
			MGD::FONT::MODE mode = MGD::FONT::MODE_MIUNICODE
			);

		//! Determine width of string in pixels assuming horizontal baseline.
		ERRVALUE TextGetStringWidth (
			const MIUNICODE *string,					//!< String
			double& width,								//!< Width in pixels returned
			MGD::FONT::MODE mode = MGD::FONT::MODE_MIUNICODE
			);

		//! Set thickness of text clip width
		void TextSetClipWidth (
			double percent				//!< Percent of height to use
			);
			
		//! Set text foreground, background and underline colors using pixel values.
		void TextSetColors (
			PIXEL fgpixel,							//!< Forground pixel value
			PIXEL bgpixel,							//!< Pixel value for shadows and "enhancement" outline
			PIXEL ulpixel							//!< Pixel value for underline
			);

		//! Set text foreground, background and underline colors.
		//! Converts the COLOR values to pixels and calls the TextSetColors
		//! method that takes pixels.
		void TextSetColors (
			const COLOR& fgcolor,			//!< Foreground color
			const COLOR& bgcolor,			//!< Background color for shadows and "enhancement" outline
			const COLOR *ulcolor = 0		//!< Color for underline, will use fgcolor if NULL
			); 

		//! Set thickness of enhanced text borders.
		void TextSetEnhanceWidth (
			double width,						//!< Width of "enhancement" border in percentage of glyph height (1.0 = '100%')
			bool ValueInPixels = true
			);

		//! Set font for text drawing.
		//!
		//! @see TextOpenFont, FontTransIndexToHandle
		ERRVALUE TextSetFont (
			int fonthandle								//!< Font "handle"
			);

		//! Set font for drawing by name.
		//! If pFontFlagsFound is not 0 and the FONTSTYLE_DoFontSub flag is set
		//! in FontFlagsRequested, it will look for specific variations and
		//! do substitution.  For example, if fontname is "Times New Roman"
		//! and FONTSTYLE_Bold|FONTSTYLE_DoFontSub, it may actually open
		//! "Times New Roman Bold" and pFontFlagsFound will tell you that it
		//! found FONTSTYLE_Bold.
		ERRVALUE TextSetFontByName (
			const MIUNICODE *fontname,							//!< Font name
			UINT32 FontFlagsRequested = 0,
			UINT32* pFontFlagsFound = 0
			);

		//! Set text glyph height.
		void TextSetHeight (
			double height,								//!< Text glyph height (ascent) in pixels
			SCREENUNIT units = SCREENUNIT_Pixels
			);

		//! Set the OpenType features to use on the current font (if the font supports them).
		//! This method takes a string of space-seperated 4-letter codes of
		//! OpenType features.  When using a TrueType font, it will look for
		//! and activate the given features.
		//!
		//! A special code of "default" will reset the features back to their
		//! default state.  
		void TextSetOpenTypeFeatures (
			const MISTRING& features
			);

		//! Set shadow position relative to foreground.
		void TextSetShadowValues (
			double offset,								//!< Offset in percentage of glyph height (1.0 = '100%')
			double angle,								//!< Angle counter-clockwise from right in radians
			bool relative								//!< True if relative to baseline, false if absolute
			);

		//! Set thickness of stroke lines and outlined text.
		void TextSetStrokeWidth (
			double width,								//!< Thickness in percentage of glyph height (1.0 = '100%'), 0 for thinnest possible lines
			bool ValueInPixels = true
			);

		//! Set text styling from TEXTSTYLE structure.
		void TextSetStyle (
			const TEXTSTYLE& style							//!< Style parameters to use
			);

		//! Set underline position and thickness.
		void TextSetUnderlineValues (
			double offset,								//!< Offset between baseline and center of underline  in percentage of glyph height (1.0 = '100%')
			double thickness							//!< Thickness of underline  in percentage of glyph height (1.0 = '100%')
			);

		int TTFComputeGlyphExtentsByIndex (
			UINT16 GlyphIndex,
			MGD::GLYPHINFO& glyphinfo				//!< Glyph metrics returned
			);

		//! Draw a given glyph given its glyph index.
		//! If the current font is not a TrueType font, then "idx" is really a
		//!  Unicode value
		int TTFDrawGlyphByIndex (
			double x,
			double y,
			double baseangle,
			UINT16 idx
			);

		//! Convert a glyph index to character code.
		int TTFGlyphToChar (
			UINT16 GlyphIndex,
			MIUNICODE& Char
			);

	protected:
		MGD::DEVICE* m_dev;
		bool m_bUsePatternPoint;		//!< Use pattern/symbol for DrawPoint/DrawPoints
		bool m_bUsePatternLine;			//!< Use pattern for drawing functions
		bool m_bUsePatternFill;			//!< Use pattern for fill functions
		bool m_bLine4Connected;			//!< Draw "thin" lines 4-connected
		bool m_bColorLocked;				//!< Force patterns to draw entirely in DrawColorPixel
		bool m_bPatternFromString;		//!< Set if pattern was built from "query string"
		bool m_bTransparencyLocked;	//!< Set if transparency value is "locked"
		bool m_bHasUsedTransparency;	//!< True if we've done any transparency
		bool m_bPlaneMaskLocked;		//!< Plane mask is locked and cannot be changed
		bool m_bDoClipForPut;			//!< Perform clipping for Put...() functions
		bool m_bDoTranspPutImage;		//!< Perform transparency blending in PutImageSpan
		bool m_bDepthVariable;			//!< 3-D Depth is variable (using function)
		bool m_bPointScaledToMap;		//!< Point was scaled to map in last style setting
		bool m_bLineScaledToMap;		//!< Line was scaled to map in last style setting
		bool m_bNoMultiPointSpan;
		bool m_bNoFillRightPolyPixels;

		PIXEL	m_DrawColorPixel;
		PIXEL	m_PlaneMask;
		double	m_LineWidth;					//!< Line width in PIXELS

		POINTSTYLE	m_PointStyle;
		LINESTYLE	m_LineStyle;
		POLYSTYLE	m_FillStyle;
		double		m_MapScale;				//!< Map scale to use for patterns if required

		MGD::FONT	*m_font;						//!< Font pointer
		MGD::TEXTSETTINGS m_Text;

		//SIMPLE_ARRAY<UINT8> m_pattbuf;
		RVC::STYLEPATTERN_BASE* m_pFillPattern;
		RVC::STYLEPATTERN_LINE* m_pLinePattern;
		RVC::STYLEPATTERN_SYMBOL* m_pPointSymbol;
		RVC::STYLEPATTERN_BITMAP* m_pPointBitmap;

		INT32		m_bitmapxorigin;				//!< Subtract from coordinates when doing bitmap fills
		INT32		m_bitmapyorigin;				//!< Subtract from coordinates when doing bitmap fills
		UINT8		m_Transparency;				//!< Amount of transparency, 0 is opaque
		float		m_dfx, m_dfy, m_dfz;			//!< Depth function coefficients for 3-D
		INT32 m_xsize, m_ysize;					//!< Rounded X/Y size in pixels
		INT32 m_xsize2, m_ysize2;				//!< Rounded X/Y size/2 in pixels

		DEVICE_MEM1_SPARSE *m_pClipDevice;				//!< Clipping mask as a BITSET_SHARED
		CONTEXT *m_pClipContext;
		REGION2D *m_ClipRegion;
		bool m_bNeedToUpdateClipMask;			//!< Gets set if ClipRegion is modified and m_pClipDevice needs to be updated
		
		MGD::LINECAP m_LineCap;
		MGD::LINEJOIN m_LineJoin;
		MGD::DEPTHTEST m_DepthTest;
		MGD::PIXELFUNC m_PixelFunc;

		//! Colorspace and possibly colormap for PutImageSpan
		COLOR_ARRAY m_PutImageColorMap;
		COLORSPACE m_PutImageColorSpace;
		int m_PutImageBytesPerPixel;

		LRECT2D m_ClipRect;
		COLORCOMPSTRUCT *m_ColorComp;
		SIMPLE_ARRAY<UINT8> m_ColorConvBuf;
		SIMPLE_ARRAY<UINT8> m_ICMConvBuf;

		bool m_IsPerspective;					//!< Projection: true for perspective and false for parallel.
		SIMPLE_ARRAY<LPOINT2D> m_WorkPoints;
		PROFILE m_SourceProfile;
		TRANSFORM m_ImageICMTransform;
		TRANSFORM m_ICMTransform;
		mutable TRANSFORM m_ICMTransformInv;	//!< Initialization defered to first call to GetPixelColor() that needs it

		//! Polygon filling work buffers
		struct EDGELIST;
		SIMPLE_ARRAY<EDGELIST> m_EdgeList;
		SIMPLE_ARRAY<INT32> m_etptr;
		SIMPLE_ARRAY<INT32> m_lptr;
		SIMPLE_ARRAY<LPOLYGON> m_PolygonArray;
		SIMPLE_ARRAY<LPOINT2D> m_PointArray;
		bool m_bAntialiasingEnabled;
		bool m_bThinLineWidthHint;

		//! Default Constructor 
		CONTEXT (
			MGD::DEVICE* disp
			);

		ERRVALUE ClipRegionUpdateMask ();

		//! Copy a rectangular area from one MGD::DEVICE to this one with sampling.
		//! The source device does not have to be the same colorspace.
		//! If the srcrect and destrect are not the same sizes, the image will be
		//! resampled as needed.
		//! This is implemented using ImageBegin()/PutImageSpan() in a loop/ImageEnd()
		ERRVALUE CopyRectSampledLow (
			const MGD::DEVICE *pDevSrc,
			const LRECT2D& srcrect, 
			const LRECT2D& destrect,
			MGD::DEVICE_MEM8G *pDevAlpha = 0
			);

		//! Calculate depth buffer value from Z value.
		float DepthCalc (
			float z
			) const { return (m_IsPerspective ? (1.0F / z) : z); }

		bool DepthTest (
			float z,
			float bufz
			) const { return (DepthCalc(z) >= bufz); }

		//! Draw the nth glyph in gidstr at the given location.
		//! The gidstr.m_str is the string being drawn (in Unicode)
		ERRVALUE _DrawGlyph (
			MGD::FONT* font,
			double x,
			double y, 
			double baseangle,
			const MGD::STRINGDATA& gidstr,
			int num
			) {
			return (v_DrawGlyph(font, x, y, baseangle, gidstr, num));
			}

		//! The default DrawLines implementation will call this method
		//! after having closed the polygon (if requested) and possibly
		//! clipping the lines (if necessary)
		//!
		//! This method is also guarenteed to never get passed a line with
		//! less than 2 points or have to deal with line patterns.  The 
		//! line patterns are decomposed into simple lines which are drawn
		//! via this function
		ERRVALUE DrawLinesLow (
			const LPOINT2D* points,
			INT32 numpts
			) {
			return (v_DrawLinesLow(points, numpts));
			}

		//! The default DrawLines implementation will call this method
		//! after having closed the polygon (if requested) and possibly
		//! clipping the lines (if necessary)
		//!
		//! This method is also guarenteed to never get passes a line with
		//! less than 2 points or have to deal with line patterns.  The 
		//! line patterns are decomposed into simple lines which are drawn
		//! via this function.
		ERRVALUE DrawLinesLow (
			const POLYLINE& line
			) { return (v_DrawLinesLow(line)); }

		//! SpaceExtra and GlyphExtra are added between each space and each
		//!  glyph (including spaces) to acheive "full" justification.
		//!
		//!  If overwritten by a derived class, the new method is responsible
		//!  for drawing the underline, enhanced mode, and shadow.
		ERRVALUE DrawStringStraight (
			const DPOINT2D& pt,	//!< Base point for the string
			double baseangle,
			const MGD::STRINGDATA& gidstr,
			int sindex,		//!< The starting glyph index to draw
			int eindex,		//!< The last glyph index to draw
			double SpaceExtra,
			double GlyphExtra
			) {
			return (v_DrawStringStraight(pt, baseangle, gidstr, sindex, eindex, SpaceExtra, GlyphExtra));
			}

		//! Return the colorspace we expect for PutImageSpan().
		//! This is set by calling ImageBegin()
		COLORSPACE GetImageColorSpace (
			) const {
			return (m_PutImageColorSpace);
			}

		int GetStringExtents (
			const DPOINT2D* baseline,
			int numpts,
			double baseangle,
			MGD::BASELINE baseflags,
			const MIUNICODE* string,
			DRECT2D& extents,
			MGD::FONT::MODE mode,
			bool TakeJustificationIntoAccount
			);
			
		int GetStringExtents (
			const DPOINT2D* baseline,
			int numpts,
			double baseangle,
			MGD::BASELINE baseflags,
			const MGD::STRINGDATA& bidstr,
			int startindex,
			int endindex,
			DRECT2D& extents,
			bool TakeJustificationIntoAccount
			);
			
		int GetStringExtentsAlongCurve (
			const DPOINT2D* baseline,
			int numpts,
			const MGD::STRINGDATA& bidstr,
			int startindex,
			int endindex,
			UINT32 baseflags,
			DRECT2D& extents
			);
		
		//! If it matches the colorspace of the display, PutImageSpan() will
		//! call ImagePutSpanDirect().  If not, the image will be translated
		//! to what the display wants first.  This function is called by
		//! ImageBegin.
		ERRVALUE ImageSetColorSpace (
			COLORSPACE colorspace,
			const COLOR_ARRAY* colmap = 0			//!< Colormap for COLORSPACE_Indexed colorspaces
			);

		//! Update the x and y parameters as if the nth glyph in gidstr had just
		//! been rendered.
		//! The default implementation just calls the font->NextPosition() method,
		//! which uses the m_pos[] settins in the gidstr.   
		//! Returns the change in x (used to calculate string with by passing
		//!  base angle of 0)
		//! SpaceExtra and GlyphExtra are added between each space and each
		//!  glyph (including spaces) to acheive "full" justification.
		double _NextGlyphPosition (
			MGD::FONT *font,
			double *x,
			double *y,
			double baseangle,
			const MGD::STRINGDATA& gidstr,
			int num,
			double SpaceExtra = 0.0,
			double GlyphExtra = 0.0
			) { return (font->NextPosition(this, x, y, baseangle, gidstr, num, SpaceExtra, GlyphExtra)); }

		//! Put a span of image data.
		//! 
		//! Note:  Before putting an image, you MUST call ImageBegin
		//! and call ImageEnd afterwards.  These are stub functions that
		//! do nothing in most implementations, but required by others
		//!
		//! This function should assume the data passed to it is exactly what it wants.
		//! The caller will actually call PutImageSpan(), and ImageBegin() will cause 
		//! PutImageSpan to either call this directly or first translate the data from
		//! one display type to what we really want.
		ERRVALUE PutImageSpanDirect (
			INT32 x, 
			INT32 y, 
			INT32 len, 
			const void* data, 
			const UINT8 *mask = 0
			);

		//! Test single point for drawing with stipple.
		//! @return true if stipple not used or if should draw at specified point.
		bool StippleTest (
			INT32 x,
			INT32 y
			) const { return (!m_StippleIsUsed || m_StippleMask.TestPoint(x,y)); }

		//! Copy all the settings from one CONTEXT to another
		virtual void v_Copy (
			const CONTEXT &rhs
			);

		//! Copy rectangular area from another device to this one.
		//! The upper-left corner of the source rectangle will be placed at the destination
		//! point.  Appropriate adjustments will have been made for portions off the edge of the display.
		virtual ERRVALUE v_CopyRect (
			const MGD::DEVICE *pDevSrc,
			const LRECT2D& RectSrc,
			const LPOINT2D& PtTgt,
			MGD::DEVICE_MEM8G *pDevAlpha = 0
			);

		//! Draw single elliptical arc.
		//!	
		//! Default implementation converts the arc into a polyline and calls 
		//! DrawLines()
		virtual ERRVALUE v_DrawArc (
			INT32 xcenter,								//!< X coordinate of center of arc
			INT32 ycenter,								//!< Y coordinate of center of arc
			INT32 xradius,								//!< X radius of ellipse (BEFORE ROTATION)
			INT32 yradius,								//!< Y radius of ellipse (BEFORE ROTATION)
			double startangle,						//!< Starting angle in RADIANS
			double sweepangle,						//!< Sweep angle in RADIANS
			double rotangle = 0,							//!< Rotation angle in RADIANS
			MGD::SHAPECLOSE closemode = SHAPECLOSE_None		
			); 

		virtual ERRVALUE v_DrawArc (
			double xcenter,								//!< X coordinate of center of arc
			double ycenter,								//!< Y coordinate of center of arc
			double xradius,								//!< X radius of ellipse (BEFORE ROTATION)
			double yradius,								//!< Y radius of ellipse (BEFORE ROTATION)
			double startangle,						//!< Starting angle in RADIANS
			double sweepangle,						//!< Sweep angle in RADIANS
			double rotangle = 0,							//!< Rotation angle in RADIANS
			MGD::SHAPECLOSE closemode = SHAPECLOSE_None		
			); 
		
		//! Draw outline of a circle.
		//!
		//! Default implementation either uses Brezenham's circle drawing 
		//! algorithm or calls DrawArc with a complete \f$ 0 - 2\pi\f$ arc.
		//! It calls DrawArc if line thickness is < 1.5 and antialiasing is
		//! turned on, or if the line thickess is >= 1.5 and radius > 100,
		//! or if the device return true for AvoidFillSpans
		//! that it prefers line drawing to span filling.
		virtual ERRVALUE v_DrawCircle (
			INT32 xcenter,								//!< X coordinate of center of circle
			INT32 ycenter,								//!< Y coordinate of center of circle
			INT32 radius								//!< Radius of circle in pixels
			);

		virtual ERRVALUE v_DrawCircle (
			double xcenter,								//!< X coordinate of center of circle
			double ycenter,								//!< Y coordinate of center of circle
			double radius								//!< Radius of circle in pixels
			);

		//! Draw outline of an ellipse.
		//! Default implementation calls DrawArc with a complete \f$ 0 - 2\pi\f$ arc
		virtual ERRVALUE v_DrawEllipse (
			INT32 xcenter,								//!< X coordinate of center of arc
			INT32 ycenter,								//!< Y coordinate of center of arc
			INT32 xradius,								//!< X radius of ellipse (BEFORE ROTATION)
			INT32 yradius,								//!< Y radius of ellipse (BEFORE ROTATION)
			double rotangle = 0							//!< Rotation angle in RADIANS
			);

		virtual ERRVALUE v_DrawEllipse (
			double xcenter,								//!< X coordinate of center of arc
			double ycenter,								//!< Y coordinate of center of arc
			double xradius,								//!< X radius of ellipse (BEFORE ROTATION)
			double yradius,								//!< Y radius of ellipse (BEFORE ROTATION)
			double rotangle							//!< Rotation angle in RADIANS
			);

		//! Draw the nth glyph in gidstr at the given location.
		//! The default implementation just calls the font->DrawGlyph() method.
		//! Special devices could elect to output the glyph itself.
		//! The gidstr.m_str is the string being drawn (in Unicode)
		virtual ERRVALUE v_DrawGlyph (
			MGD::FONT* font,
			double x,
			double y, 
			double baseangle,
			const MGD::STRINGDATA& gidstr,
			int num
			);

		//! Draw a 2 point line
		//!
		//! Default implemementation just calls DrawLines() with a 2 point line
		virtual ERRVALUE v_DrawLine (
			INT32 x1, 
			INT32 y1, 
			INT32 x2, 
			INT32 y2
			);

		//! Draw 3D line.
		//!
		//! Default implementation does 3d transformation and calls DrawLine()
		virtual ERRVALUE v_DrawLine3D (
			INT32 x1,									//!< Starting X coordinate
			INT32 y1,									//!< Starting Y coordinate
			float z1,									//!< Starting Z coordinate
			INT32 x2,									//!< Ending X coordinate
			INT32 y2,									//!< Ending Y coordinate
			float z2										//!< Ending Z coordinate
			);
		
		//! Draw dashed polyline.
		virtual ERRVALUE v_DrawLinesDashed (
			const LPOINT2D *points,					//!< Point array
			INT32 numpoints,							//!< Number of points
			const double *DashArray,				//!< Alternating dash lengths/spacings
			INT32 DashCount,							//!< Number of DashArray items
			double DashOffset = 0.0,				//!< Start of first dash.
			bool close = false						//!< Force closed polygon if true (Not very effecient)
			);

		//! Derived classes can override this instead of v_DrawLines.
		//! The default DrawLines implementation will call this method
		//! after having closed the polygon (if requested) and possibly
		//! clipping the lines (if necessary)
		//!
		//! This method is also guarenteed to never get passed a line with
		//! less than 2 points or have to deal with line patterns.  The 
		//! line patterns are decomposed into simple lines which are drawn
		//! via this function
		virtual ERRVALUE v_DrawLinesLow (
			const LPOINT2D* points,
			INT32 numpts
			);

		//! Derived classes can override this instead of v_DrawLines.
		//! The default DrawLines implementation will call this method
		//! after having closed the polygon (if requested) and possibly
		//! clipping the lines (if necessary)
		//!
		//! This method is also guarenteed to never get passes a line with
		//! less than 2 points or have to deal with line patterns.  The 
		//! line patterns are decomposed into simple lines which are drawn
		//! via this function
		//!
		//! Note: The default version of this method converts the POLYLINE
		//! into an array of LPOINT2Ds and calls the other version of this
		//! virtual method.  If you override this method, you must also 
		//! override the other method because either could be called.
		//!
		//! The only reason to override this method is if your device would
		//! benifit from getting passed floating-point coordinates or could
		//! skip a step by doing its own rounding (X has to convert points
		//! to 16-bit integers, so it overrides this to skip converting them
		//! to LPOINT2Ds first)
		virtual ERRVALUE v_DrawLinesLow (
			const POLYLINE& line
			);

		//! Draw a single pixel, using current color.
		//!
		//! Default implementation calls FillSpan() with a single point span
		//! This function completely ignores 3D
		virtual ERRVALUE v_DrawPixel (
			INT32 x, 
			INT32 y
			); 

		//! Draw a point, using current point style.
		//! Default implementation calls either DrawPointPattern() or DrawPixel(),
		//! depending on current context settings.
		virtual ERRVALUE v_DrawPoint (
			INT32 x, 
			INT32 y
			);

		//! Draw a point, using current point style.
		//! Override if you can make use of floating point coordinates.
		//! Otherwise let the default implementation call the other
		virtual ERRVALUE v_DrawPoint (
			const DPOINT2D& pt
			);

		//! Draw a bunch of points.
		//!
		//! The default implementation just calls DrawPoint for each point, but
		//! could be overridden to use a more efficiant method if the display
		//! can take more than one at a time.
		virtual ERRVALUE v_DrawPoints (
			const LPOINT2D* points, 
			INT32 NumPoints
			);
		
		//! Draw rectangle outline.
		//! Default implementation calls DrawLines()
		virtual ERRVALUE v_DrawRectangle (
			INT32 x1, 							//!< Upper left X
			INT32 y1, 							//!< Upper left Y
			INT32 x2, 							//!< Lower right X
			INT32 y2								//!< Lower right Y
			);

		//! Draw rectangle outline.
		//! Default implementation calls v_DrawRectangle(x1,x2,y1,y2)
		virtual ERRVALUE v_DrawRectangle (
			const LRECT2D& rect
			);

		//! Draw rectangle outline.
		//!
		//! Default implementation calls v_DrawRectangle(x1,x2,y1,y2)
		virtual ERRVALUE v_DrawRectangle (
			const DRECT2D& rect
			);

		//! Draw multiple 2-point line segments.
		//!
		//! DefaultImplementation calls DrawLines for each segment
		virtual ERRVALUE v_DrawSegments (
			const LSEGMENT2D* segments, 
			INT32 NumSegments
			);

		//! SpaceExtra and GlyphExtra are added between each space and each
		//!  glyph (including spaces) to acheive "full" justification.
		//!
		//!  If overwritten by a derived class, the new method is responsible
		//!  for drawing the underline, enhanced mode, and shadow.
		virtual ERRVALUE v_DrawStringStraight (
			const DPOINT2D& pt,	//!< Base point for the string
			double baseangle,
			const MGD::STRINGDATA& gidstr,
			int sindex,		//!< The starting glyph index to draw
			int eindex,		//!< The last glyph index to draw
			double SpaceExtra,
			double GlyphExtra
			);

		//! Erase rectangle to specified color, replacing pixels regardless of transparency.
		//! Default implementation uses FillRectangle.
		//! Devices supporting alpha in pixel should override.
		virtual ERRVALUE v_EraseRectangle (
			const LRECT2D& rectangle,				//!< Rectangle to erase
			const COLOR& color						//!< Color to erase with
			);

		//! Fill single elliptical arc chord or wedge.
		//!
		//! Default implementation converts the arc into a polygon and calls
		//! FillPolygon()
		virtual ERRVALUE v_FillArc (
			INT32 xcenter,								//!< X coordinate of center of arc
			INT32 ycenter,								//!< Y coordinate of center of arc
			INT32 xradius,								//!< X radius of ellipse (BEFORE ROTATION)
			INT32 yradius,								//!< Y radius of ellipse (BEFORE ROTATION)
			double startangle,						//!< Starting angle in RADIANS
			double sweepangle,						//!< Sweep angle in RADIANS
			double rotangle = 0,							//!< Rotation angle in RADIANS
			MGD::SHAPECLOSE closemode = SHAPECLOSE_Direct
			);

		virtual ERRVALUE v_FillArc (
			double xcenter, 
			double ycenter, 
			double xradius, 
			double yradius, 
			double startangle, 
			double sweepangle, 
			double rotangle, 
			MGD::SHAPECLOSE closemode 		
			);

		//! Fill a circle.
		//!
		//! Default implementation uses FillSpans()
		virtual ERRVALUE v_FillCircle (
			INT32 xcenter,								//!< X coordinate of center of circle
			INT32 ycenter,								//!< Y coordinate of center of circle
			INT32 radius								//!< Radius of circle in pixels
			);
			
		virtual ERRVALUE v_FillCircle (
			double xcenter,								//!< X coordinate of center of circle
			double ycenter,								//!< Y coordinate of center of circle
			double radius								//!< Radius of circle in pixels
			);

		//! Fill an ellipse.
		//!
		//! Default implementation calls FillArc with a full \f$ 0 - 2\pi\f$ arc
		virtual ERRVALUE v_FillEllipse (
			INT32 xcenter,								//!< X coordinate of center of arc
			INT32 ycenter,								//!< Y coordinate of center of arc
			INT32 xradius,								//!< X radius of ellipse (BEFORE ROTATION)
			INT32 yradius,								//!< Y radius of ellipse (BEFORE ROTATION)
			double rotangle = 0							//!< Rotation angle in RADIANS
			);

		virtual ERRVALUE v_FillEllipse (
			double xcenter,								//!< X coordinate of center of arc
			double ycenter,								//!< Y coordinate of center of arc
			double xradius,								//!< X radius of ellipse (BEFORE ROTATION)
			double yradius,								//!< Y radius of ellipse (BEFORE ROTATION)
			double rotangle							//!< Rotation angle in RADIANS
			);
		
		//! Fill a polygon.
		//!
		//! Default implementation calls FillPolygons with a single polygon
		virtual ERRVALUE v_FillPolygon (
			const LPOINT2D* points, 
			INT32 NumPoints, 
			MGD::POLYSHAPE shape = POLYSHAPE_Complex, 
			bool DrawOutline = false
			);

		//! Fill polygons.
		//! The first polygon is the polygon to fill, additional polygons are
		//! treated as islands (with possible islands inside the islands, etc)
		//!
		//! Default implementation calls HatchPolygons() if the current context
		//! wants them, otherwise it fills using FillSpan()
		virtual ERRVALUE v_FillPolygons (
			const LPOLYGON* polygons, 
			INT32 NumPolys, 
			bool DrawOutline = false
			);

		//! Fill a rectangle.
		//! 
		//! Default implementation converts the rectangle to a polygon and calls
		//! fill Polygon if the context is set to use a hatch pattern or if the
		//! device's AvoidFillSpans() returns true.  Otherwise it fills using
		//! FillSpan()
		virtual ERRVALUE v_FillRectangle (
			INT32 x1, 		//!< Upper left X
			INT32 y1, 		//!< Upper left Y
			INT32 x2, 		//!< Lower right X
			INT32 y2			//!< Lower right Y
			);

		//! Fill a rectangle.
		//!
		//! Default implementation calls the other version of FillRectangle()
		virtual ERRVALUE v_FillRectangle (
			const LRECT2D& rect
			);

		//! Fill a rectangle.
		//!
		//! Default implementation calls the other version of FillRectangle()
		virtual ERRVALUE v_FillRectangle (
			const DRECT2D& rect
			);
		
		//! Fill a region.
		//! Converts the region into LPOLYGONs and calls FillPolygons.
		virtual ERRVALUE v_FillRegion (
			const REGION2D& polygons,
			bool DrawOutline = false
			);

		//! Fill a region.
		//! Converts the region into LPOLYGONs and calls FillPolygons.
		virtual ERRVALUE v_FillSubRegion (
			const SUBREGION2D& polygons,
			bool DrawOutline = false
			);

		//! Tell the display what kind of data we'll provide to PutImageSpan().
		//! If it matches the colorspace of the display, PutImageSpan() will
		//! call ImagePutSpanDirect().  If not, the image will be translated
		//! to what the display wants first.
		//!
		//! If overloaded, derived class should call base class first.
		//! Doing so will setup the protected m_ColorComp member and
		//! PutImageSpanDirect() can assume the image data sent to it 
		//! is in the colorspace returned by GetColorSpace()
		virtual ERRVALUE v_ImageBegin (
			INT32 x,										//!< X coordinate of left edge
			INT32 y,										//!< Y coordinate of top edge
			INT32 width,								//!< Width of image in pixels
			INT32 height,								//!< Height of image in pixels
			COLORSPACE colorspace,					//!< Colorspace of the data we'll be sending
			const COLOR_ARRAY* colormap = 0,			//!< Colormap for COLORSPACE_Indexed colorspaces
			UINT32 flags = 0
			);
		
		//! Notify device that image is complete.
		//! Needed for some displays to flush output or terminate image operations
		//! Derived method should call base method after doing its own work.
		virtual ERRVALUE v_ImageEnd (
			UINT32 flags = 0
			);

		//! Put a span of image data.
		//! 
		//! Note:  Before putting an image, you MUST call ImageBegin
		//! and call ImageEnd afterwards.  These are stub functions that
		//! do nothing in most implementations, but required by others
		//!
		//! This function should assume the data passed to it is exactly what it wants.
		//! The caller will actually call PutImageSpan(), and ImageBegin() will cause 
		//! PutImageSpan to either call this directly or first translate the data from
		//! one display type to what we really want.
		virtual ERRVALUE v_PutImageSpanDirect (
			INT32 x, 
			INT32 y, 
			INT32 len, 
			const void* data, 
			const UINT8 *mask = 0
			);

		//! Put single pixel on generic display.
		//!
		//! Default implementation calls SetColor() and DrawPixel()
		//! and the default DrawPixel() just calls FillSpan with a one-pixel span
		//! If there is any 3D depth testing active, PutPixel() calculates z and 
		//! calls PutPixel3D()
		virtual ERRVALUE v_PutPixel (
			INT32 x, 
			INT32 y, 
			PIXEL pixel
			);

		//! Put 3D pixel at specified coordinates.
		//! Default implementation checks the Z buffer and calls v_PutPixel
		//! if the point would not be hidden.  99% of the time this is all you
		//! need it to do.  Derived class should only implement this if it can
		//! do something useful with the z value.
		virtual ERRVALUE v_PutPixel3D (
			INT32 x, 
			INT32 y, 
			float z, 
			PIXEL pixel
			);

		//! Reset all drawing parameters (except color) to defaults.
		//! There is a default implementation that calls set functions
		//! derived class shouldn't need to override it, but can if it would be faster 
		//! to do it all at once.  It could also override it to reset private
		//! things, but then should call base class version
		virtual void v_Reset ();

		//! Set origin for bitmap pattern drawing.
		//!
		//! The origin will be SUBTRACTED from the screen coordinate when indexing into the bitmap pattern.
		//! If a rectangle is filled whose upper-left corner is the specified origin the upper-left corner
		//! of the pattern will be at the upper-left corner of the rectangle.
		//!
		//! The default implementation records the new settings
		virtual void v_SetBitmapOrigin (
			INT32 xorigin,								//!< X origin
			INT32 yorigin								//!< Y origin
			);

		//! Default method does nothing.
		//! Used during symbol rendering for vector formats like pdf/svg etc.
		//! to indicate that the color being rendered is the variable color for 
		//! the pattern/symbol.
		virtual void v_SetColorIsPatternVariable (
			bool value
			);

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT

		SIMPLE_ARRAY<PRIVDATA*> m_PrivData;
		int m_CurrentPassNum;
		int m_NumPassesNeeded;
		MGD::OPACITY_MODE m_OpacityBlendMode;
		UINT16 m_FrameBuffer;
		STIPPLEMASK m_StippleMask;
		bool m_StippleIsUsed;

		static ERRVALUE OpenFontTTF (const MIUNICODE* fontname, MGD::FONT** pFont);
		static ERRVALUE OpenFontOF (const MIUNICODE* fontname, MGD::FONT** pFont);
		
		// Unimplemented.
		CONTEXT (const CONTEXT&);
		CONTEXT& operator= (const CONTEXT&);

		void ClipRegionValidate ();
		void ClipRegionInit ();

		ERRVALUE ComputeLabelFrame (
			REGION2D& Frame,
			const LABELSTYLE& Style,
			const DRECT2D& Extents,
			const DRECT2D* ExtentsSansDecenders,
			const DPOINT2D& BasePt,
			double RotAngle
			);

		ERRVALUE ComputeLabelExtentsAlongCurve (
			REGION2D& extents,
			const MIUNICODE* text,
			const POLYLINE& BaseLine,
			const LABELSTYLE& Style,
			const POLYLINELIST& LeaderLines,
			bool bCheckClipping = false
			);
			
		ERRVALUE ComputeTextExtentsShrinkWrapped (
			REGION2D& CombinedExtents,				//!< Returned union of extents of each line of text
			const MIUNICODE* string,
			const POLYLINE& Baseline,
			const TEXTSTYLE& style,
			double BufferDistance = 0.0,
			MGD::BASELINE baseflags = MGD::BASELINE_Straight,
			MGD::FONT::MODE mode = MGD::FONT::MODE_MIUNICODE
			);

		ERRVALUE DrawLabelAlongCurve (const MIUNICODE *text, const POLYLINE& BaseLine, const LABELSTYLE& Style, const POLYLINELIST& LeaderLines);
		ERRVALUE DrawLabelFrameAndLeaders (const REGION2D& Frame, const LABELSTYLE& Style, const POLYLINELIST& LeaderLines);

		ERRVALUE DrawLinesPattern (const LPOINT2D* points, INT32 NumPoints);
		ERRVALUE DrawLinesPattern (const POLYLINE& lines);
		ERRVALUE DrawLinesThick (const LPOINT2D* points, INT32 NumPoints);
		ERRVALUE DrawLinesThickAA (const LPOINT2D* points, INT32 NumPoints);
		ERRVALUE DrawLinesThickAA (const POLYLINE& line);
		ERRVALUE DrawLinesThin (const LPOINT2D* points, INT32 NumPoints);
		ERRVALUE DrawLinesThinAA (const LPOINT2D* points, INT32 NumPoints);
		ERRVALUE DrawLinesThinAA (const POLYLINE& line);

		ERRVALUE DrawPointPattern (INT32 x, INT32 y);

		ERRVALUE DrawStringAlongCurve (const POLYLINE& BaseLine, const MGD::STRINGDATA& gidstr, int sindex, int eindex, UINT32 baseflags);

		ERRVALUE FillSpanSub (INT32 x1, INT32 x2, INT32 y);
		ERRVALUE FillSpanWithDepthTest (INT32 x1, INT32 x2, INT32 y);

		ERRVALUE GenArcPoints (
			const LPOINT2D& center,
			INT32 xradius,
			INT32 yradius,
			double StartAngle,
			double SweepAngle,
			double RotAngle,
			SIMPLE_ARRAY<LPOINT2D>& points,
			bool bKeepExistingPoints = false
			);

		ERRVALUE GenRoundedRectPoints (const LRECT2D& rect, INT32 radius, SIMPLE_ARRAY<LPOINT2D>& points);

		ERRVALUE HatchMBR (const LRECTXY &mbr, RVC::STYLEPATTERN_HATCH& HatchPatt, const void *vpHatchElement);
		ERRVALUE HatchPolygons (const LPOLYGON *poly, INT32 numpoly);
		ERRVALUE HatchPolygons (const REGION2D&);
		ERRVALUE HatchPolygonsStyle (const LPOLYGON *poly, INT32 numpoly);
		ERRVALUE HatchPolygonsStyle (const REGION2D& region);

		ERRVALUE LabelFrameMergeBalloonLeaders (REGION2D& frame, const LABELSTYLE& Style, const POLYLINELIST& LeaderLines);

		bool SetupICMTransform ();

		friend class MGD::DEVICE;			// Only MGD::DEVICE::CreateGC() can call our constructor
		friend struct MGD::TEXTBLOCK;		// Needs to call private methods
		friend class SAVEFONTFLAGS;	
		friend class LOCALOPENFONTS;		// Located in mg2text.c
		#endif // GENERATING_DOXYGEN_OUTPUT

		//! Draw single icon.
		//!	Default implementation does nothing.
		virtual ERRVALUE v_DrawIcon (
			INT32 x, 
			INT32 y, 
			MICONSTRUCT* icon, 
			ICONFLAGS flags
			);

		//! Draw "interface text" MIUNICODE* string.
		//!
		//!	This and related InterfaceText functions are used for rendering text through the
		//!	underlying graphic system.  For example, X text will be rendered using BDF fonts.
		//!	For user interfaces this will generally be much faster than using the Text or
		//!	TextBlock functions, but is subject to the limitations of the user-selected font
		//!	and style.
		//!
		//!	@param x X coordinate of basepoint of text
		//!	@param y Y coordinate of basepoint of text
		//!	@param string The string to draw
		//!	@param baseorigin Where to draw the text relative to x,y
		//!	@param bVisualOrder If true, the text is assumed to be in visual order and no BIDI flipping will be done.
		//!	@param style Interface text style
		//!
		//! Default implementation does nothing.
		//!	This method is only implemented on MGD::DEVICE_X and should not be
		//!	used anywhere WYSIWYG text is desired.  
		virtual ERRVALUE v_InterfaceTextDrawString (
			INT32 x, 
			INT32 y, 
			const MIUNICODE* string, 
			TEXTBASEORIGIN baseorigin, 
			bool bVisualOrder,
			INTERFACETEXTSTYLE style
			);

		//! Get extents of "interface text" MIUNICODE* string.
		//! Default implementation does nothing.
		//!	This method is only implemented on MGD::DEVICE_X and should not be
		//!	used anywhere WYSIWYG text is desired.  
		virtual ERRVALUE v_InterfaceTextGetExtents (
			const MIUNICODE* string,					//!< String
			int *width,									//!< Width of string in pixels returned, 0 if don't need
			int *height,							//!< Height in pixels returned, 0 if don't need
			int *ascent,							//!< Ascent of first line in pixels returned, 0 if don't need
			INTERFACETEXTSTYLE style
			);
		
		//! Get metrics of "interface text" MIUNICODE* string.
		//! Default implementation does nothing.
		//!	This method is only implemented on MGD::DEVICE_X and should not be
		//!	used anywhere WYSIWYG text is desired.  
		virtual ERRVALUE v_InterfaceTextGetMetrics (
			const MIUNICODE* string,					//!< String
			TEXTMETRICS& metrics,
			INTERFACETEXTSTYLE style
			);
		
		//! Get width of "interface text" MIUNICODE* string.
		//! Default implementation does nothing.
		//!	This method is only implemented on MGD::DEVICE_X and should not be
		//!	used anywhere WYSIWYG text is desired.  
		virtual int v_InterfaceTextGetWidth (
			const MIUNICODE* string,					//!< String
			INTERFACETEXTSTYLE style,
			bool bInkMetrics
			);

		//! Called before a point symbol is to be drawn.
		//! Symbol is described by m_PointStyle and m_pattbuf
		//! Default implementation does nothing
		virtual void v_SymbolBegin ();

		//! Called after a point symbol is drawn.
		//! Default implementation does nothing
		virtual void v_SymbolEnd ();

		//! Called at the beginning of a TextBlockDraw().
		//! This allows the derived class to just keep track of the
		//! glyphs and dump them all out in some format at the end of
		//! the string.
		//! Default implementation does nothing
		virtual void v_TextBlockBegin (
			const MGD::TEXTBLOCK& block	
			);

		//! Called at the end of a TextBlockDraw().
		//! This allows the derived class to just keep track of the
		//! glyphs and dump them all out in some format at the end of
		//! the string.
		//! Default implementation does nothing
		virtual void v_TextBlockEnd (
			const MGD::TEXTBLOCK& block	
			);

		//! Called just before starting a run of _DrawGlyph calls.
		//! This allows the derived class to just keep track of the
		//! glyphs and dump them all out in some format at the end of
		//! the string.
		//! Default implementation does nothing
		virtual void v_TextSpanBegin ();

		//! Called just after ending a run of _DrawGlyph calls.
		//! This allows the derived class to just keep track of the
		//! glyphs and dump them all out in some format at the end of
		//! the string.
		//! Default implementation does nothing
		virtual void v_TextSpanEnd ();

		//! Called when clipping is set to None.
		//! This method can be overridden by a derived class if it needs
		//! to notify the underlying device about a change to the 
		//! context.  The default implementation does nothing.
		virtual void v_UpdateClipNone ();

		//! Called when the ClipRect is changed.
		//! This method can be overridden by a derived class if it needs
		//! to notify the underlying device about a change to the 
		//! context.  The default implementation does nothing.
		virtual void v_UpdateClipRect ();

		//! Called when the ClipRegion or ClipRect is changed.
		//! This method can be overridden by a derived class if it needs
		//! to notify the underlying device about a change to the 
		//! context.  The default implementation does nothing.
		virtual void v_UpdateClipRegion ();

		//! Called when the current drawing color changes (via a PIXEL).
		//! This method can be overridden by a derived class if it needs
		//! to notify the underlying device about a change to the 
		//! context.  The default implementation does nothing.
		//! Note that this method is always called when the color is changed,
		//! where as the COLOR version is only called if the COLOR version
		//! of SetColor() is called.
		virtual void v_UpdateColor ();

		//! Called when the current drawing color changes (via a COLOR).
		//! This method can be overridden by a derived class if it needs
		//! to notify the underlying device about a change to the 
		//! context.  The default implementation does nothing.
		//! Note that when this method is called, the PIXEL version will
		//! also get called, but the reverse is not always true.  This version
		//! of the method is only called when the COLOR version of SetColor is
		//! called.
		virtual void v_UpdateColor2 (const COLOR&);

		//! Called when the Font is changed.
		//! This method can be overridden by a derived class if it needs
		//! to notify the underlying device about a change to the 
		//! context.  The default implementation does nothing.
		virtual void v_UpdateFont (const MGD::FONT*);

		//! Called when the LineAttributes is changed.
		//! This method can be overridden by a derived class if it needs
		//! to notify the underlying device about a change to the 
		//! context.  The default implementation does nothing.
		virtual void v_UpdateLineAttributes ();

		//! Called when the LineWidth is changed.
		//! This method can be overridden by a derived class if it needs
		//! to notify the underlying device about a change to the 
		//! context.  The default implementation does nothing.
		virtual void v_UpdateLineWidth ();

		//! Called when the PixelFunc is changed.
		//! This method can be overridden by a derived class if it needs
		//! to notify the underlying device about a change to the 
		//! context.  The default implementation does nothing.
		virtual void v_UpdatePixelFunc ();

		//! Called when the PlaneMask changes.
		//! This method can be overridden by a derived class if it needs
		//! to notify the underlying device about a change to the 
		//! context.  The default implementation does nothing.
		virtual void v_UpdatePlaneMask ();

		//! Called when stipple settings changed.
		//! This method can be overridden by a derived class if it needs
		//! to notify the underlying device about a change to the 
		//! context.  The default implementation does nothing.
		virtual void v_UpdateStipple ();

	};

//=====================================================================================================================
//! Save the clip region of an MGD::CONTEXT.  Automatically restores the region
//! when it goes out of scope.
class SAVECLIP {
	public:
		SAVECLIP (
			MGD::CONTEXT& gc
			);

		~SAVECLIP();

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT
		MGD::CONTEXT& m_gc;
		REGION2D* m_SavedClip;	
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
//=====================================================================================================================
//! Structure to hold intermediate values used while rendering a string.
struct STRINGDATA {
	STRINGDATA() {}
	~STRINGDATA() {}

	STRINGDATA (
		MGD::CONTEXT* context, 
		const MISTRING& str
		);
	
	STRINGDATA (
		MGD::CONTEXT* context, 
		const MIUNICODE* str,
		MGD::FONT::MODE mode
		);

	struct CHARDATA {
		UINT16 gid;				//!< Glyph ID
		UINT16 flags;
		FTFACE* face;			//!< FreeType font face to use (for .ttc only)
		DPOINT2D pos;
		};
	
	SIMPLE_ARRAY<UINT16> m_gid;			//!< Glyph IDs for each character
	SIMPLE_ARRAY<FTFACE*> m_face;			//!< FreeType font face to use
	SIMPLE_ARRAY<UINT16> m_gflags;		//!< Glyph flags (not used)
	SIMPLE_ARRAY<DPOINT2D> m_pos;			//!< Position of glyph relative to base point of first glyph
	MISTRING m_str;			//!< May be used by overloaded _DrawGlyph() call
	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT
		STRINGDATA(const STRINGDATA&);

		ERRVALUE Initialize (
			MGD::CONTEXT* context, 
			const MIUNICODE* str,
			MGD::FONT::MODE mode
			);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};



//=====================================================================================================================
//! Class to manage a 'color lock' on a CONTEXT.
//! The lock will automatically be released (if set) when the COLORLOCK
//! instance goes out of scope.
class COLORLOCK {
	public:

		//! Constructor, does not set color lock.
		COLORLOCK (
			CONTEXT *pContext
			) : m_pContext(pContext), m_locked(false) { }

		//! Destructor, releases color lock if set.
		~COLORLOCK (
			) { Clear(); }

		//! Clear color lock if set.
		void Clear (
			) { if (m_locked) m_pContext->SetColorLock(false); m_locked = false; }

		//! Set color lock if not already.
		void Set (
			) { if (!m_locked) m_pContext->SetColorLock(true); m_locked = true; }

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT
		CONTEXT *m_pContext;
		bool m_locked;

		COLORLOCK (const COLORLOCK&);
		COLORLOCK& operator= (const COLORLOCK&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================
//! An MGD::CONTEXT which is automatically deleted when it goes out of scope.
class CONTEXT_AUTOPTR {
	public:

		//! Default constructor, creates context for 'null' device.
		CONTEXT_AUTOPTR (
			): m_pContext(DEVICE::GetNull().CreateContext())
			{ }

		//! Construct context for specified device.
		explicit CONTEXT_AUTOPTR (
			DEVICE& dev
			): m_pContext(dev.CreateContext())
			{ }

		~CONTEXT_AUTOPTR (
			) { delete m_pContext; }

		CONTEXT& operator* () const {
			return (*m_pContext);
			}

		CONTEXT* operator-> () const {
			return (m_pContext);
			}

		//! Cast to CONTEXT*.
		operator CONTEXT* (
			) const { return (m_pContext); }

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT
		CONTEXT_AUTOPTR(const CONTEXT_AUTOPTR&);
		CONTEXT_AUTOPTR& operator=(const CONTEXT_AUTOPTR&);

		CONTEXT *m_pContext;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================
//		Public functions in MGD namespace

ERRVALUE DlgGetScalableFont (
	MDLGPARENT dlgparent,			//!< Parent for dialog
	MISTRING& FontName,				//!< Font name passed/returned
	const MISTRING& PromptStr,		//!< Prompt for dialog, TEXTID__None for default
	bool TrueTypeOnly = true		//!< Only allow selection of TrueType fonts
	);

//! Get embedded TrueType font family name for a font based on its ttf file name
//! \deprecated Use the version that takes an MISTRING instead
DEPRECATED int TextGetFontByName (
	INIHANDLE ini,								//!< Open tntproc.ini if calling this in a loop
	MIUNICODE *fontname,						//!< Font name returned (eg: "times.ttf" )
	MIUNICODE *family,						//!< Family name requested
	UINT32 StyleFlags,						//!< FONTSTYLE_Flags requested
	UINT32 *pStyleFlags						//!< FONTSTYLE_Flags found (closest match)
	);

//! Get embedded TrueType font family name for a font based on its ttf file name
int TextGetFontByName (
	MISTRING& fontname,						//!< Font name returned (eg: "times.ttf" )
	const MIUNICODE *family,					//!< Family name requested
	UINT32 StyleFlags,						//!< FONTSTYLE_Flags requested
	UINT32 *pStyleFlags,						//!< FONTSTYLE_Flags found (closest match)
	INIHANDLE ini = 0							//!< Open tntproc.ini if calling this in a loop
	);

int TextGetFontByFamily (
	MISTRING& fontname,						//!< Font name returned (eg: "times.ttf" )
	const MIUNICODE *family,					//!< Family name requested
	UINT32 StyleFlags,						//!< FONTSTYLE_Flags requested
	UINT32 *pStyleFlags,						//!< FONTSTYLE_Flags found (closest match)
	INIHANDLE ini = 0							//!< Open tntproc.ini if calling this in a loop
	);

int TextGetFontByFullName (
	MISTRING& fontname,						//!< Font name returned (eg: "times.ttf" )
	const MIUNICODE *realname,					//!< real name requested
	UINT32 StyleFlags,						//!< FONTSTYLE_Flags requested
	UINT32 *pStyleFlags,						//!< FONTSTYLE_Flags found (closest match)
	INIHANDLE ini = 0							//!< Open tntproc.ini if calling this in a loop
	);

//! Get font information for specified font handle.
ERRVALUE TextGetFontInfo (
	int fhandle,								//!< Font "handle"
	MGD::FONT *font								//!< Font information returned
	);

//! Return font name for specified font handle.
ERRVALUE TextGetFontName (
	int fonthandle,							//!< Font "handle"
	MISTRING& fontname							//!< String to contain font name
	);

//! Return font name for specified font handle.
// Can be removed after 24 Mar 07 and that the module that calls this is changed
ERRVALUE TextGetFontNameOld (
	int fonthandle,							//!< Font "handle"
	MIUNICODE* fontname							//!< String to contain font name
	);

ERRVALUE TextGetTTFName (
	INIHANDLE ini,								//!< Open tntproc.ini if calling this in a loop
	const MIUNICODE *fontname,						//!< Full path or just "times.ttf"
	MGD::TTFNAMEID nameid,									//!< TTFNAMEID_Xxxx
	MISTRING& name,
	UINT32 *pStyleFlags						//!< FONTSTYLE_Flags returned
	);

void GetDefaultFontName (
	MISTRING& name
	);

//=====================================================================================================================

} // End of namespace MGD

#endif	// INC_MI32_MGD2_H


