/****
 * \file netnghbr.h <mi32/netnghbr.h>
 * \brief Header file for NETNEIGHBOR class
 *
 * \if NODOC
 * $Id: netnghbr.h_v 1.13 2007/05/07 20:45:56 mju Exp $
 *
 * $Log: netnghbr.h_v $
 * Revision 1.13  2007/05/07 20:45:56  mju
 * Use milist hdr.
 *
 * Revision 1.12  2006/02/16 16:37:09  dwilliss
 * replaced an ifdef's define check with one that's not deprecated
 *
 * Revision 1.11  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.10  2002/05/17 19:44:24  dwilliss
 * Export the class even on the Mac using it's stupid way of doing it.
 *
 * Revision 1.9  2000/07/11 17:37:02  mju
 * Undef CLASSLIBEXPORT at end.
 *
 * Revision 1.8  2000/07/07 19:56:43  sparsons
 * Genitor documentation.
 *
 * Revision 1.7  2000/03/14 23:18:39  scowan
 * Does more than 136 entries now.
 *
 * Revision 1.6  2000/01/25 23:05:39  scowan
 * *** empty log message ***
 *
 * Revision 1.5  2000/01/21 21:53:49  scowan
 * Added method to get a desc.
 *
 * Revision 1.4  1999/12/02 21:48:33  scowan
 * *** empty log message ***
 *
 * Revision 1.3  1999/11/17 23:50:10  scowan
 * Updated methods.
 *
 * Revision 1.2  1999/10/29 20:37:10  scowan
 * Now is working.
 *
 * Revision 1.1  1999/10/27 16:41:54  scowan
 * Initial revision
 *
 *
 * \endif
****/

#ifndef	INC_MI32_NETNGHBR_H
#define	INC_MI32_NETNGHBR_H

#ifndef	INC_MI32_MISTRING
	#include <mi32/mistring.h>
#endif

#ifndef	INC_MI32_MILIST
	#include <mi32/milist.h>
#endif

#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

class CLASSLIBEXPORT NETNEIGHBOR {
public:

	enum CHILDTYPE {
		CHILDTYPE_Invalid = 0,
		CHILDTYPE_Container,
		CHILDTYPE_Connectable
		};

	// CONSTRUCTION / DESTRUCTION

	//! Default constructor, start at top of network.
	NETNEIGHBOR (							
		);

	//! Destructor.
	~NETNEIGHBOR (
		);

	//! Copy constructor.
	NETNEIGHBOR (							
		const NETNEIGHBOR& rhs
		);

	//! Assignment operator.
	NETNEIGHBOR& operator= (			
		const NETNEIGHBOR& rhs
		);

	// METHODS

	//! Clear list of children for new enumeration.
	void Clear (							
		);

	//! Enumerate a list of children from root.
	//!
	//! @return Number of children or error code if < 0.
	int EnumChildren (					
		MISTRINGLIST &HostList				//!< List of machine hosts found
		);											 
		
	//! Enumerate a list of children from parent.
	//!
	//! @return Number of children or error code if < 0.
	int EnumChildren (					
		const NETNEIGHBOR& parent,			//!< Parent to generate a list of children from
		const MISTRING& name,				//!< Object name from parent to generate list of its children from
		MISTRINGLIST &ShareList				//!< List of machine shares found
		);											

	//! Return type of child, Container or Connectable.
	CHILDTYPE GetChildType (			
		const MISTRING& name					//!< Name of child to check
		) const;

	//! Return description of child.
	void GetDesc (							
		const MISTRING& name,				//!< Name of child to get description of
		MISTRING& desc							//!< Description RETURNED
		) const;

	//! Does container have children?
	bool HasChildren (					
		) const;

private:
	#ifndef GENERATING_DOXYGEN_OUTPUT

	int ListChildren (					//! List children from enumeration
		MISTRINGLIST &List,					//!< List of items found
		INT32 StartOffset
		);											//! Error code if < 0
		
	struct NETRES {
		UINT32 dwScope;
		UINT32 dwType;
		UINT32 dwDisplayType;
		UINT32 dwUsage;
		MISTRING lpLocalName;
		MISTRING lpRemoteName;
		MISTRING lpComment;
		MISTRING lpProvider;
		};
		
	MILIST<NETRES>::CONST_ITERATOR FindEntry (
		const MISTRING& name
		) const;

	MILIST<NETRES> *m_ChildArray;		//!< This was a void pointer, keep it a pointer for compatability
	void *m_EnumHandle;
	UINT32 m_ChildArraySize;
	UINT32 m_NumChildren;
	#endif // GENERATING_DOXYGEN_OUTPUT

};

#undef LIBEXPORT
#undef CLASSLIBEXPORT


#endif	//!<  INC_MI32_NETNGHBR_H 
