/***
 *	\file mi32/rvctin.h
 *	\brief Definitions for TIN (Triangulated Irregular Network) objects
 *
 * \if NODOC
 * $Id: rvctin.h_v 1.64 2011/11/04 13:48:56 mju Exp $
 *
 * $Log: rvctin.h_v $
 * Revision 1.64  2011/11/04 13:48:56  mju
 * Remove redundant hdr.
 *
 * Revision 1.63  2007/11/12 20:40:12  mju
 * Remove mstatus hdr.
 *
 * Revision 1.62  2007/11/09 18:32:39  mju
 * Use mistatusmanager in delaunay fns.
 *
 * Revision 1.61  2007/04/09 15:56:17  scowan
 * Removed legacy include file.
 *
 * Revision 1.60  2007/02/16 23:32:04  scowan
 * Removed more legacy unused functions.
 *
 * Revision 1.59  2007/02/03 00:22:29  scowan
 * Remove unused inline function.
 *
 * Revision 1.58  2006/08/24 15:08:42  scowan
 * Removed unused prototypes.
 *
 * Revision 1.57  2005/07/19 16:45:54  scowan
 * Removed more unused legacy functions.
 *
 * Revision 1.56  2005/07/14 17:37:10  scowan
 * Removed many unused legacy functions.
 *
 * Revision 1.55  2005/02/14 21:30:01  scowan
 * Moved unused items to private tin include file.
 *
 * Revision 1.54  2005/02/14 18:43:31  mju
 * Remove legacy object selection dialog inlines.
 *
 * Revision 1.53  2005/01/04 18:48:54  vdronov
 * fixed typo in TRIANGLEINFO
 *
 * Revision 1.52  2005/01/03 15:57:08  mju
 * No_Depr for legacy object selection dlgs.
 *
 * Revision 1.51  2004/08/24 22:09:08  scowan
 * Added total line length field to header.
 *
 * Revision 1.50  2004/03/12 15:07:04  mju
 * Remove another unused legacy object selection dialog inline.
 *
 * Revision 1.49  2004/03/10 22:53:26  mju
 * Mark legacy dialogs as deprecated.
 *
 * Revision 1.48  2004/01/30 18:57:51  vdronov
 * make it HardEdge
 *
 * Revision 1.47  2004/01/29 23:42:34  vdronov
 * added FLAG_BreakEdge to EDGEINFO
 *
 * Revision 1.46  2004/01/08 20:29:23  vdronov
 * added cople more structures
 *
 * Revision 1.45  2004/01/08 00:01:15  vdronov
 * added TRIANGLENODES, TRIANGLEEDGES, EDGEINFO structures.
 *
 * Revision 1.44  2003/10/15 17:39:43  mju
 * Deprecate groupkey.
 *
 * Revision 1.43  2003/10/13 16:10:58  scowan
 * Removed more inline unused functions.
 *
 * Revision 1.42  2003/10/13 15:42:36  scowan
 * Removed unused function prototypes.
 *
 * Revision 1.41  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.40  2002/12/26 16:23:54  scowan
 * Fixed invalid extents handling.
 *
 * Revision 1.39  2002/10/04 21:46:45  scowan
 * Added more methodcs.
 *
 * Revision 1.38  2002/09/10 22:54:59  scowan
 * Added ext function version.
 *
 * Revision 1.37  2002/09/10 21:03:58  scowan
 * Added is valid methods.
 *
 * Revision 1.36  2002/08/21 21:55:46  scowan
 * Fixed derecated stuff.
 *
 * Revision 1.35  2002/08/20 15:20:03  mju
 * Use prevent_memfunc macro always.
 * Remove obsolete dialog defines.
 *
 * Revision 1.34  2002/03/08 18:38:05  scowan
 * Added clear method.
 *
 * Revision 1.33  2002/01/03 22:50:30  scowan
 * Added functions to manage tin object extents.
 *
 * Revision 1.32  2001/09/14 14:58:45  mju
 * No mdlg in win32native.
 *
 * Revision 1.31  2001/01/31 14:49:45  scowan
 * Fixed dll import / export error.
 *
 * Revision 1.30  2000/10/30 18:52:15  msmith
 * Genitor documentation.
 *
 * Revision 1.29  2000/10/19 17:46:56  scowan
 * Added function to get the number of elements.
 *
 * Revision 1.28  2000/08/11 19:40:42  scowan
 * Update prompts.
 *
 * Revision 1.27  2000/05/08 16:52:32  mju
 * Remove TRIANGLEDATA and TINCASCADING struct defns as nothing uses them.
 *
 * Revision 1.26  2000/04/21 15:49:34  mju
 * Add op= for assignment of GENINFO to TININFO.
 *
 * Revision 1.24  1999/10/05  22:48:49  scowan
 * Changed headers to use inheritance.
 *
 * Revision 1.23  1999/06/14 19:43:55  mju
 * Add defaults for Mdlg fns.
 *
 * Revision 1.22  1999/06/11  14:46:32  mju
 * Obj sel dlg.
 *
 * Revision 1.21  1999/06/10  22:11:28  mju
 * Object selection dialog update.
 *
 * Revision 1.20  1999/05/07  21:23:58  mju
 * Hdr restruct.
 *
 * Revision 1.19  1999/04/08  15:36:20  mju
 * Use wrong inclusion guard for rvcdefns.h
 *
 * Revision 1.18  1999/01/05  19:13:49  scowan
 * Swapped multiple keywords
 *
 * Revision 1.17  1998/09/30 21:54:46  scowan
 * Setup DLL.
 *
 * Revision 1.15  1998/09/02  20:22:22  scowan
 * Exported Dealaunay functions.
 *
 * Revision 1.14  1998/02/04  23:27:01  scowan
 * Added element id's.
 *
 * Revision 1.13  1996/12/20  16:37:34  coffee!dwilliss
 * Added __cplusplus things
 *
 * Revision 1.11  1996/09/26  21:46:44  dfrolov
 * ComputeTriangleContoursExt added
 *
 * Revision 1.10  1996/09/26  19:18:04  dfrolov
 * add ComputeTriangleContours
 *
 * Revision 1.9  1996/09/20  00:03:22  dfrolov
 * nothing important
 *
 * Revision 1.8  1996/01/11  17:39:51  scowan
 * Added MfReadOpenTINHeader().
 *
 * Revision 1.7  1995/09/01  23:16:25  scowan
 * Added prototype for MfResizeTIN().
 *
 * Revision 1.6  1995/02/01  21:45:09  dfrolov
 * eliminate problem with versions
 *
 * Revision 1.4  1994/12/16  00:04:53  dfrolov
 * Add Delaunay family
 *
 * Revision 1.3  1994/11/07  17:45:54  mju
 * Add FindClosestTIN...() functions and PointInTINTriangle().
 *
 * Revision 1.2  1994/11/02  18:22:16  scowan
 * Fixed spelling error.
 *
 * Revision 1.1  1994/11/02  17:40:02  scowan
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_RVCTIN_H
#define	INC_MI32_RVCTIN_H

#ifndef	INC_MI32_RECT_H
	#include <mi32/rect.h>
#endif


//! TIN node structure.
struct RVCTINNODE : public DPOINT3D {

	RVCTINNODE& operator= (
		const DPOINT3D& pt
		) { 
		*static_cast<DPOINT3D*>(this) = pt; 
		return (*this);
		}

	//! Get extents of TIN node.
	//! @return extents of TIN node
	DRECT3D GetExtents (
		) const { return (DRECT3D(*this)); }

	bool IsValid (
		) const { return (true); }
	};

//! TIN triangle structure.
struct RVCTINTRIANGLE {
	INT32 node1;								//!< Id of node 1
	INT32 node2;								//!< Id of node 2
	INT32 node3;								//!< Id of node 3
	INT32 triangle12;							//!< Connected triangle id, shared edge 1-2
	INT32 triangle23;							//!< Connected triangle id, shared edge 2-3
	INT32 triangle31;							//!< Connected triangle id, shared edge 3-1
	INT32 edge12;								//!< Id of edge 1-2
	INT32 edge23;								//!< Id of edge 2-3
	INT32 edge31;								//!< Id of edge 3-1
	UINT8 edgeflags12;						//!< Flags for edge 1-2
	UINT8 edgeflags23;						//!< Flags for edge 2-3
	UINT8 edgeflags31;						//!< Flags for edge 3-1
	#ifndef GENERATING_DOXYGEN_OUTPUT
	UINT8 spare;
	#endif //!< GENERATING_DOXYGEN_OUTPUT
	
	bool IsValid (
		) const { return (true); }
	};

//! TIN edge structure.
struct RVCTINEDGE {
	INT32 nodefrom;							//!< Id of the starting node
	INT32 nodeto;								//!< Id of the ending node
	INT32 trileft;								//!< Id of the left triangle
	INT32 triright;							//!< Id of the right triangle

	bool IsValid (
		) const { return (true); }
	};

//! TIN hull structure.
struct RVCTINHULL {
	DPOINT3D minval;							//!< Minimum range value for HULL
	DPOINT3D maxval;							//!< Maxmum range value for HULL
	DOUBLE length;								//!< Perimeter
	DOUBLE area;								//!< Area, excluding islands area
	DOUBLE volume;								//!< Volume if applicable
	DOUBLE zerolevel;							//!< Zero-level for volume, if applicable
	INT32 exthull;								//!< id of the external hull, -1 if none
	UINT16 hulltype;							//!< TINHULL_Convex, TINHULL_Clip or TINHULL_Exclude
	UINT16 spare;								//!< For double alignment

	//! Get extents of TIN object.
	//! @return extents of TIN object
	DRECT3D GetExtents (
		) const {
		DRECT3D retval(minval);
		retval.Extend(maxval);
		return (retval);
		}

	bool IsValid (
		) const { return (true); }
		
	//! Set extents of TIN object.
	void SetExtents (
		const DRECT3D& rect
		) {
		minval.x = rect.xinit;		
		minval.y = rect.yinit;		
		minval.z = rect.zinit;		
		maxval.x = rect.xlast;
		maxval.y = rect.ylast;
		maxval.z = rect.zlast;
		return;
		}

	};

//! TIN hull edge structure.
struct RVCTINHULLEDGE {
	INT32 nodefrom;							//!< Starting node
	INT32 trileft;								//!< Left triangle
	};


//! Triangle structure.
struct TRIANGLEREF {
	INT32 node1;								//!< Id of node 1
	INT32 node2;								//!< Id of node 2
	INT32 node3;								//!< Id of node 3
	INT32 triangle12;							//!< Connected triangle id, shared edge 1-2
	INT32 triangle23;							//!< Connected triangle id, shared edge 2-3
	INT32 triangle31;							//!< Connected triangle id, shared edge 3-1
	};

//! Triangle Nodes structure.
struct TRIANGLENODES {
	INT32 node1;		
	INT32 node2; 
	INT32 node3;	 
	UINT32 flags;	  
	};

//! Triangle Edges structure.
struct TRIANGLEEDGES {
	INT32 edge1;		
	INT32 edge2; 
	INT32 edge3;	 
	UINT32 flags;	  
	};

//! Triangle Triangles structure.
struct TRIANGLETRIANGLES {
	INT32 triangle1;		
	INT32 triangle2; 
	INT32 triangle3;	 
	UINT32 flags;	  
	};

//! Triangle Info structure.
struct TRIANGLEINFO {
	INT32 node1;					  
	INT32 node2;							 
	INT32 node3;						 
	INT32 triangle12;						 
	INT32 triangle23;					  
	INT32 triangle31;						 
	INT32 edge12;							  
	INT32 edge23;							
	INT32 edge31;							  
	UINT32 flags;	  
	};

//! Edge Info structure.
struct EDGEINFO {

	enum FLAG {
		FLAG_HardEdge = 0x00000001
		};

	INT32 startnode;		
	INT32 endnode; 
	INT32 lefttriangle;	
	INT32 righttriangle;	
	UINT32 flags;	  
	};

//! Values for RVCTINTRIANGLE.edgeflagsXX
#define	TINEDGE_Hard	0x01

//! Values for RVCTINHULL.hulltype
#define TINHULL_Convex  0
#define TINHULL_Clip    1
#define TINHULL_Exclude 2

#if defined(__cplusplus)
extern "C" {
#endif

#ifdef GEOMDLL
	#define GEOMLIBEXPORT MI_DLLEXPORT
#else
	#define GEOMLIBEXPORT MI_DLLIMPORT
#endif

//! Performs Delaunay triangulation (full parameters list).
//!
//! NOTE: The triangle vertices in TRIANGLEREF structure are organized in
//! clockwise direction (when looking down)
//! WARNING: The order of input Nodes will be changed, all nodes will be sorted
//! using X coordinate (then Y if two points have equal X coordinates).
//! WARNING: NUMBER OF NODES RETURNED MIGHT BE DIFFERENT FROM INPUT,
//! because of duplicate point removal using tolerance value.
GEOMLIBEXPORT int DelaunayProc (
	DPOINT3D **Nodes,							//!< Input/output array of points
	TRIANGLEREF **Triangles,				//!< Output array of triangles
	RVCTINHULLEDGE **Con,					//!< Output array of hull edges
	RVCTINHULL *HullInfo,					//!< Information about convex hull
	INT32 *NumNodes,							//!< Number of input/output nodes passed/returned
	INT32 *NumTriangles,						//!<	Number of the output triangles
	INT32 *NumHullEdges,						//!< Number of the hull edges
	double Tolerance							//!< Spatial tolerance
	);

//! Performs Delaunay triangulation (short parameters list).
//!
//! Note: This function calls DelaunayProc.
GEOMLIBEXPORT int Delaunay3D (
	DPOINT3D **Nodes,							//!< Input/output array of points
	TRIANGLEREF **Triangles,				//!< Output array of triangles
	INT32 *NumNodes,							//!< Number of input/output nodes passed/returned
	INT32 *NumTriangles,						//!< Number of the output triangles
	double Tolerance							//!< Spatial tolerance
	);

#undef GEOMLIBEXPORT

#if defined(__cplusplus)
}
#endif

#endif		// #ifndef INC_MI32_RVCTIN_H 

