/***
** Copyright 1993, 1994 by Markku Savela and
**	Technical Research Centre of Finland
 *
 * \if NODOC
 * $Log: texted.h_v $
 * Revision 1.39  2011/11/08 13:35:21  mju
 * Remove redundant hdr and add inclusion guard check.
 *
 * Revision 1.38  2008/09/08 13:24:19  mju
 * Fix doxygen warning.
 *
 * Revision 1.37  2006/10/04 22:15:16  dwilliss
 * Added function to get the EDITMENU
 *
 * Revision 1.36  2005/05/16 14:12:33  dwilliss
 * *** empty log message ***
 *
 * Revision 1.35  2005/03/31 16:57:21  fileserver!dwilliss
 * Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 * Revision 1.34  2005/03/25 16:07:46  dwilliss
 * new proto
 *
 * Revision 1.33  2004/08/12 21:04:30  dwilliss
 * Added a prototype for new function
 *
 * Revision 1.32  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * \endif
*/

#ifndef INC_MI32_TEXTED_H
#define INC_MI32_TEXTED_H

#ifndef	INC_MI32_TEXT_H
#include <mi32/text.h>
#endif

#ifndef	INC_MI32_ELEMSTYL_H
#include <mi32/elemstyl.h>
#endif


/* Parameters:

 Name		     Class		RepType		Default Value
 ----		     -----		-------		-------------
 alignment           XeAlignment        XeAlignment     XeStartAligned
 allowEdit           AllowEdit          Boolean         True
 background	     Background		Pixel		XtDefaultBackground
 border		     BorderColor	Pixel		XtDefaultForeground
 borderWidth	     BorderWidth	Dimension	1
 contentFile         ContentFile        String          NULL
 contentLength       ContentLength      int            Computed from content
 contentOffset       ContentOffset      int            0
 contentStream       ContentStream      FILE *          NULL
 contentString       ContentString      Pointer         NULL
 cursor              Cursor             Cursor          None
 cursorPosition      CursorPosition     int            0
 destroyCallback     Callback		Pointer		NULL
 displayCaret        Output             Boolean         True
 exportFormat        ExportFormat       XeTextExport    XeTextExport_ODIF
 firstLineOffset     Offset             int             0
 fonts               XeFonts            XeFont *        NULL
 format              Format             Boolean         False
 graphicRendition    Rendition          String          NULL
 height		     Height		Dimension	0
 indentation         Offset             Dimension       0
 initialState        Sequence           String          NULL
 itemization         Itemization        XeItemization * NULL
 kerningOffset       KerningOffset      Dimension *     NULL
 lineLayoutTable     LineLayoutTable    XeTabStop *     NULL
 lineSpacing         LineSpacing        Dimension       Computed from font
 mappedWhenManaged   MappedWhenManaged	Boolean		True
 modifyCallback      ModifyCallback     Pointer         NULL
 proportional        Proportional       Boolean         False
 width		     Width		Dimension	0
 x		     Position		Position	0
 y		     Position		Position	0

*/

/* New fields */
#define XtNallowEdit "allowEdit"
#define XtCAllowEdit "AllowEdit"
#define XtNcursorPosition "cursorPosition"
#define XtCCursorPosition "CursorPosition"
#define XtNmodifyCallback "modifyCallback"
#define XtCModifyCallback "ModifyCallback"
#define XtNblinkRate "blinkRate"
#define XtCBlinkRate "BlinkRate"
#define XtNpasswordMode "passwordMode"
#define XtCPasswordMode "PasswordMode"
#ifndef XtNdisplayCaret
#define XtNdisplayCaret "displayCaret"
#endif
#ifndef XtCOutput
#define XtCOutput "Output"
#endif
/* Class record constants */

#if defined(__cplusplus)
extern "C" {
#endif

extern WidgetClass xeTextEdWidgetClass;

#if defined(__cplusplus)
}
#endif

typedef struct _XeTextEdClassRec *XeTextEdWidgetClass;
typedef struct _XeTextEdRec      *XeTextEdWidget;

typedef struct {
	MIUNICODE *ucptr;
	int length;
	} XeTextBlockRec, *XeTextBlock;

struct TEXTSTYLE;

struct XeTextVerifyCallbackStruct {
	int reason;
	XEvent *event;
	Boolean doit;
	int currInsert, newInsert;
	int startPos, endPos;
	XeTextBlock text;
	const TEXTSTYLE* style;
	};

#define XeCR_UP_ARROW	XmCR_INCREMENT
#define XeCR_DOWN_ARROW	XmCR_DECREMENT

/*
** XeTextModifyCallbackData
**	is the data supplied for the XtNmodifyCallback just before
**	the content is prepared for any modifications. The callback
**	function has the option of preventing the modification by
**	setting the field 'cancel' to True.
*/
typedef struct XeTextModifyCallbackData
    {
	int reason;		//!<  == XeCR_MODIFY_TEXT 
	Boolean cancel;		//!<  Initialized to False 
    } XeTextModifyCallbackData;

/*
** XeTextUnit
*/
typedef enum
    {
	XeTextUnit_POINT,	//!<  Current point 
	XeTextUnit_POSITION,	//!<  Virtual positions 
	XeTextUnit_WORD,	//!<  A word 
	XeTextUnit_LINE,	//!<  A line 
	XeTextUnit_LINE_BEGIN,	//!<  Beginning of current line 
	XeTextUnit_LINE_END,	//!<  End of current line 
	XeTextUnit_PARAGRAPH,	//!<  Paragraph 
	XeTextUnit_TAG,		//!<  Content with the same tag 
	XeTextUnit_ALL		//!<  All content, End/Begin "file" 
    } XeTextUnit;

//!:Associate with "XeText Functions"
//!\addtogroup XeText XeText Functions
//!@{

_XFUNCPROTOBEGIN

void XeTextEnableDisplay
	(
#if NeedFunctionPrototypes
	Widget
#endif
	);

void XeTextDisableDisplay
	(
#if NeedFunctionPrototypes
	 Widget
#endif
	 );

void XeTextDisplay
	(
#if NeedFunctionPrototypes
	 Widget
#endif
	 );

Boolean XeTextSaveAsFile
	(
#if NeedFunctionPrototypes
	 Widget,	//!<  TextEd Widget pointer 
	 String		//!<  Name (path) of the file to be written 
#endif
	 );

void XeTextSetString
	(
	 Widget,	 //!<  TextEd Widget pointer 
	 const char *		/* String to set (in ASCII ISO-2022) */
	 );

#define XeTextSetStringUnicode XeTextSetStringUC	//!<  Old name 

void XeTextSetStringUC
	(
	 Widget,	//!<  TextEd Widget pointer 
	 const MIUNICODE *	/* Unicode string to set */
	 );

String XeTextGetString
	(
	 Widget,	//!<  TextEd Widget pointer 
	 int *		/* Number of octets returned */
	 );

#define XeTextGetStringUnicode XeTextGetStringUC	//!<  Old name 

MIUNICODE *XeTextGetStringUC
	(
	 Widget	//!<  TextEd Widget pointer 
	 );


MIUNICODE *XeTextGetSelectionUC (
	 Widget	//!<  TextEd Widget pointer 
	 );

String XeTextGetSubstring
	(
	 Widget,	//!<  TextEd Widget pointer 
	 int *,	/* Number of octets returned */
	 int,		//!<  Virtual offset to the start of region 
	 int		//!<  Virtual offset to the end of region 
	 );

int XeTextGetLength
	(
	 Widget,	//!<  TextEd Widget pointer 
	 int 		//!<  for future use 
	 );

#define XeTextMakeCursorPositionVisible XeTextMakeCursorPositionVisable

void XeTextMakeCursorPositionVisable
	(
	 Widget,	//!<  TextEd Widget pointer 
	 void *		/* Location or NULL for current insert pos */
	 );

int XeTextGetInsertionPosition
	(
	 Widget	//!<  TextEd Widget pointer 
	 );

void XeTextSetInsertionPosition
	(
	 Widget,	//!<  TextEd Widget pointer 
	 int		//!<  Position 
	 );

typedef int (*XeTextExtractFeed)
 (
  MIUNICODE *,		/* Extracted text portion */
  int,			//!<  The length of the text 
  XeTextTag,		//!<  The content TAG value 
  XtPointer		//!<  client_data 
 );

void XeTextExtract
	(
	 Widget,	//!<  TextEd Widget pointer 
	 int,		//!<  Virtual offset to the start of region 
	 int,		//!<  Virtual offset to the end of region 
	 XeTextExtractFeed, //!<  Function to receive the content (feed) 
	 XtPointer	//!<  client_data for the feed function 
	 );

void XeTextReplace
	(
	 Widget,	//!<  TextEd Widget pointer 
	 int,		//!<  Virtual offset to the start of the replace 
	 int,		//!<  Virtual offset to the end of the replace 
	 const char *,	/* Pointer to the octest to be inserted */
	 int		//!<  Number of octets to insert 
	 );

void XeTextInsert
	(
#if NeedFunctionPrototypes
	 Widget,	//!<  TextEd Widget pointer 
	 const char *,	/* Pointer to octest to be inserted */
	 int		//!<  Number of octets to insert 
#endif
	 );

void XeTextInsertUnicode
	(
#if NeedFunctionPrototypes
	 Widget,	//!<  TextEd Widget pointer 
	 const MIUNICODE *	/* Pointer to Unicode string to be inserted */
#endif
	 );

void XeTextReplaceTagged
	(
	 Widget,	//!<  TextEd Widget pointer 
	 int,		//!<  Virtual offset to the start of replace 
	 int,		//!<  Virtual offset to the end of replace 
	 const char *,	/* Pointer to octets to be inserted */
	 int,		//!<  Number of octets to insert 
	 XeTextTag	//!<  The Text tag to be used 
	 );

void XeTextReplaceTaggedUC
	(
	 Widget,	//!<  TextEd Widget pointer 
	 int,		//!<  Virtual offset to the start of replace 
	 int,		//!<  Virtual offset to the end of replace 
	 const MIUNICODE *,	/* Pointer to octets to be inserted */
	 int,		//!<  Number of octets to insert 
	 XeTextTag	//!<  The Text tag to be used 
	 );

int XeTextGetInsertionPoint
	(
#if NeedFunctionPrototypes
	 Widget		//!<  TextEd Widget pointer 
#endif
	 );

int XeTextScan
	(
#if NeedFunctionPrototypes
	 Widget,	//!<  TextEd Widget pointer 
	 XeTextUnit,	//!<  Type of unit(s) to scan 
	 int		//!<  Amount and direction of the scan 
#endif
	 );

int XeTextSearch
	(
#if NeedFunctionPrototypes
	 Widget,	//!<  TextEd Widget pointer 
	 int,		//!<  Amount and direction of the search 
	 const char *,	/* "Compound" string to search */
	 int,		//!<  Number of octets in the search string 
	 int		//!<  case sensitive/case insensitive flag 
#endif
	 );

XeTextTag XeTextGetInsertionTag
	(
#if NeedFunctionPrototypes
	 Widget		//!<  TextEd Widget pointer 
#endif
	 );

void XeTextGetSelectionPosition
	(
#if NeedFunctionPrototypes
	 Widget,	//!<  TextEd Widget pointer 
	 int *,	/* Virtual offset of the start of selection */
	 int *		/* Virtual offset of the end of selection */
#endif
	 );

void XeTextSetSelection
	(
#if NeedFunctionPrototypes
	 Widget,	//!<  TextEd Widget pointer 
	 int,		//!<  Virtual offset of the start of selection 
	 int		//!<  Virtual offset of the end of selection 
#endif
	 );

void XeTextUnsetSelection
	(
#if NeedFunctionPrototypes
	 Widget		//!<  TextEd Widget pointer 
#endif
	 );

void XeTextCut(Widget);
void XeTextCopy(Widget);
void XeTextPaste(Widget);
void XeTextRemove(Widget);	/* like cut but don't change clipboard */

void _XeTextMarkSelection(Widget);
void _XeTextPrepareToScrol(Widget);
void _XeTextUpdateFromScro(Widget);
void _XeAdjustScrollBars(Widget);

//! Scroll to an anchor.
//! An anchor (which is hidden) is imbeded in the text stream as..
//!   	\\033[5m\@NAME\\033[25m
//! Where NAME is the name of the anchor.
//! \return true if the anchor was found, false if not
bool XeTextScrollToAnchor(Widget, const MIUNICODE* anchor);

void XeTextKeyEventHandler(Widget, XtPointer, XEvent*, Boolean*);

_XFUNCPROTOEND


#if defined(__cplusplus)

//! Return contents of XeText widget as MISTRING.
inline MISTRING XeTextGetString (
	Widget xetext								//!< XeText widget
	) {
	MIUNICODE *ucstr = XeTextGetStringUC(xetext);
	MISTRING string(ucstr);
	free(ucstr);
	return (string);
	}

//! Copy contents of XeText widget into MISTRING
inline void XeTextGetString (
	Widget xetext,								//!< XeText widget
	MISTRING& string							//!< String to contain result
	) {
	MIUNICODE *ucstr = XeTextGetStringUC(xetext);
	string = ucstr;
	free(ucstr);
	return;
	}

//! Insert char* string into XeText widget.
inline void XeTextInsert (
	Widget xetext,								//!< XeText widget
	const char* string
	) {
	XeTextInsert(xetext,string,strlen(string));
	return;
	}

//! Insert MIUNICODE string into XeText widget.
inline void XeTextInsert (
	Widget xetext,								//!< XeText widget
	const MIUNICODE* string					//!< Unicode string to insert
	) {
	XeTextInsertUnicode(xetext,string);
	return;
	}

class EDITMENU;

//! Add a popup menu with cut/copy/paste to an XeText widget
EDITMENU* XeTextAddPopupEditMenu (
	Widget xetext
	);

//! Get the EDITMENU for this widget (assuming there is on)
EDITMENU* XeTextGetEditMenu (
	Widget xetext
	);

//! Get the text style at the given location
const TEXTSTYLE& XeTextGetStyle (
	Widget widget,
	int offset	//!< use -1 for current cursor location
	);

//! Set the text style at the given location
//! TEXTSTYLEDIFF is derived from TEXTSTYLE, and contains flags saying which
//! values to actually change.
void XeTextSetStyle (
	Widget widget,
	const TEXTSTYLEDIFF& style,	//!< Change
	int startoffset = -1,	//!< use -1 for current cursor selection
	int endoffset = -1	//!< use -1 for current cursor selection
	);

#ifndef GENERATING_DOXYGEN_OUTPUT
//! Set the base text style.
//! This doesn't actually change how anyting is rendered.  It changes
//! how text is returned from the widget.  Any style differences between
//! the base style and the first snip's style will be output as {~} codes
//! before the first snip.  So if the first snip contains red, 14pt Times 
//! New Roman text and that's what the BaseStyle says it should be, no {~}
//! codes will be output for those settings.  However, if the first snip is
//! bold and the BaseStyle doesn't have that set, it will output a {~B} 
//! before the first bit of text.
//!
//! XXX  I may implement this or do it another way.
/*
void XeTextSetBaseStyle (
	Widget widget,
	const TEXTSTYLE& style
	);
	*/
#endif //!< GENERATING_DOXYGEN_OUTPUT

#endif	//!<  __cplusplus 

//!@}

#define XeIsXeText(w) XtIsSubclass((w), xeTextEdWidgetClass)


#endif  // INC_MI32_TEXTED_H
