/******************************************************************
* \file mi32/variogram.h - 
*
* \if NODOC
* $Id: variogram.h_v 1.4 2013/01/18 16:30:29 mju Exp $
*
* $Log: variogram.h_v $
* Revision 1.4  2013/01/18 16:30:29  mju
* Remove mxdialog dependency.
*
* Revision 1.2  2007/05/09 21:55:58  vdronov
* added plot drawing
*
* Revision 1.1  2007/03/08 23:51:12  vdronov
* Initial revision
* \endif
*******************************************************************/

#ifndef INC_MI32_VARIOGRAM_H
#define INC_MI32_VARIOGRAM_H

#ifndef	INC_MI32_DOUBLEAR_H
#include <mi32/doublear.h>
#endif

#ifndef	INC_MI32_SIMPLEAR_H
#include <mi32/simplear.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations
struct McbAnyCallbackStruct;
#endif


class VARIOGRAM {
	public:

		enum MODEL {
			MODEL_Linear = 0,
			MODEL_Spherical = 1,
			MODEL_Exponential = 2,
			MODEL_Quadratic = 3,
			MODEL_Gaussian = 4
			};

		VARIOGRAM (
			);

		VARIOGRAM (
			const VARIOGRAM& rhs
			);

		~VARIOGRAM (
			);

		VARIOGRAM operator= (
			const VARIOGRAM& rhs
			);

		void Clear ();

		double ComputeCovariance (
			const DPOINT2D& p1,
			const DPOINT2D& p2
			) const ;

		double GetAnisotropicAngle (
			) const { return m_AnisotropicAngle; }

		double GetAnisotropicRatio (
			) const { return m_AnisotropicRatio; }

		MODEL GetModel (
			) const { return m_Model; }

		double GetNugget (
			) const { return m_Nugget; }

		double GetRange (
			) const { return m_Range; }

		double GetSill (
			) const { return m_Sill; }

		ERRVALUE Init (
			const DOUBLE_ARRAY<DPOINT3D>& points,
			const DRECT3D extents
			);

		bool IsInitialized (
			) const { return m_IsInitialized; }

		void SetAnisotropicAngle (
			const double angle
			);

		double SetAnisotropicRatio (
			const double ratio
			);

		void SetModel (
			const MODEL model
			);

		void SetNugget (
			const double nugget
			);

		void SetRange (
			const double range
			);

		void SetSill (
			const double sill
			);

		ERRVALUE DisplayPlot ();

		ERRVALUE Save ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		bool m_IsInitialized;

		MODEL m_Model;
		double m_Nugget;
		double m_Range;
		double m_Sill;
		double m_AnisotropicAngle;
		double m_AnisotropicRatio;
		MXDHANDLE m_XYHandle;						

		///////////////

		double m_C;
		double m_Sin;
		double m_Cos;
		bool m_IsAnisotropic;

		///////////////////

		static void CB_XYDestroy (McbAnyCallbackStruct *cbs, void *vdata);

		void CheckAnisotropic ();
		double ComputeLinear (const double distance) const;
		double ComputeSpherical (const double distance) const;
		double ComputeExponential (const double distance) const;
		double ComputeQuadratic (const double distance) const;
		double ComputeGaussian (const double distance) const;

		//////////////////

		struct ITEM {
			double m_Variance;
			double m_Distance;
			INT32 m_Count;
			};

		SIMPLE_ARRAY<ITEM> m_Items;
		double m_LagLength;
		INT32 m_NumLags;

		//////////////////

		double EstimateSill (const DOUBLE_ARRAY<DPOINT3D>& points);
		double EstimateRange (const double sill);
		ERRVALUE Setup (const DRECT3D& extents, const INT32 number);
		ERRVALUE Normalize ();
		ERRVALUE Add (const DPOINT3D& point);
		ERRVALUE UpdatePlot ();

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#endif
