/**
 * \file rvc/container.h 
 * \brief interface for the RVC::CONTAINER class.
 *
 * \if NODOC
 * $Id: container.h_v 1.2 2006/04/19 15:47:02 scowan Exp $
 *
 * $Log: container.h_v $
 * Revision 1.2  2006/04/19 15:47:02  scowan
 * *** empty log message ***
 *
 * Revision 1.1  2006/04/19 15:32:32  fileserver!scowan
 * Initial revision
 *
 * \endif
**/

//!
//!	\class RVC::CONTAINER rvc/container.h
//!	\brief Manages RVC::OBJTYPE_Container RVC objects.  
//!
//!	This class inherits from RVC::OBJECT
//!	
//!	RVC Object Types that this class supports: RVC::OBJTYPE_Container.
//!
//!	Default RVC Object Type for this class: RVC::OBJTYPE_Container.
//!
//!	Valid parents for this class:
//!		Everything but...
//!		- RVC::OBJTYPE_File
//!		- RVC::OBJTYPE_Folder

#ifndef INC_RVC_CONTAINER_H
#define INC_RVC_CONTAINER_H

#ifndef INC_RVC_OBJECT_H
	#include <rvc/object.h>
#endif

namespace RVC {

class CONTAINER : public RVC::OBJECT {  
	public:

		//! Default constructor
		CONTAINER (
			);

		//! Copy constructor
		CONTAINER (
			const RVC::CONTAINER& obj
			);

		//! Destructor
		virtual ~CONTAINER (
			);

		//! Assignment
		RVC::CONTAINER& operator= (
			const RVC::CONTAINER& rhs
			);

		//! Make a child rvc container object from a given parent with a name and description
		ERRVALUE Make (
			const RVC::OBJECT& parent,
			const RVC::DESCRIPTOR& Descriptor,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Make a child rvc container object from a given RVCOBJITEM
		ERRVALUE Make (
			const RVC::OBJITEM& ObjItem,
			const char* Source = 0,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Implementation of RVC::OBJECT virtual methods
		virtual OBJTYPE v_GetDftObjectType () const;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}		//! End of RVC namespace

#endif		//!< INC_RVC_CONTAINER_H
