/**
 * \file rvc/dlgimghg.h
 * \brief RVC::DLG_IMAGE_HISTOGRAM class definitions
 *
 * \if NODOC
 * $Id: dlgimghg.h_v 1.6 2011/09/02 13:06:59 mju Exp $
 *
 * $Log: dlgimghg.h_v $
 * Revision 1.6  2011/09/02 13:06:59  mju
 * Exclude macro from dox.
 *
 * Revision 1.5  2004/07/12 20:02:30  mju
 * Use region2d.
 *
 * Revision 1.4  2003/12/16 16:05:37  mju
 * Use dlgshell.
 *
 * Revision 1.1  2002/08/13 17:18:20  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_RVC_DLGIMGHG_H
#define	INC_RVC_DLGIMGHG_H

#ifndef	INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif


class REGION2D;

namespace RVC {

class OBJITEM;


class DLG_IMAGE_HISTOGRAM : public MGUI::DLGSHELL {
	public:

		enum STYLE {
			STYLE_Default =			0x00,
			STYLE_DeleteOnClose =	0x01,		//!< Delete dialog instance when closed
			STYLE_NoSelectObject =	0x02,		//!< Don't allow user to select object to view histogram of
			STYLE_NoSaveAsText =		0x04,		//!< Don't allow saving of information to text file
			STYLE_NoMultiObject =	0x08
			};

		typedef MILIST<DLG_IMAGE_HISTOGRAM*> DLGLIST;

		//! Get list of all histogram dialogs.
		static DLGLIST& GetDlgList (
			) { return (s_DlgList); }

		//! Constructor.
		explicit DLG_IMAGE_HISTOGRAM (
			STYLE style = STYLE_Default
			);

		//! Destructor.
		virtual ~DLG_IMAGE_HISTOGRAM (
			);

		//! Add object to view histogram of.
		ERRVALUE AddObject (
			const RVC::OBJITEM& objitem,
			COLORREF color = 0x000000
			);

		//! Determine if dialog has specified object.
		bool HasObject (
			const RVC::OBJITEM& objitem
			) const;

		//! Set region for histogram.
		ERRVALUE SetRegion (
			const REGION2D& region
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		static DLGLIST s_DlgList;

		class PRIV;
		PRIV *m_pPriv;

		virtual ERRVALUE v_CreateContent ();

		DLG_IMAGE_HISTOGRAM (const DLG_IMAGE_HISTOGRAM&);
		DLG_IMAGE_HISTOGRAM& operator= (const DLG_IMAGE_HISTOGRAM&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(DLG_IMAGE_HISTOGRAM::STYLE)
#endif

}	// End of namespace RVC


#endif	// INC_RVC_DLGIMGHG_H
