/**
 * \file rvc/opgeomtovoronoi.h
 * \brief Operation to convert geometric points to voronoi polygons
 *
 * \if NODOC
 * $Id: opgeomtovoronoi.h_v 1.3 2011/09/02 12:53:09 mju Exp $
 *
 * $Log: opgeomtovoronoi.h_v $
 * Revision 1.3  2011/09/02 12:53:09  mju
 * Dox hdr.
 *
 * Revision 1.2  2006/11/13 18:34:26  mju
 * Fix doxygen comment.
 *
 * Revision 1.1  2006/09/11 17:16:25  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_OPGEOMTOVORONOI_H
#define	INC_RVC_OPGEOMTOVORONOI_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class BITSET_UNOWNED;
namespace RVC {
	class OBJITEM;
	}
#endif

namespace RVC {
namespace OP {

//=====================================================================================================================

//! Generate voronoi polygons from a source point object
class GEOMETRIC_TO_VORONOI_POLYGONS {
	public:

		static ERRVALUE Process (
			const RVC::OBJITEM& SrcGeomObjItem,	//!< Source geometric object that contains points
			BITSET_UNOWNED* PointSel,				//!< Bit-array for selecting points for processing (may be NULL to select all)
			BITSET_UNOWNED* OutputSel,				//!< Bit-array for selecting points to merge together (may be NULL if just want simple Voronoi)
			RVC::OBJITEM& DestVectObjItem,		//!< Destination RVC::OBJITEM genreated with CreateNew(), RETURNED with voronoi vector object
			double ExtFactor							//!< Boundary extension factor between 0.0....1.0
			);
	};

//=====================================================================================================================

}	// End namespace OP
}	// End namespace RVC

#endif	// INC_RVC_OPGEOMTOVORONOI_H
