/**
 * \file rvc/prtcolbl.h 
 * \brief interface for the RVC::PRINTER_COLOR_BALANCE class.
 *
 * \if NODOC
 * $Id: prtcolbl.h_v 1.15 2007/09/06 17:18:53 scowan Exp $
 *
 * $Log: prtcolbl.h_v $
 * Revision 1.15  2007/09/06 17:18:53  scowan
 * Read and write virtual methods changed to take objopenparms.
 *
 * Revision 1.14  2007/04/09 17:47:19  dwilliss
 * Changed printer gamma structure
 *
 * Revision 1.13  2007/04/06 22:29:21  dwilliss
 * changed to store contrast in contrastparm internally
 *
 * Revision 1.12  2006/10/10 14:29:33  dwilliss
 * Added a private method.  Overwrite make data
 *
 * Revision 1.11  2005/12/09 17:29:46  scowan
 * Updated to use credentials.
 *
 * Revision 1.10  2005/07/06 16:31:20  scowan
 * Added management of reading and writing color balance information.
 *
 * Revision 1.9  2005/01/21 23:18:17  scowan
 * Fixed deprecated items.
 *
 * Revision 1.8  2003/10/07 17:31:04  scowan
 * Object virtual method changes.
 *
 * Revision 1.7  2003/09/30 16:52:06  dwilliss
 * Doxygen
 *
 * Revision 1.6  2003/09/25 21:20:04  dwilliss
 * Fixed doxygen comment
 *
 * Revision 1.5  2003/09/15 13:49:45  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.4  2001/06/15 17:54:45  scowan
 * More const.
 *
 * Revision 1.3  2001/06/15 15:02:38  scowan
 * Added to RVC namespace.
 *
 * Revision 1.2  2000/10/19 21:13:33  scowan
 * Checked in for vacation.
 *
 * Revision 1.1  2000/09/29 19:59:54  scowan
 * Initial revision
 *
 * \endif
**/

//!	\class RVC::PRINTER_COLOR_BALANCE rvc/prtcolbl.h
//!	\brief manages RVC::OBJTYPE_PrinterColorBalance objects.
//!
//!	RVC Object Types that this class supports: RVC::OBJTYPE_PrinterColorBalance
//!
//!	Default RVC Object Type for this class: RVC::OBJTYPE_PrinterColorBalance
//!
//!	Valid parents for this class:
//!		- All types
//!
//!	Read() and Write() methods inherited from RVC::SIMPLE class.  INI Handle from RVC::INITEXT class.
//!	View RVC::SIMPLE and RVC::INITEXT docs for inherited methods

#ifndef INC_RVC_PRTCOLBL_H
#define INC_RVC_PRTCOLBL_H

#ifndef INC_RVC_INITEXT_H
	#include <rvc/initext.h>
#endif

#ifndef INC_MI32_COLORBAL_H
	#include <mi32/colorbal.h>
#endif

#ifndef	INC_MI32_MPRINT_H
	#include <mi32/mprint.h>
#endif

namespace RVC {

class PRINTER_COLOR_BALANCE : public RVC::INITEXT {
	public:
		PRINTER_COLOR_BALANCE (
			);

		PRINTER_COLOR_BALANCE (
			const RVC::PRINTER_COLOR_BALANCE& rhs
			);

		virtual ~PRINTER_COLOR_BALANCE (
			);

		RVC::PRINTER_COLOR_BALANCE& operator= (
			const RVC::PRINTER_COLOR_BALANCE& rhs
			);
			
		const CBPARMS& GetColorBalanceParms (
			) const { return (m_CBParms); }

		//! Fills in an array of 3 PRINTGAMMAs
		void GetGammaParms (
			PRINTGAMMA* Gamma
			) const;

		void SetColorBalanceParms (
			const CBPARMS& CBParms
			) { m_CBParms = CBParms; }

		//! Takes an array of 3 PRINTGAMMAs 
		void SetGammaParms (
			const PRINTGAMMA* Gamma
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		CBPARMS m_CBParms;
		PRINTGAMMA m_Gamma[3];
		
		//! Implementation of RVC::OBJECT virtual methods
		virtual OBJTYPE v_GetDftObjectType () const;
		
		//! Implementation of RVC::SIMPLE virtual methods
		virtual ERRVALUE v_MakeData (RVC::OBJMAKEPARMS& MakeParms);
		virtual ERRVALUE v_ReadData (RVC::OBJOPENPARMS& ObjOpenParms);
		virtual ERRVALUE v_WriteData (RVC::OBJOPENPARMS& ObjOpenParms);
		
		//! Implementation of RVC::INITEXT virtual methods
		virtual ERRVALUE v_GetDftNameDesc (const OBJECT& parent, DESCRIPTOR& Descriptor);

		void WriteSub();

		#endif // GENERATING_DOXYGEN_OUTPUT

	};

}		//! End of RVC namespace

#endif		//!< INC_RVC_PRTCOLBL_H
