/**
 * \file rvc/uiformdbfieldpicklist.h
 *
 * \if NODOC
 * $Id: uiformdbfieldpicklist.h_v 1.9 2011/12/27 16:50:06 mju Exp $
 *
 * $Log: uiformdbfieldpicklist.h_v $
 * Revision 1.9  2011/12/27 16:50:06  mju
 * Remove redundant hdr.
 *
 * Revision 1.8  2010/02/22 18:52:24  mju
 * Specify table/field FOR when create.
 *
 * Revision 1.7  2010/02/22 15:11:02  mju
 * Remove unused member.
 *
 * Revision 1.6  2010/02/19 22:40:49  mju
 * Add dlgEditSettings.
 *
 * Revision 1.5  2010/02/18 18:32:41  mju
 * Move some stuff to private picklist class.
 *
 * Revision 1.4  2010/02/15 18:54:43  mju
 * Simplify by use of dbtable::record::value.
 *
 * Revision 1.3  2010/02/12 22:47:53  mju
 * Redesign to use tree so can support hierarchical.
 *
 * Revision 1.2  2009/08/21 17:59:01  mju
 * Change sort to return errvalue.
 *
 * Revision 1.1  2009/08/21 13:32:53  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_UIFORMDBFIELDPICKLIST_H
#define	INC_RVC_UIFORMDBFIELDPICKLIST_H

#ifndef	INC_MGUI_TREE_H
#include <mgui/tree.h>
#endif

#ifndef	INC_MGUI_COMBOBOX_H
#include <mgui/combobox.h>
#endif

#ifndef	INC_MGUI_IMAGELST_H
#include <mgui/imagelst.h>
#endif

#ifndef	INC_RVC_DBTABLE_H
#include <rvc/dbtable.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class SERIALIZERITEM;
#endif


namespace RVC {
namespace UI {
//=====================================================================================================================

//! Form for picking database field value from list.
class FORM_DB_FIELD_PICKLIST : public MGUI::FORM_COMPOSITE {
	public:

		enum FORMSTYLE {
			FORMSTYLE_SelectSingle =		0x0000,		//!< Single selection
			FORMSTYLE_SelectMulti =			0x0001,		//!< Allow selection of multiple items
			FORMSTYLE_ComboBox =				0x0002,		//!< Use combobox for selection, cannot use with FORMSTYLE_SelectMulti
			FORMSTYLE_AllowSelectNone =	0x0004,		//!< User is allowed choice of <none>
			FORMSTYLE_UseGroupBox =			0x0008,		//!< Place form in groupbox even if no label
			FORMSTYLE_NoEditSettings =		0x0010,		//!< Don't include Settings choice on popup menu
			};

		class SETTINGS {
			public:
				static const SERIALIZERITEM* SerialGetItemDef ();

				SETTINGS ();
				~SETTINGS ();

				INT32 GetFieldNumDisplay (
					) const { return (m_FieldNumDisplay); }
				INT32 GetFieldNumGroupBy (
					) const { return (m_FieldNumGroupBy); }
				INT32 GetFieldNumSortBy (
					) const { return (m_FieldNumSortBy); }
				const LPOINT2D& GetGridSize (
					) const { return (m_GridSize); }
				bool GetStyleSampleShow (
					) const { return (m_StyleSampleShow); }
				INT32 GetStyleSampleSize (
					) const { return (m_StyleSampleSize); }
				bool IsChanged (
					) const { return (m_IsChanged); }
				bool IsEqual (
					const SETTINGS& rhs
					) const;
				void ResetChanged (
					) { m_IsChanged = false; }
				void SetFieldNumDisplay (
					INT32 FieldNumDisplay
					) { if (FieldNumDisplay != m_FieldNumDisplay) {m_IsChanged = true; m_FieldNumDisplay = FieldNumDisplay; } }
				void SetFieldNumGroupBy (
					INT32 FieldNumGroupBy		//!< Field number, -1 for automatic, -2 for none
					) { if (FieldNumGroupBy != m_FieldNumGroupBy) {m_IsChanged = true; m_FieldNumGroupBy = FieldNumGroupBy; } }
				void SetFieldNumSortBy (
					INT32 FieldNumSortBy
					) { if (FieldNumSortBy != m_FieldNumSortBy) {m_IsChanged = true; m_FieldNumSortBy = FieldNumSortBy; } }
				void SetGridSize (
					INT32 x,
					INT32 y
					) { if (x != m_GridSize.x || y != m_GridSize.y) { m_IsChanged = true; m_GridSize.x = x; m_GridSize.y = y; } }
				void SetGridSize (
					const LPOINT2D& GridSize
					) { if (GridSize != m_GridSize) { m_IsChanged = true; m_GridSize = GridSize; } }
				void SetStyleSampleShow (
					bool StyleSampleShow
					) { if (StyleSampleShow != m_StyleSampleShow) { m_IsChanged = true; m_StyleSampleShow = StyleSampleShow; } }
				void SetStyleSampleSize (
					INT32 StyleSampleSize
					) { if (StyleSampleSize != m_StyleSampleSize) { m_IsChanged = true; m_StyleSampleSize = StyleSampleSize; } }
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				typedef SETTINGS THISCLASS;
				bool m_IsChanged;
				bool m_StyleSampleShow;
				INT32 m_StyleSampleSize;
				INT32 m_FieldNumDisplay;
				INT32 m_FieldNumSortBy;
				INT32 m_FieldNumGroupBy;
				LPOINT2D m_GridSize;
				#endif
			};

		//! Dialog to change picklist form settings.
		//! @return TRUE if changed, FALSE if not, < 0 if error.
		static int DlgEditSettings (
			MDLGPARENT dlgparent,
			const RVC::DBTABLE& TableFor,
			INT32 FieldNumFor,
			FORM_DB_FIELD_PICKLIST::SETTINGS& settings,		//!< Settings passed/returned
			bool ForGrouping = false
			);

		//! Constructor.
		FORM_DB_FIELD_PICKLIST ();

		//! Destructor.
		virtual ~FORM_DB_FIELD_PICKLIST ();

		//! Create with default settings.
		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,
			const MISTRING& label,
			const RVC::DBTABLE& TableFor,			//!< Table picklist is FOR
			INT32 FieldNumFor,						//!< Field picklist is FOR
			FORMSTYLE FormStyle,
			MGUI::LAYOUT_SIZEALIGN SizeAlign = MGUI::LAYOUT_SIZEALIGN_Expand
			);

		//! Create with specified settings.
		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,
			const MISTRING& label,
			const RVC::DBTABLE& TableFor,			//!< Table picklist is FOR
			INT32 FieldNumFor,						//!< Field picklist is FOR
			const SETTINGS& settings,
			FORMSTYLE FormStyle,
			MGUI::LAYOUT_SIZEALIGN SizeAlign = MGUI::LAYOUT_SIZEALIGN_Expand
			);

		//! Get current settings.
		const SETTINGS& GetSettings (
			) const { return (m_Settings); }

		//! Get table picking from.
		const RVC::DBTABLE& GetTablePickFrom (
			) const;

		//! Get value for selected item in single-selection mode, if any.
		ERRVALUE GetValueSelected (
			RVC::DBTABLE::RECORD& RecordRet,				//!< Record to place value into
			INT32 FieldNumRet									//!< Field in RecordRet to contain value
			) const;

		//! Determine if has any item selected.
		bool HasSelection (
			) const;

		//! Determine if value from specified record is selected.
		bool IsValueSelected (
			const RVC::DBTABLE::RECORD& RecordTest,	//!< Record containing value to test
			INT32 FieldNumTest								//!< Field in RecordTest containing value to test
			) const;

		//! Set delegate for when selection changed.
		void SetDelegateOnChangeSelection (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnChangeSelection = delegate; }

		//! Set value selected.
		ERRVALUE SetValueSelected (
			const RVC::DBTABLE::RECORD& RecordSet,		//!< Record to get value to set from
			INT32 FieldNumSet,								//!< Field in RecordSet to set from
			bool notify = true								//!< Notify via delegate if changed
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		typedef FORM_DB_FIELD_PICKLIST THISCLASS;

		class SET_VALUE;
		class PICKLIST;

		FORMSTYLE m_FormStyle;
		SETTINGS m_Settings;
		RVC::DBTABLE m_TableFor;
		INT32 m_FieldNumFor;
		PICKLIST *m_pPickList;
		MGUI::CTRL_GROUPBOX m_GroupBox;
		MGUI::CTRL_PUSHBUTTON m_BtnSelectAll;
		MGUI::CTRL_PUSHBUTTON m_BtnClearAll;
		MGUI::CTRL_TREE m_Tree;
		MGUI::FORM_COMBOBOX m_Cbx;
		MGUI::IMAGELIST m_ImageList;
		int m_IconIdxOff;
		int m_IconIdxOn;
		RVC::DBTABLE_STYLE *m_pTableStyle;
		RVC::DBTABLE::RECORD m_RecordStyle;
		INT32 m_FieldNumStyleIdx;
		INT32 m_FieldNumStyleName;
		RVC::STYLE *m_pObjStyle;
		RVC::STYLETYPE m_StyleType;
		bool m_CanShowStyle;
		bool m_HasPointStyle;
		bool m_HasLineStyle;
		bool m_HasPolygonStyle;
		UINT8 m_MaxStyleCount;
		BITSET m_SelectedSet;
		INT32 m_AddNone;
		INT32 m_SelectedIdx;
		mutable SET_VALUE *m_pSetValuesSelected;	// Set of currently selected pick values
		DELEGATE_VOID_NOPARMS m_DelegateOnChangeSelection;

		ERRVALUE AddIdxToSet (INT32 idx) const;
		void ClearValueSets ();
		MGUI::CTRL_TREE::ITEMHND GetTreeItemHndFromIdx (INT32 idx) const
			{ return (m_Tree.FindChildByID(MGUI::CTRL_TREE::ITEMHND_Root,idx,true)); }
		void DoPopupRightClick (MGUI::CTRL& MenuParent, const LPOINT2D& pt);
		void LoadSettings ();
		void NotifyChangeSelection ();
		void OnBtnClearAll ();
		void OnBtnSelectAll ();
		void OnCbxSelection ();
		MGUI::CUSTOMDRAW::RETVALUE OnTreeCustomDraw (MGUI::CUSTOMDRAW_TREE& CustomDraw);
		bool OnTreePointerButtonDown (MGUI::CTRL_TREE::ITEMHND hndItem, MGUI::CTRL_TREE::HITRESULT HitResult, const LPOINT2D& pt, MGUI::POINTERBUTTON button, MGUI::KEYSTATE keystate);
		bool OnTreePointerButtonUp (MGUI::CTRL_TREE::ITEMHND hndItem, MGUI::CTRL_TREE::HITRESULT HitResult, const LPOINT2D& pt, MGUI::POINTERBUTTON button, MGUI::KEYSTATE keystate);
		void OnTreeSize (int xsize, int ysize);
		void SaveSettings ();
		void SetSelectedIdx (INT32 SelectedIdx);
		void SetupTree ();
		ERRVALUE SetupTreeHierarchical ();
		ERRVALUE SetupStyleType ();
		bool ShowStyleSample (MGUI::CUSTOMDRAW_TREE& CustomDraw, const LRECT2D& rect, RVC::STYLETYPE StyleType, INT32 FieldNumStyleIdx, INT32 FieldNumStyleName);

		#endif
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FORM_DB_FIELD_PICKLIST::FORMSTYLE)
#endif

inline bool operator== (
	const FORM_DB_FIELD_PICKLIST::SETTINGS& lhs,
	const FORM_DB_FIELD_PICKLIST::SETTINGS& rhs
	) { return (lhs.IsEqual(rhs)); }

inline bool operator!= (
	const FORM_DB_FIELD_PICKLIST::SETTINGS& lhs,
	const FORM_DB_FIELD_PICKLIST::SETTINGS& rhs
	) { return (!lhs.IsEqual(rhs)); }


//=====================================================================================================================
}	// End namespace UI
}	// End namespace RVC

#endif	// INC_RVC_UIFORMDBFIELDPICKLIST_H
