/******************************************************************************
 *
 * \file sml/classnames.h
 * \brief SML -- SMLCLASSNAME defines
 *
 * These defined values should be used wherever you need the name of an 
 * SMLCLASS to avoid use of incorrect names and to make code easier
 * to understand.
 *
 * Class names in SML are case-insensitive.  The convention is to make
 * them all uppercase.  Many existing scripts will have mixed case though.
 *
 * The "Mie" classes are not listed here for simplicity.
 *
 * Keep names in alphabetical order.
 *
 * \if NODOC
 * $Log: classnames.h_v $
 * Revision 1.37  2011/10/03 18:38:45  bcolombe
 * Add filter_palettize
 *
 * Revision 1.36  2011/08/02 20:59:06  mju
 * Add pipeline_target_bil/ers.
 *
 * Revision 1.35  2011/07/18 19:47:41  bcolombe
 * Add SOURCE_GDAL
 *
 * Revision 1.34  2011/05/12 21:59:59  bcolombe
 * Add TILESET class
 *
 * Revision 1.33  2011/02/02 14:32:13  mju
 * Add image_pipeline_autoregister.
 *
 * Revision 1.32  2010/12/06 19:13:51  bcolombe
 * Add GRE_LAYER_LIDAR
 *
 * Revision 1.31  2010/01/28 20:25:27  bcolombe
 * Added tileset source/targets
 *
 * Revision 1.30  2009/02/11 19:38:01  bcolombe
 * Added RVC_DATABASE_SHAPE
 *
 * Revision 1.29  2008/12/17 21:21:53  mju
 * Add image_filter_limit_range.
 *
 * Revision 1.28  2008/12/17 21:18:17  bcolombe
 * Added MIJOB
 *
 * Revision 1.27  2008/12/15 18:05:02  bcolombe
 * Added AUTOLABELPARMS
 *
 * Revision 1.26  2008/12/05 23:04:57  mju
 * Add gre_tool_... names.
 *
 * Revision 1.25  2008/11/20 22:18:23  bcolombe
 * Added FILTER_NULLTOALPHA
 *
 * Revision 1.24  2008/11/07 16:19:03  mju
 * Add image_pipeline_target_null.
 *
 * Revision 1.23  2008/10/10 17:22:59  bcolombe
 * Added ZIPFILE
 *
 * Revision 1.22  2008/02/20 15:41:54  mju
 * Move gui_... names here.
 *
 * Revision 1.21  2008/02/15 18:16:14  mju
 * Add gui_gadget...
 *
 * Revision 1.20  2008/02/14 18:28:18  mju
 * *** empty log message ***
 *
 * Revision 1.19  2008/02/14 15:23:52  mju
 * Add gre_tool_rectangle.
 *
 * Revision 1.18  2008/02/14 13:52:01  mju
 * Add gre_tool_...
 *
 * Revision 1.17  2008/01/30 17:45:28  bcolombe
 * Added SOURCE_REGION
 *
 * Revision 1.16  2008/01/04 22:27:22  bcolombe
 * Fixed GENERAL_CHGPROP typo
 *
 * Revision 1.15  2008/01/03 23:28:54  bcolombe
 * Added SAMPLEITERATOR
 *
 * Revision 1.14  2008/01/03 18:39:36  bcolombe
 * Added FILTER_DIVIDE
 *
 * Revision 1.13  2007/11/12 04:19:14  bcolombe
 * Added SOURCE_MRSID
 *
 * Revision 1.12  2007/10/11 20:34:43  bcolombe
 * Added RVC_FOLDER
 *
 * Revision 1.11  2007/10/01 16:33:54  mju
 * Add gre_view_base.
 *
 * Revision 1.10  2007/09/11 19:55:22  bcolombe
 * Added filter_focal pipeline classnames
 *
 * Revision 1.9  2007/08/03 16:02:10  dwilliss
 * added GRE_OBJECT
 *
 * Revision 1.8  2007/07/26 16:22:49  bcolombe
 * Added TARGET_MGD, TARGET_RVC_BASE, TARGET_RVC_MULTIFILE
 *
 * Revision 1.7  2007/07/16 14:11:22  bcolombe
 * Added stereoparm, filter_stereo, filter_cacheimage
 *
 * Revision 1.6  2007/06/20 19:42:01  bcolombe
 * Added SOURCE_TIFF class name
 *
 * Revision 1.5  2007/05/31 20:36:55  drb
 * Added email classname
 *
 * Revision 1.4  2007/05/24 13:58:52  mju
 * Add target_tiff_settings.
 *
 * Revision 1.3  2007/05/07 14:05:00  dwilliss
 * *** empty log message ***
 *
 * Revision 1.2  2007/05/04 19:58:37  dwilliss
 * comments
 *
 * Revision 1.1  2007/05/04 19:56:08  dwilliss
 * Initial revision
 *
 * \endif
 */
/*****************************************************************************/

#ifndef INC_SML_CLASSNAMES_H
#define INC_SML_CLASSNAMES_H

#define SMLCLASSNAME_ALLOCATION				"ALLOCATION"
#define SMLCLASSNAME_AUTOLABELPARMS			"AUTOLABELPARMS"
#define SMLCLASSNAME_BITMAPPATTERN			"BITMAPPATTERN"
#define SMLCLASSNAME_BITSET_ITERATOR		"BITSET_ITERATOR"
#define SMLCLASSNAME_BUTTONITEM				"BUTTONITEM"
#define SMLCLASSNAME_CAD					"CAD"
#define SMLCLASSNAME_CADINFO				"CADINFO"
#define SMLCLASSNAME_CADELEMOPT				"CADELEMOPT"
#define SMLCLASSNAME_CALLBACKLIST			"CALLBACKLIST"
#define SMLCLASSNAME_CBPARMS				"CBPARMS"
#define SMLCLASSNAME_CENTER					"CENTER"
#define SMLCLASSNAME_COLOR					"COLOR"
#define SMLCLASSNAME_COLORMAP				"COLORMAP"
#define SMLCLASSNAME_COMPOSITEWIDGET		"COMPOSITEWIDGET"
#define SMLCLASSNAME_CONTEXT				"CONTEXT"
#define SMLCLASSNAME_CONTRAST				"CONTRAST"
#define SMLCLASSNAME_CTRLEDGE				"CTRLEDGE"
#define SMLCLASSNAME_CTRLPOINT				"CTRLPOINT"
#define SMLCLASSNAME_DATABASE				"DATABASE"
#define SMLCLASSNAME_DATATIP				"DATATIP"
#define SMLCLASSNAME_DATETIME				"DATETIME"
#define SMLCLASSNAME_DBEDITOR				"DBEDITOR"
#define SMLCLASSNAME_DBEDITOR_SINGLERECORDVIEW		"DBEDITOR_SINGLERECORDVIEW"
#define SMLCLASSNAME_DBEDITOR_SINGLERECORDWIDGET	"DBEDITOR_SINGLERECORDWIDGET"
#define SMLCLASSNAME_DBEDITOR_TABULARVIEW	"DBEDITOR_TABULARVIEW"
#define SMLCLASSNAME_DBEDITORTABLE			"DBEDITORTABLE"
#define SMLCLASSNAME_DBFIELDINFO			"DBFIELDINFO"
#define SMLCLASSNAME_DBTABLEINFO			"DBTABLEINFO"
#define SMLCLASSNAME_DBTABLEVAR				"DBTABLEVAR"
#define SMLCLASSNAME_DIALOGSHELL			"DIALOGSHELL"
#define SMLCLASSNAME_DISP					"DISP"
#define SMLCLASSNAME_DISPCALLBACKLIST		"DISPCALLBACKLIST"
#define SMLCLASSNAME_DISPLAYINFO			"DISPLAYINFO"
#define SMLCLASSNAME_EXIF					"EXIF"
#define SMLCLASSNAME_ELEMENTREF				"ELEMENTREF"  //old name. Now RVC_ELEMENT
#define SMLCLASSNAME_ELEMSELECTPARMS		"ELEMSELECTPARMS"
#define SMLCLASSNAME_EMAIL_MESSAGE			"EMAIL_MESSAGE"
#define SMLCLASSNAME_FILE					"FILE"
#define SMLCLASSNAME_FILEPATH				"FILEPATH"
#define SMLCLASSNAME_FRAME					"FRAME"
#define SMLCLASSNAME_GEOMETRICINFO			"GEOMETRICINFO"
#define SMLCLASSNAME_GEOREF					"GEOREF"
#define SMLCLASSNAME_GPSCALLBACKLIST		"GPSCALLBACKLIST"
#define SMLCLASSNAME_GPSDATA				"GPSDATA"
#define SMLCLASSNAME_GPSDBASE				"GPSDBASE"
#define SMLCLASSNAME_GPSPORT				"GPSPORT"
#define SMLCLASSNAME_GPSSYMBOL				"GPSSYMBOL"
#define SMLCLASSNAME_GRAPHICSCONTEXT		"GC"	// old name
#define SMLCLASSNAME_GC						"GC"
#define SMLCLASSNAME_GRDEVICE				"GRDEVICE"
#define SMLCLASSNAME_GRDEVICE_MEM_BINARY	"GRDEVICE_MEM_BINARY"
#define SMLCLASSNAME_GRE_GROUP				"GRE_GROUP"
#define SMLCLASSNAME_GRE_LABELS				"GRE_LABELS"
#define SMLCLASSNAME_GRE_LAYER				"GRE_LAYER"
#define SMLCLASSNAME_GRE_LAYER_CAD			"GRE_LAYER_CAD"
#define SMLCLASSNAME_GRE_LAYER_GEOFORMULA	"GRE_LAYER_GEOFORMULA"
#define SMLCLASSNAME_GRE_LAYER_LIDAR		"GRE_LAYER_LIDAR"
#define SMLCLASSNAME_GRE_LAYER_MAPGRID		"GRE_LAYER_MAPGRID"
#define SMLCLASSNAME_GRE_LAYER_PINMAP		"GRE_LAYER_PINMAP"
#define SMLCLASSNAME_GRE_LAYER_RASTER		"GRE_LAYER_RASTER"
#define SMLCLASSNAME_GRE_LAYER_REGION		"GRE_LAYER_REGION"
#define SMLCLASSNAME_GRE_LAYER_SCALEBAR	"GRE_LAYER_SCALEBAR"
#define SMLCLASSNAME_GRE_LAYER_SCRIPT		"GRE_LAYER_SCRIPT"
#define SMLCLASSNAME_GRE_LAYER_SHAPE		"GRE_LAYER_SHAPE"
#define SMLCLASSNAME_GRE_LAYER_TEXT			"GRE_LAYER_TEXT"
#define SMLCLASSNAME_GRE_LAYER_TIN			"GRE_LAYER_TIN"
#define SMLCLASSNAME_GRE_LAYER_VECTOR		"GRE_LAYER_VECTOR"
#define SMLCLASSNAME_GRE_LAYERDC				"GRE_LAYERDC"
#define SMLCLASSNAME_GRE_LAYERMANAGER		"GRE_LAYERMANAGER"
#define SMLCLASSNAME_GRE_LAYOUT				"GRE_LAYOUT"
#define SMLCLASSNAME_GRE_OBJECT				"GRE_OBJECT"
#define SMLCLASSNAME_GRE_TOOL_BUILTIN		"GRE_TOOL_BUILTIN"
#define SMLCLASSNAME_GRE_TOOL_GENERAL		"GRE_TOOL_GENERAL"
#define SMLCLASSNAME_GRE_TOOL_POINT			"GRE_TOOL_POINT"
#define SMLCLASSNAME_GRE_TOOL_POLYLINE		"GRE_TOOL_POLYLINE"
#define SMLCLASSNAME_GRE_TOOL_RECTANGLE	"GRE_TOOL_RECTANGLE"
#define SMLCLASSNAME_GRE_TOOL_REGION		"GRE_TOOL_REGION"
#define SMLCLASSNAME_GRE_TOOL_SEGMENT		"GRE_TOOL_SEGMENT"
#define SMLCLASSNAME_GRE_VECTOR_LABELS		"GRE_VECTOR_LABELS"
#define SMLCLASSNAME_GRE_VECTOR_LINES		"GRE_VECTOR_LINES"
#define SMLCLASSNAME_GRE_VECTOR_NODES		"GRE_VECTOR_NODES"
#define SMLCLASSNAME_GRE_VECTOR_POINTS		"GRE_VECTOR_POINTS"
#define SMLCLASSNAME_GRE_VECTOR_POLYS		"GRE_VECTOR_POLYS"
#define SMLCLASSNAME_GRE_VIEW					"GRE_VIEW"
#define SMLCLASSNAME_GRE_VIEW_BASE			"GRE_VIEW_BASE"
#define SMLCLASSNAME_GRE_VIEW3D				"GRE_VIEW3D"
#define SMLCLASSNAME_GRE_VIEWABLE			"GRE_VIEWABLE"
#define SMLCLASSNAME_GROUPXPOSN				"GROUPXPOSN"
#define SMLCLASSNAME_GROUPYPOSN				"GROUPYPOSN"
#define SMLCLASSNAME_GUI_CANVAS	 				"GUI_CANVAS"
#define SMLCLASSNAME_GUI_CMD_ROUTER		 		"GUI_CMD_ROUTER"
#define SMLCLASSNAME_GUI_CTRL			 			"GUI_CTRL"
#define SMLCLASSNAME_GUI_CTRL_COLORBUTTON		"GUI_CTRL_COLORBUTTON"
#define SMLCLASSNAME_GUI_CTRL_COMBOBOX	 		"GUI_CTRL_COMBOBOX"
#define SMLCLASSNAME_GUI_CTRL_EDIT_BASE		"GUI_CTRL_EDIT_BASE"
#define SMLCLASSNAME_GUI_CTRL_EDIT_NUMBER		"GUI_CTRL_EDIT_NUMBER"
#define SMLCLASSNAME_GUI_CTRL_EDIT_STRING		"GUI_CTRL_EDIT_STRING"
#define SMLCLASSNAME_GUI_CTRL_EDIT_TEXT		"GUI_CTRL_EDIT_TEXT"
#define SMLCLASSNAME_GUI_CTRL_GLG	 			"GUI_CTRL_GLG"
#define SMLCLASSNAME_GUI_CTRL_GROUPBOX			"GUI_CTRL_GROUPBOX"
#define SMLCLASSNAME_GUI_CTRL_LABEL				"GUI_CTRL_LABEL"
#define SMLCLASSNAME_GUI_CTRL_LISTBOX	 		"GUI_CTRL_LISTBOX"
#define SMLCLASSNAME_GUI_CTRL_MENUBUTTON	 	"GUI_CTRL_MENUBUTTON"
#define SMLCLASSNAME_GUI_CTRL_PUSHBUTTON	 	"GUI_CTRL_PUSHBUTTON"
#define SMLCLASSNAME_GUI_CTRL_TOGGLEBUTTON	"GUI_CTRL_TOGGLEBUTTON"
#define SMLCLASSNAME_GUI_DLG				 		"GUI_DLG"
#define SMLCLASSNAME_GUI_FORM_COMPOSITE 		"GUI_FORM_COMPOSITE"
#define SMLCLASSNAME_GUI_FORM_EDIT_NUMBER		"GUI_FORM_EDIT_NUMBER"
#define SMLCLASSNAME_GUI_FORM_EDIT_RANGE		"GUI_FORM_EDIT_RANGE"
#define SMLCLASSNAME_GUI_FORM_EDIT_STRING		"GUI_FORM_EDIT_STRING"
#define SMLCLASSNAME_GUI_FORM_HTML	 			"GUI_FORM_HTML"
#define SMLCLASSNAME_GUI_FORM_RADIOGROUP		"GUI_FORM_RADIOGROUP"
#define SMLCLASSNAME_GUI_FORMDATA		 		"GUI_FORMDATA"
#define SMLCLASSNAME_GUI_GADGET					"GUI_GADGET"
#define SMLCLASSNAME_GUI_GADGET_POINT			"GUI_GADGET_POINT"
#define SMLCLASSNAME_GUI_GADGET_POLYLINE		"GUI_GADGET_POLYLINE"
#define SMLCLASSNAME_GUI_GADGET_RECTANGLE		"GUI_GADGET_RECTANGLE"
#define SMLCLASSNAME_GUI_GADGET_REGION			"GUI_GADGET_REGION"
#define SMLCLASSNAME_GUI_GADGET_SEGMENT		"GUI_GADGET_SEGMENT"
#define SMLCLASSNAME_GUI_GADGETCALLBACKLIST	"GUI_GADGETCALLBACKLIST"
#define SMLCLASSNAME_GUI_LAYOUT_BOOK 			"GUI_LAYOUT_BOOK"
#define SMLCLASSNAME_GUI_LAYOUT_EQUALIZER		"GUI_LAYOUT_EQUALIZER"
#define SMLCLASSNAME_GUI_LAYOUT_PAGE 			"GUI_LAYOUT_PAGE"
#define SMLCLASSNAME_GUI_LAYOUT_PANE 			"GUI_LAYOUT_PANE"
#define SMLCLASSNAME_GUI_LAYOUT_PANE_BASE 	"GUI_LAYOUT_PANE_BASE"
#define SMLCLASSNAME_GUI_LAYOUT_PANE_ROOT 	"GUI_LAYOUT_PANE_ROOT"
#define SMLCLASSNAME_GUI_PANE_XML				"GUI_PANE_XML"
#define SMLCLASSNAME_GUI_TIMER					"GUI_TIMER"
#define SMLCLASSNAME_HATCHPATTERN			"HATCHPATTERN"
#define SMLCLASSNAME_HISTOGRAM				"HISTOGRAM"
#define SMLCLASSNAME_IMAGE_PIPELINE_AUTOREGISTER						"IMAGE_PIPELINE_AUTOREGISTER"
#define SMLCLASSNAME_IMAGE_PIPELINE_BASE						"IMAGE_PIPELINE_BASE"
#define SMLCLASSNAME_IMAGE_PIPELINE_DIMENSIONS					"IMAGE_PIPELINE_DIMENSIONS"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER						"IMAGE_PIPELINE_FILTER"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_CACHEIMAGE			"IMAGE_PIPELINE_FILTER_CACHEIMAGE"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_COLORBALANCE			"IMAGE_PIPELINE_FILTER_COLORBALANCE"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_COMPOSITE			"IMAGE_PIPELINE_FILTER_COMPOSITE"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_CROP					"IMAGE_PIPELINE_FILTER_CROP"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_DATATYPE				"IMAGE_PIPELINE_FILTER_DATATYPE"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_DATATYPE_SETTINGS	"IMAGE_PIPELINE_FILTER_DATATYPE_SETTINGS"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_DEINDEX				"IMAGE_PIPELINE_FILTER_DEINDEX"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_DIVIDE				"IMAGE_PIPELINE_FILTER_DIVIDE"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_EXTEND				"IMAGE_PIPELINE_FILTER_EXTEND"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_FOCAL				"IMAGE_PIPELINE_FILTER_FOCAL"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_FOCAL_AMPM			"IMAGE_PIPELINE_FILTER_FOCAL_AMPM"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_FOCAL_CONTRAST		"IMAGE_PIPELINE_FILTER_FOCAL_CONTRAST"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_FOCAL_CS				"IMAGE_PIPELINE_FILTER_FOCAL_CS"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_FOCAL_FROST			"IMAGE_PIPELINE_FILTER_FOCAL_FROST"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_FOCAL_GRADIENT		"IMAGE_PIPELINE_FILTER_FOCAL_GRADIENT"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_FOCAL_KUANADAPTIVE	"IMAGE_PIPELINE_FILTER_FOCAL_KUANADAPTIVE"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_FOCAL_LEE			"IMAGE_PIPELINE_FILTER_FOCAL_LEE"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_FOCAL_LUM			"IMAGE_PIPELINE_FILTER_FOCAL_LUM"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_FOCAL_MLM			"IMAGE_PIPELINE_FILTER_FOCAL_MLM"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_FOCAL_MULTIPLICATIVE	"IMAGE_PIPELINE_FILTER_FOCAL_MULTIPLICATIVE"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_FOCAL_OLYMPIC		"IMAGE_PIPELINE_FILTER_FOCAL_OLYMPIC"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_FOCAL_PMEDIAN		"IMAGE_PIPELINE_FILTER_FOCAL_PMEDIAN"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_FOCAL_RANGE			"IMAGE_PIPELINE_FILTER_FOCAL_RANGE"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_FOCAL_SIGMA			"IMAGE_PIPELINE_FILTER_FOCAL_SIGMA"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_FOCAL_STATISTICS		"IMAGE_PIPELINE_FILTER_FOCAL_STATISTICS"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_FOCAL_TEAGER			"IMAGE_PIPELINE_FILTER_FOCAL_TEAGER"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_FOCAL_VOLTERRA		"IMAGE_PIPELINE_FILTER_FOCAL_VOLTERRA"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_FOCAL_WMMR			"IMAGE_PIPELINE_FILTER_FOCAL_WMMR"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_FOCALTOPOGRAPHIC		"IMAGE_PIPELINE_FILTER_FOCALTOPOGRAPHIC"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_FOCALTOPOGRAPHIC_SETTINGS	"IMAGE_PIPELINE_FILTER_FOCALTOPOGRAPHIC_SETTINGS"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_FUSION				"IMAGE_PIPELINE_FILTER_FUSION"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_FUSION_BASICHBS		"IMAGE_PIPELINE_FILTER_FUSION_BASICHBS"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_FUSION_BASICHIS		"IMAGE_PIPELINE_FILTER_FUSION_BASICHIS"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_FUSION_BROVEY		"IMAGE_PIPELINE_FILTER_FUSION_BROVEY"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_FUSION_MODHIS		"IMAGE_PIPELINE_FILTER_FUSION_MODHIS"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_FUSION_TEXTUREPCA	"IMAGE_PIPELINE_FILTER_FUSION_TEXTUREPCA"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_GENERAL_CHGPROP		"IMAGE_PIPELINE_FILTER_GENERAL_CHGPROP"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_GENERAL_INPLACE		"IMAGE_PIPELINE_FILTER_GENERAL_INPLACE"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_LIMIT_RANGE			"IMAGE_PIPELINE_FILTER_LIMIT_RANGE"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_LINEAR				"IMAGE_PIPELINE_FILTER_LINEAR"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_LOOKUP				"IMAGE_PIPELINE_FILTER_LOOKUP"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_MARGIN				"IMAGE_PIPELINE_FILTER_MARGIN"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_MASKSELECT			"IMAGE_PIPELINE_FILTER_MASKSELECT"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_MASKVALIDITY			"IMAGE_PIPELINE_FILTER_MASKVALIDITY"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_MORPHOLOGY			"IMAGE_PIPELINE_FILTER_MORPHOLOGY"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_MORPHOLOGY_DILATION	"IMAGE_PIPELINE_FILTER_MORPHOLOGY_DILATION"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_MORPHOLOGY_EROSION	"IMAGE_PIPELINE_FILTER_MORPHOLOGY_EROSION"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_MORPHOLOGY_CLOSING	"IMAGE_PIPELINE_FILTER_MORPHOLOGY_CLOSING"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_MORPHOLOGY_OPENING	"IMAGE_PIPELINE_FILTER_MORPHOLOGY_OPENING"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_MOSAIC				"IMAGE_PIPELINE_FILTER_MOSAIC"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_MULTIPLY				"IMAGE_PIPELINE_FILTER_MULTIPLY"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_NULLTOALPHA			"IMAGE_PIPELINE_FILTER_NULLTOALPHA"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_OVERRIDETYPE			"IMAGE_PIPELINE_FILTER_OVERRIDETYPE"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_PALETTIZE			"IMAGE_PIPELINE_FILTER_PALETTIZE"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_PIXELTABLE			"IMAGE_PIPELINE_FILTER_PIXELTABLE"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_PIXELTYPE			"IMAGE_PIPELINE_FILTER_PIXELTYPE"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_QUANTIZE				"IMAGE_PIPELINE_FILTER_QUANTIZE"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_REPLACENULL			"IMAGE_PIPELINE_FILTER_REPLACENULL"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_RESAMPLE				"IMAGE_PIPELINE_FILTER_RESAMPLE"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_SCALEOFFSET			"IMAGE_PIPELINE_FILTER_SCALEOFFSET"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_SELECT				"IMAGE_PIPELINE_FILTER_SELECT"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_STEREO				"IMAGE_PIPELINE_FILTER_STEREO"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_TESTEQUAL			"IMAGE_PIPELINE_FILTER_TESTEQUAL"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_TESTRANGE			"IMAGE_PIPELINE_FILTER_TESTRANGE"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_VALIDITYNEAR			"IMAGE_PIPELINE_FILTER_VALIDITYNEAR"
#define SMLCLASSNAME_IMAGE_PIPELINE_FILTER_ZOOM					"IMAGE_PIPELINE_FILTER_ZOOM"
#define SMLCLASSNAME_IMAGE_PIPELINE_GEOREFERENCE				"IMAGE_PIPELINE_GEOREFERENCE"
#define SMLCLASSNAME_IMAGE_PIPELINE_PIXEL						"IMAGE_PIPELINE_PIXEL"
#define SMLCLASSNAME_IMAGE_PIPELINE_PIXELTABLE					"IMAGE_PIPELINE_PIXELTABLE"
#define SMLCLASSNAME_IMAGE_PIPELINE_SAMPLE						"IMAGE_PIPELINE_SAMPLE"
#define SMLCLASSNAME_IMAGE_PIPELINE_SAMPLEITERATOR				"IMAGE_PIPELINE_SAMPLEITERATOR"
#define SMLCLASSNAME_IMAGE_PIPELINE_SAMPLETABLE					"IMAGE_PIPELINE_SAMPLETABLE"
#define SMLCLASSNAME_IMAGE_PIPELINE_SOURCE						"IMAGE_PIPELINE_SOURCE"
#define SMLCLASSNAME_IMAGE_PIPELINE_SOURCE_CONSTANT				"IMAGE_PIPELINE_SOURCE_CONSTANT"
#define SMLCLASSNAME_IMAGE_PIPELINE_SOURCE_GDAL					"IMAGE_PIPELINE_SOURCE_GDAL"
#define SMLCLASSNAME_IMAGE_PIPELINE_SOURCE_GRDEVICE				"IMAGE_PIPELINE_SOURCE_GRDEVICE"
#define SMLCLASSNAME_IMAGE_PIPELINE_SOURCE_JPEG					"IMAGE_PIPELINE_SOURCE_JPEG"
#define SMLCLASSNAME_IMAGE_PIPELINE_SOURCE_MRSID				"IMAGE_PIPELINE_SOURCE_MRSID"
#define SMLCLASSNAME_IMAGE_PIPELINE_SOURCE_PNG					"IMAGE_PIPELINE_SOURCE_PNG"
#define SMLCLASSNAME_IMAGE_PIPELINE_SOURCE_REGION				"IMAGE_PIPELINE_SOURCE_REGION"
#define SMLCLASSNAME_IMAGE_PIPELINE_SOURCE_RVC					"IMAGE_PIPELINE_SOURCE_RVC"
#define SMLCLASSNAME_IMAGE_PIPELINE_SOURCE_TIFF					"IMAGE_PIPELINE_SOURCE_TIFF"
#define SMLCLASSNAME_IMAGE_PIPELINE_SOURCE_TILESET				"IMAGE_PIPELINE_SOURCE_TILESET"
#define SMLCLASSNAME_IMAGE_PIPELINE_SOURCE_WBMP					"IMAGE_PIPELINE_SOURCE_WBMP"
#define SMLCLASSNAME_IMAGE_PIPELINE_SPANS						"IMAGE_PIPELINE_SPANS"
#define SMLCLASSNAME_IMAGE_PIPELINE_STAGE						"IMAGE_PIPELINE_STAGE"
#define SMLCLASSNAME_IMAGE_PIPELINE_STAGE_ARRAY					"IMAGE_PIPELINE_STAGE_ARRAY"
#define SMLCLASSNAME_IMAGE_PIPELINE_SECTION						"IMAGE_PIPELINE_SECTION"
#define SMLCLASSNAME_IMAGE_PIPELINE_TARGET						"IMAGE_PIPELINE_TARGET"
#define SMLCLASSNAME_IMAGE_PIPELINE_TARGET_BIL					"IMAGE_PIPELINE_TARGET_BIL"
#define SMLCLASSNAME_IMAGE_PIPELINE_TARGET_ERS					"IMAGE_PIPELINE_TARGET_ERS"
#define SMLCLASSNAME_IMAGE_PIPELINE_TARGET_FILE					"IMAGE_PIPELINE_TARGET_FILE"
#define SMLCLASSNAME_IMAGE_PIPELINE_TARGET_GRDEVICE			"IMAGE_PIPELINE_TARGET_GRDEVICE"
#define SMLCLASSNAME_IMAGE_PIPELINE_TARGET_J2K					"IMAGE_PIPELINE_TARGET_J2K"
#define SMLCLASSNAME_IMAGE_PIPELINE_TARGET_J2K_SETTINGS		"IMAGE_PIPELINE_TARGET_J2K_SETTINGS"
#define SMLCLASSNAME_IMAGE_PIPELINE_TARGET_JPEG					"IMAGE_PIPELINE_TARGET_JPEG"
#define SMLCLASSNAME_IMAGE_PIPELINE_TARGET_NULL					"IMAGE_PIPELINE_TARGET_NULL"
#define SMLCLASSNAME_IMAGE_PIPELINE_TARGET_PNG					"IMAGE_PIPELINE_TARGET_PNG"
#define SMLCLASSNAME_IMAGE_PIPELINE_TARGET_RVC					"IMAGE_PIPELINE_TARGET_RVC"
#define SMLCLASSNAME_IMAGE_PIPELINE_TARGET_RVC_BASE			"IMAGE_PIPELINE_TARGET_RVC_BASE"
#define SMLCLASSNAME_IMAGE_PIPELINE_TARGET_RVC_MULTIFILE		"IMAGE_PIPELINE_TARGET_RVC_MULTIFILE"
#define SMLCLASSNAME_IMAGE_PIPELINE_TARGET_TIFF					"IMAGE_PIPELINE_TARGET_TIFF"
#define SMLCLASSNAME_IMAGE_PIPELINE_TARGET_TIFF_SETTINGS		"IMAGE_PIPELINE_TARGET_TIFF_SETTINGS"
#define SMLCLASSNAME_IMAGE_PIPELINE_TARGET_TILESET_GOOGLEMAPS	"IMAGE_PIPELINE_TARGET_TILESET_GOOGLEMAPS"
#define SMLCLASSNAME_IMAGE_PIPELINE_TARGET_TILESET_GOOGLEEARTH	"IMAGE_PIPELINE_TARGET_TILESET_GOOGLEEARTH"
#define SMLCLASSNAME_IMAGE_PIPELINE_TARGET_TILESET_MICROSOFTBING	"IMAGE_PIPELINE_TARGET_TILESET_MICROSOFTBING"
#define SMLCLASSNAME_IMAGE_PIPELINE_TARGET_TILESET_NASAWORLDWIND	"IMAGE_PIPELINE_TARGET_TILESET_NASAWORLDWIND"
#define SMLCLASSNAME_IMAGE_PIPELINE_TILESET			"IMAGE_PIPELINE_TILESET"
#define SMLCLASSNAME_INIHANDLE				"INIHANDLE"
#define SMLCLASSNAME_LABELATTACHMENT		"LABELATTACHMENT"
#define SMLCLASSNAME_LABELFRAMEPARMS		"LABELFRAMEPARMS"
#define SMLCLASSNAME_LABELPLACE				"LABELPLACE"
#define SMLCLASSNAME_LABELSTYLE				"LABELSTYLE"
#define SMLCLASSNAME_LEGENDVIEW				"LEGENDVIEW"
#define SMLCLASSNAME_LINEPATTERN			"LINEPATTERN"
#define SMLCLASSNAME_LINESTYLE				"LINESTYLE"
#define SMLCLASSNAME_LINETOOL				"LINETOOL"
#define SMLCLASSNAME_LMCOMPONANT			"LMCOMPONANT"	//! Spelled incorrectly
#define SMLCLASSNAME_LMCOMPONENT			SMLCLASSNAME_LMCOMPONANT
#define SMLCLASSNAME_MAPPROJ				"MAPPROJ"
#define SMLCLASSNAME_MAT3X3					"MAT3X3"
#define SMLCLASSNAME_MATRIX					"MATRIX"
#define SMLCLASSNAME_MDISPPOLYLINETOOL		"MDISPPOLYLINETOOL"
#define SMLCLASSNAME_MDISPREGIONTOOL		"MDISPREGIONTOOL"
#define SMLCLASSNAME_MDISPTOOL				"MDISPTOOL"
#define SMLCLASSNAME_MENUITEM				"MENUITEM"
#define SMLCLASSNAME_MIJOB					"MIJOB"
#define SMLCLASSNAME_MULTIROUTE				"MULTIROUTE"
#define SMLCLASSNAME_MOVIE					"MOVIE"
#define SMLCLASSNAME_NETWORK				"NETWORK"
#define SMLCLASSNAME_OBJECT					"OBJECT"
#define SMLCLASSNAME_OBJECTINFO				"OBJECTINFO"
#define SMLCLASSNAME_POINT2D				"POINT2D"
#define SMLCLASSNAME_POINT3D				"POINT3D"
#define SMLCLASSNAME_POINTSTYLE				"POINTSTYLE"
#define SMLCLASSNAME_POINTSYMBOL			"POINTSYMBOL"
#define SMLCLASSNAME_POINTTOOL				"POINTTOOL"
#define SMLCLASSNAME_POLYLINE				"POLYLINE"
#define SMLCLASSNAME_POLYSTYLE				"POLYSTYLE"
#define SMLCLASSNAME_PORT					"PORT"
#define SMLCLASSNAME_PORTCALLBACKLIST		"PORTCALLBACKLIST"
#define SMLCLASSNAME_PRINCOMPSTATS			"PRINCOMPSTATS"
#define SMLCLASSNAME_PRINTPARMS				"PRINTPARMS"
#define SMLCLASSNAME_PROMPT					"PROMPT"
#define SMLCLASSNAME_PROMPTNUM				"PROMPTNUM"
#define SMLCLASSNAME_PROMPTSTR				"PROMPTSTR"
#define SMLCLASSNAME_PUSHBUTTONITEM			"PUSHBUTTONITEM"
#define SMLCLASSNAME_RASTER					"RASTER"
#define SMLCLASSNAME_RASTERINFO				"RASTERINFO"
#define SMLCLASSNAME_RECT					"RECT"
#define SMLCLASSNAME_RECT3D					"RECT3D"
#define SMLCLASSNAME_REGION					"REGION"			// an RVC object
#define SMLCLASSNAME_REGION2D				"REGION2D"		// A region in memory
#define SMLCLASSNAME_REGIONDATA				"REGIONDATA"	// Deprecated (old name for REGION2D)
#define SMLCLASSNAME_REGIONTOOL				"REGIONTOOL"
#define SMLCLASSNAME_ROUTE					"ROUTE"
#define SMLCLASSNAME_RVC_CAD				"RVC_CAD"
#define SMLCLASSNAME_RVC_COLORMAP			"RVC_COLORMAP"
#define SMLCLASSNAME_RVC_CREDENTIALS		"RVC_CREDENTIALS"
#define SMLCLASSNAME_RVC_DESCRIPTOR			"RVC_DESCRIPTOR"
#define SMLCLASSNAME_RVC_DBASE				"RVC_DBASE"
#define SMLCLASSNAME_RVC_DBASE_ELEMENT		"RVC_DBASE_ELEMENT"
#define SMLCLASSNAME_RVC_DBASE_ELEMENT_ATTACHMENTITEM	"RVC_DBASE_ELEMENT_ATTACHMENTITEM"
#define SMLCLASSNAME_RVC_DBASE_ELEMTRANSLIST			"RVC_DBASE_ELEMTANSLIST"
#define SMLCLASSNAME_RVC_DBASE_CAD			"RVC_DBASE_CAD"
#define SMLCLASSNAME_RVC_DBASE_NODE			"RVC_DBASE_NODE"
#define SMLCLASSNAME_RVC_DBASE_POINT		"RVC_DBASE_POINT"
#define SMLCLASSNAME_RVC_DBASE_LINE			"RVC_DBASE_LINE"
#define SMLCLASSNAME_RVC_DBASE_POLYGON		"RVC_DBASE_POLYGON"
#define SMLCLASSNAME_RVC_DBASE_RASTER		"RVC_DBASE_RASTER"
#define SMLCLASSNAME_RVC_DBASE_SHAPE		"RVC_DBASE_SHAPE"
#define SMLCLASSNAME_RVC_DBASE_TINEDGE		"RVC_DBASE_TINEDGE"
#define SMLCLASSNAME_RVC_DBASE_TINHULL		"RVC_DBASE_TINHULL"
#define SMLCLASSNAME_RVC_DBASE_TINNODE		"RVC_DBASE_TINNODE"
#define SMLCLASSNAME_RVC_DBASE_TINTRIANGLE	"RVC_DBASE_TINTRIANGLE"
#define SMLCLASSNAME_RVC_DBTABLE			"RVC_DBTABLE"
#define SMLCLASSNAME_RVC_DBFIELDINFO_CONSTRAINT		"RVC_DBFIELDINFO_CONSTRAINT"
#define SMLCLASSNAME_RVC_DBFIELDINFO				"RVC_DBFIELDINFO"
#define SMLCLASSNAME_RVC_DBTABLE_CONSTRAINT			"RVC_DBTABLE_CONSTRAINT"
#define SMLCLASSNAME_RVC_DBTABLE_LINKPARMS			"RVC_DBTABLE_LINKPARMS"
#define SMLCLASSNAME_RVC_DBTABLE_LINKPARM_DBASE4	"RVC_DBTABLE_LINKPARM_DBASE4"
#define SMLCLASSNAME_RVC_DBTABLE_LINKPARM_RDBMS		"RVC_DBTABLE_LINKPARM_RDBMS"
#define SMLCLASSNAME_RVC_DBTABLE_MAKEPARMS	"RVC_DBTABLE_MAKEPARMS"
#define SMLCLASSNAME_RVC_DBTABLE_RECORD		"RVC_DBTABLE_RECORD"
#define SMLCLASSNAME_RVC_DBTABLE_STYLE		"RVC_DBTABLE_STYLE"
#define SMLCLASSNAME_RVC_ELEMENT			"RVC_ELEMENT"
#define SMLCLASSNAME_RVC_FOLDER				"RVC_FOLDER"
#define SMLCLASSNAME_RVC_GEOMETRIC			"RVC_GEOMETRIC"
#define SMLCLASSNAME_RVC_GEOREFERENCE		"RVC_GEOREFERENCE"
#define SMLCLASSNAME_RVC_GEOSPATIAL			"RVC_GEOSPATIAL"
#define SMLCLASSNAME_RVC_IMAGE				"RVC_IMAGE"
#define SMLCLASSNAME_RVC_NULLMASK			"RVC_NULLMASK"
#define SMLCLASSNAME_RVC_OBJECT				"RVC_OBJECT"
#define SMLCLASSNAME_RVC_OBJITEM			"RVC_OBJITEM"
#define SMLCLASSNAME_RVC_OP_CUTFILL			"RVC_OP_CUTFILL"
#define SMLCLASSNAME_RVC_RASTER				"RVC_RASTER"
#define SMLCLASSNAME_RVC_RECORDNUM			"RVC_RECORDNUM"
#define SMLCLASSNAME_RVC_SHAPE				"RVC_SHAPE"
#define SMLCLASSNAME_RVC_SIMPLE				"RVC_SIMPLE"
#define SMLCLASSNAME_RVC_STYLE				"RVC_STYLE"
#define SMLCLASSNAME_RVC_TEXT				"RVC_TEXT"
#define SMLCLASSNAME_RVC_TIN				"RVC_TIN"
#define SMLCLASSNAME_RVC_VECTOR				"RVC_VECTOR"
#define SMLCLASSNAME_SHAPE			  		"SHAPE"
#define SMLCLASSNAME_SR_ELLIPSOID  			"SR_ELLIPSOID"
#define SMLCLASSNAME_SR_PRIMEMERIDIAN		"SR_PRIMEMERIDIAN"
#define SMLCLASSNAME_SR_DATUM   			"SR_DATUM"
#define SMLCLASSNAME_SR_COORDAXIS  			"SR_COORDAXIS"
#define SMLCLASSNAME_SR_COORDSYS  			"SR_COORDSYS"
#define SMLCLASSNAME_SR_COORDOPPARM 		"SR_COORDOPPARM"
#define SMLCLASSNAME_SR_COORDOPMETHOD 		"SR_COORDOPMETHOD"
#define SMLCLASSNAME_SR_COORDOPDEF  		"SR_COORDOPDEF"
#define SMLCLASSNAME_SR_COORDREFSYS 		"SR_COORDREFSYS"
#define SMLCLASSNAME_STATUSCONTEXT			"STATUSCONTEXT"
#define SMLCLASSNAME_STATUSHANDLE			"STATUSHANDLE"
#define SMLCLASSNAME_STATUSDIALOG			"STATUSDIALOG"
#define SMLCLASSNAME_STEREOPARM				"STEREOPARM"
#define SMLCLASSNAME_STOP					"STOP"
#define SMLCLASSNAME_STRING					"STRING"
#define SMLCLASSNAME_STRINGLIST				"STRINGLIST"
#define SMLCLASSNAME_STYLEOBJECT			"STYLEOBJECT"
#define SMLCLASSNAME_TEXTSTYLE				"TEXTSTYLE"
#define SMLCLASSNAME_TIMER					"TIMER"
#define SMLCLASSNAME_TIMEINTERVAL			"TIMEINTERVAL"
#define SMLCLASSNAME_TIN					"TIN"
#define SMLCLASSNAME_TOGGLEBUTTONITEM		"TOGGLEBUTTONITEM"
#define SMLCLASSNAME_TOOL					"TOOL"
#define SMLCLASSNAME_TOOLCALLBACKLIST		"TOOLCALLBACKLIST"
#define SMLCLASSNAME_TOOLTIP				"TOOLTIP"
#define SMLCLASSNAME_TRANS2D_AFFINE			"TRANS2D_AFFINE"
#define SMLCLASSNAME_TRANS2D_MAPGEN			"TRANS2D_MAPGEN"
#define SMLCLASSNAME_TRANSAFFINE			"TRANSAFFINE"
#define SMLCLASSNAME_TRANSMODEL				"TRANSMODEL"
#define SMLCLASSNAME_TRANSPARM				"TRANSPARM"
#define SMLCLASSNAME_TRIANGULATOR			"TRIANGULATOR"
#define SMLCLASSNAME_UNIT					"UNIT"
#define SMLCLASSNAME_VECTOR					"VECTOR"
#define SMLCLASSNAME_VECTORINFO				"VECTORINFO"
#define SMLCLASSNAME_VIEWPOINT3D			"VIEWPOINT3D"
#define SMLCLASSNAME_WATERSHED				"WATERSHED"
#define SMLCLASSNAME_WATERSHEDPOLYGON		"WATERSHEDPOLYGON"
#define SMLCLASSNAME_WATERSHEDPOUR			"WATERSHEDPOUR"
#define SMLCLASSNAME_XMLNODE					"XMLNODE"
#define SMLCLASSNAME_ZIPFILE					"ZIPFILE"

//! The following class names are left as mixed case because of historical usage.
#define SMLCLASSNAME_Widget					"Widget"
#define SMLCLASSNAME_XmBulletinBoard		"XmBulletinBoard"
#define SMLCLASSNAME_XmCallbackList			"XmCallbackList"
#define SMLCLASSNAME_XmDrawingArea			"XmDrawingArea"
#define SMLCLASSNAME_XmForm					"XmForm"
#define SMLCLASSNAME_XmFrame				"XmFrame"
#define SMLCLASSNAME_XmHTML					"XmHTML"
#define SMLCLASSNAME_XmLabel				"XmLabel"
#define SMLCLASSNAME_XmList					"XmList"
#define SMLCLASSNAME_XmManager				"XmManager"
#define SMLCLASSNAME_XmOptionMenu			"XmOptionMenu"
#define SMLCLASSNAME_XmPrimitive			"XmPrimitive"
#define SMLCLASSNAME_XmPushButton			"XmPushButton"
#define SMLCLASSNAME_XmRowColumn			"XmRowColumn"
#define SMLCLASSNAME_XmScale				"XmScale"
#define SMLCLASSNAME_XmScrollBar			"XmScrollBar"
#define SMLCLASSNAME_XmScrolledWindow		"XmScrolledWindow"
#define SMLCLASSNAME_XmSeparator			"XmSeparator"
#define SMLCLASSNAME_XmToggleButton			"XmToggleButton"


#endif		//  INC_SML_CLASSNAMES_H

