/******************************************************************************
 *
 * \file sml/parm.h
 * \brief SML PARM
 *
 * \if NODOC
 * $Log: parm.h_v $
 * Revision 1.2  2007/05/08 15:46:11  dwilliss
 * SML namespace
 *
 * Revision 1.1  2007/05/04 22:46:55  dwilliss
 * Initial revision
 *
 * \endif
*/
/*****************************************************************************/

#ifndef INC_SML_PARM_H
#define INC_SML_PARM_H

#ifndef	INC_SML_SMLSTYPE_H
	#include <sml/stype.h>
#endif

#ifndef	INC_MI32_STDDEFNS_H
	#include <mi32/stddefns.h>
#endif


namespace SML {

class FLAG;

/*****************************************************************************/
/*
** This class is used to define the parameters to an SML::FUNC.  See SML::FUNC for
** an example.
**
** This structure is designed to be statically defined, and as such strings MUST
** be (char*) (at least until all compilers have a way to define Unicode string
** constants ).
**
** @see SML::FUNC
*/
class PARM {
	public:
		const char *name;			//!< Parameter name
		STYPE type;				//!< STYPE_Xxxxx : See <mi32/sml.h>
		UINT32 flags;				//!< PARM_Optional, et.al.
		const char *desc;			//!< default description of parm - only used to create smlfuncs.txt  (And not really used for that anymore either)
		FLAG *flag;				//!< (SML::FLAG*) array
		const char *ClassName;	//!< Name of parameter class (if STYPE_Class)
	};

} // end of SML namespace

/* Flags in SMLSYMBOL.flags (only a short) */

#define PARM_Optional		0x0001
#define PARM_ByRef			0x0002
#define PARM_UndefOK			0x0004
#define PARM_TypeConfirmed	0x0008	//!<  Variable actually seen used as this type
#define PARM_NeedNextParm	0x0010
#define PARM_Optional2		(PARM_Optional|PARM_NeedNextParm)

typedef SML::PARM SMLPARM;

#endif		//  INC_SML_PARM_H


