/**
 * \file spatmod/imagefiltervaliditynear.h
 * \brief SPATMOD IMAGE::FILTER_VALIDITYNEAR definitions
 *
 * \if NODOC
 * $Id: imagefiltervaliditynear.h_v 1.1 2007/02/28 20:39:24 mju Exp $
 *
 * $Log: imagefiltervaliditynear.h_v $
 * Revision 1.1  2007/02/28 20:39:24  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERVALIDITYNEAR_H
#define	INC_SPATMOD_IMAGEFILTERVALIDITYNEAR_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

#ifndef	INC_MI32_MILIST_H
#include <mi32/milist.h>
#endif

#ifndef	INC_MI32_DOUBLEAR_H
#include <mi32/doublear.h>
#endif

struct COLOR;

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Filter to determine validity based on nearness to specified pixel values.
//! At present, all samples selected for testing must have the same DATATYPE.
class FILTER_VALIDITYNEAR : public FILTER {
	public:

		//! Measurement method for distance.
		enum MEASURE {
			MEASURE_Euclidean,
			MEASURE_Orthogonal
			};

		//! Constructor.
		FILTER_VALIDITYNEAR (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership,						//!< Filter takes ownership of previous stage and will delete when needed
			MEASURE Measure,							//!< Measurement method for nearness
			bool ResultValid,							//!< True if pixels "near" test values are valid, false if not
			bool TestOpacity = false				//!< True to test opacity samples, false to ignore
			);

		virtual ~FILTER_VALIDITYNEAR ();

		//! Add value to set to test.
		//! Must be done before Initialize().
		ERRVALUE AddValue (
			const PIXEL& value,						//!< Value to add, type must match image
			double distance							//!< Distance
			);

		//! Add value as COLOR to test.
		//! Image pixel type must be appropriate (eg, RGB or RGBA).
		ERRVALUE AddValue (
			const COLOR& color,						//!< Color to add
			double distance,							//!< Distance
			bool use8bit = true						//!< Use 8-bit RGB values regardless of input pixel datatype
			);

		//! Set which samples to test.
		//! If used, must be done before Initialize.
		ERRVALUE SetSamplesTest (
			const UINT16 *SamplesTest,				//!< Array of sample indices to test
			UINT16 NumSamplesTest					//!< Number of sample indices to test
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MEASURE m_Measure;
		bool m_ResultValid;
		SIMPLE_ARRAY<UINT16> m_SamplesTest;
		MILIST<PIXEL> m_PixelList;
		DOUBLE_ARRAY<double> m_DistanceArray;
		DATATYPE m_DataTypeTest;

		void ProcessUINT8 (BUFFER& buffer, INT32 NumRows, INT32 NumColumns) const;
		void ProcessSINT8 (BUFFER& buffer, INT32 NumRows, INT32 NumColumns) const;
		void ProcessUINT16 (BUFFER& buffer, INT32 NumRows, INT32 NumColumns) const;
		void ProcessSINT16 (BUFFER& buffer, INT32 NumRows, INT32 NumColumns) const;
		void ProcessUINT32 (BUFFER& buffer, INT32 NumRows, INT32 NumColumns) const;
		void ProcessSINT32 (BUFFER& buffer, INT32 NumRows, INT32 NumColumns) const;
		void ProcessFloat (BUFFER& buffer, INT32 NumRows, INT32 NumColumns) const;
		void ProcessDouble (BUFFER& buffer, INT32 NumRows, INT32 NumColumns) const;

		// STAGE overrides.
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);
		virtual ERRVALUE v_Initialize ();

		// Unimplemented
		FILTER_VALIDITYNEAR (const FILTER_VALIDITYNEAR&);
		FILTER_VALIDITYNEAR& operator= (const FILTER_VALIDITYNEAR&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERVALIDITYNEAR_H
