/**
 * \file imagesource.h <spatmod/imagesource.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::SOURCE definition.
 *
 * \if NODOC
 * $Id: imagesource.h_v 1.5 2011/03/31 21:36:13 mju Exp $
 *
 * $Log: imagesource.h_v $
 * Revision 1.5  2011/03/31 21:36:13  mju
 * Add v_logCreate
 *
 * Revision 1.4  2011/03/31 15:26:15  mju
 * Add v_getTypeName.
 *
 * Revision 1.3  2007/01/03 23:02:57  mju
 * Add getMetadata.
 *
 * Revision 1.2  2005/08/19 14:26:29  mju
 * Remove setTotalSection.
 *
 * Revision 1.1  2005/08/16 15:10:54  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGESOURCE_H
#define	INC_SPATMOD_IMAGESOURCE_H

#ifndef	INC_SPATMOD_IMAGESTAGE_H
#include <spatmod/imagestage.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Base for image source in pipeline.
class SOURCE : public STAGE {
	public:

		// Default constructor.
		SOURCE ();

		// Destructor.
		virtual ~SOURCE ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		void v_GetMetadata (MISTRING& result, METADATACONTENT content, METADATAFORMAT format) const;
		virtual const char* v_GetTypeName () const;
		virtual ERRVALUE v_LogCreate (const char *label, bool OnlyEnds, MILOG::MODE mode);

		// Unimplemented
		SOURCE (const SOURCE&);
		SOURCE& operator= (const SOURCE&);
		#endif // GENERATING_DOXYGEN_OUTPUT

		// STAGE overrides.

		//! Finish pulling subsections.
		//! Default implementation does nothing.
		virtual ERRVALUE v_PullEnd (
			);

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGESOURCE_H

