/**
 * \file spatmod/imagesourcejpeg.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::SOURCE_JPEG definition.
 *
 * \if NODOC
 * $Id: imagesourcejpeg.h_v 1.8 2011/10/11 14:42:37 mju Exp $
 *
 * $Log: imagesourcejpeg.h_v $
 * Revision 1.8  2011/10/11 14:42:37  mju
 * Remove unneeded hdrs.
 *
 * Revision 1.7  2010/02/19 14:22:29  mju
 * Remove obsolete v_getTotalColumns/Rows.
 *
 * Revision 1.6  2007/03/29 13:21:37  mju
 * Add v_getDimensions.
 *
 * Revision 1.5  2007/02/23 14:10:04  mju
 * Derive from source_file.
 *
 * Revision 1.4  2007/01/03 23:03:01  mju
 * Add getMetadata.
 *
 * Revision 1.3  2006/11/08 15:59:26  mju
 * Support zooming.
 *
 * Revision 1.2  2006/11/06 14:45:07  mju
 * Add readBuffer member.
 *
 * Revision 1.1  2006/11/03 22:07:06  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGESOURCEJPEG_H
#define	INC_SPATMOD_IMAGESOURCEJPEG_H

#ifndef	INC_SPATMOD_IMAGESOURCEFILE_H
#include <spatmod/imagesourcefile.h>
#endif

#ifndef	GENERATING_DOXYGEN_OUTPUT
class JPEGEXT;
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Pipeline source for reading JPEG images from memory or file.
class SOURCE_JPEG : public SOURCE_FILE {
	public:

		//! Construct with FILEPATH.
		explicit SOURCE_JPEG (
			const FILEPATH& FilePath
			);

		//! Construct with memory buffer (SIMPLE_ARRAY).
		//! Will take ownership of buffer.
		explicit SOURCE_JPEG (
			SIMPLE_ARRAY<UINT8>& MemBuffer	//!< Memory buffer, will be returned empty
			);

		//! Construct with memory buffer (pointer/size).
		//! Will take ownership of buffer, pointer will be returned as NULL
		SOURCE_JPEG (
			UINT8*& MemBuffer,					//!< Memory buffer, will be returned as NULL
			UINT32 BufferSize						//!< Buffer size
			);

		//! Destructor.
		virtual ~SOURCE_JPEG ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		JPEGEXT *m_pReader;
		bool m_DoingPull;
		SIMPLE_ARRAY<UINT8> m_ReadBuffer;

		// BASE overrides.
		virtual ERRVALUE v_GetDimensions (DIMENSIONS& dimensions, double zoom) const;
		virtual ZOOMABILITY v_GetZoomAbility (SIMPLE_ARRAY<double>& ZoomsAllowed) const;
		virtual float v_QueryIterator (const SECTION_ITERATOR& iterator) const;

		// STAGE overrides.
		virtual void v_GetMetadataThisOnly (MISTRING& result, METADATACONTENT content, METADATAFORMAT format) const;
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullEnd ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		SOURCE_JPEG (const SOURCE_JPEG&);
		SOURCE_JPEG& operator= (const SOURCE_JPEG&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGESOURCEJPEG_H

